/* Generated by Cython 0.29.36 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/home/machri/.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/_core/include/numpy/arrayobject.h",
            "/home/machri/.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/_core/include/numpy/arrayscalars.h",
            "/home/machri/.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/_core/include/numpy/ndarrayobject.h",
            "/home/machri/.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/_core/include/numpy/ndarraytypes.h",
            "/home/machri/.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/_core/include/numpy/ufuncobject.h"
        ],
        "include_dirs": [
            "/home/machri/.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/_core/include"
        ],
        "name": "agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy",
        "sources": [
            "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx"
        ]
    },
    "module_name": "agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_36"
#define CYTHON_HEX_VERSION 0x001D24F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1 && PYPY_VERSION_NUM >= 0x07030C00)
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PY_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS (PY_VERSION_HEX < 0x030C00A5)
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030A0000)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS ((PY_VERSION_HEX >= 0x030600B1) && (PY_VERSION_HEX < 0x030C00A5))
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__agox__models__descriptors__fingerprint_cython__angular_fingerprintFeature_cy
#define __PYX_HAVE_API__agox__models__descriptors__fingerprint_cython__angular_fingerprintFeature_cy
/* Early includes */
#include <string.h>
#include <stdio.h>

    /* Using NumPy API declarations from "numpy/__init__.pxd" */
    
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"
#include <math.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx",
  "__init__.pxd",
  "type.pxd",
  "cymem.pxd",
};

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":658
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":659
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":660
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * 
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":661
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_uint8      uint8_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":663
 * ctypedef npy_int64      int64_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":664
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":665
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * 
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":666
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_float32    float32_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":668
 * ctypedef npy_uint64     uint64_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":669
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":676
 * ctypedef double complex complex128_t
 * 
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":677
 * 
 * ctypedef npy_longlong   longlong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":679
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":680
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":682
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":683
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":684
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef float complex       cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< long double > __pyx_t_long_double_complex;
  #else
    typedef long double _Complex __pyx_t_long_double_complex;
  #endif
#else
    typedef struct { long double real, imag; } __pyx_t_long_double_complex;
#endif
static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double, long double);


/*--- Type declarations ---*/
struct __pyx_obj_5cymem_5cymem_PyMalloc;
struct __pyx_obj_5cymem_5cymem_PyFree;
struct __pyx_obj_5cymem_5cymem_Pool;
struct __pyx_obj_5cymem_5cymem_Address;

/* "cymem/cymem.pxd":1
 * ctypedef void* (*malloc_t)(size_t n)             # <<<<<<<<<<<<<<
 * ctypedef void (*free_t)(void *p)
 * 
 */
typedef void *(*__pyx_t_5cymem_5cymem_malloc_t)(size_t);

/* "cymem/cymem.pxd":2
 * ctypedef void* (*malloc_t)(size_t n)
 * ctypedef void (*free_t)(void *p)             # <<<<<<<<<<<<<<
 * 
 * cdef class PyMalloc:
 */
typedef void (*__pyx_t_5cymem_5cymem_free_t)(void *);
struct __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point;
typedef struct __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point;

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":23
 * 
 * # Custom functions
 * ctypedef struct Point:             # <<<<<<<<<<<<<<
 *     double coord[3]
 * 
 */
struct __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point {
  double coord[3];
};

/* "cymem/cymem.pxd":4
 * ctypedef void (*free_t)(void *p)
 * 
 * cdef class PyMalloc:             # <<<<<<<<<<<<<<
 *     cdef malloc_t malloc
 *     cdef void _set(self, malloc_t malloc)
 */
struct __pyx_obj_5cymem_5cymem_PyMalloc {
  PyObject_HEAD
  struct __pyx_vtabstruct_5cymem_5cymem_PyMalloc *__pyx_vtab;
  __pyx_t_5cymem_5cymem_malloc_t malloc;
};


/* "cymem/cymem.pxd":10
 * cdef PyMalloc WrapMalloc(malloc_t malloc)
 * 
 * cdef class PyFree:             # <<<<<<<<<<<<<<
 *     cdef free_t free
 *     cdef void _set(self, free_t free)
 */
struct __pyx_obj_5cymem_5cymem_PyFree {
  PyObject_HEAD
  struct __pyx_vtabstruct_5cymem_5cymem_PyFree *__pyx_vtab;
  __pyx_t_5cymem_5cymem_free_t free;
};


/* "cymem/cymem.pxd":16
 * cdef PyFree WrapFree(free_t free)
 * 
 * cdef class Pool:             # <<<<<<<<<<<<<<
 *     cdef readonly size_t size
 *     cdef readonly dict addresses
 */
struct __pyx_obj_5cymem_5cymem_Pool {
  PyObject_HEAD
  struct __pyx_vtabstruct_5cymem_5cymem_Pool *__pyx_vtab;
  size_t size;
  PyObject *addresses;
  PyObject *refs;
  struct __pyx_obj_5cymem_5cymem_PyMalloc *pymalloc;
  struct __pyx_obj_5cymem_5cymem_PyFree *pyfree;
};


/* "cymem/cymem.pxd":28
 * 
 * 
 * cdef class Address:             # <<<<<<<<<<<<<<
 *     cdef void* ptr
 *     cdef readonly PyMalloc pymalloc
 */
struct __pyx_obj_5cymem_5cymem_Address {
  PyObject_HEAD
  void *ptr;
  struct __pyx_obj_5cymem_5cymem_PyMalloc *pymalloc;
  struct __pyx_obj_5cymem_5cymem_PyFree *pyfree;
};



/* "cymem/cymem.pxd":4
 * ctypedef void (*free_t)(void *p)
 * 
 * cdef class PyMalloc:             # <<<<<<<<<<<<<<
 *     cdef malloc_t malloc
 *     cdef void _set(self, malloc_t malloc)
 */

struct __pyx_vtabstruct_5cymem_5cymem_PyMalloc {
  void (*_set)(struct __pyx_obj_5cymem_5cymem_PyMalloc *, __pyx_t_5cymem_5cymem_malloc_t);
};
static struct __pyx_vtabstruct_5cymem_5cymem_PyMalloc *__pyx_vtabptr_5cymem_5cymem_PyMalloc;


/* "cymem/cymem.pxd":10
 * cdef PyMalloc WrapMalloc(malloc_t malloc)
 * 
 * cdef class PyFree:             # <<<<<<<<<<<<<<
 *     cdef free_t free
 *     cdef void _set(self, free_t free)
 */

struct __pyx_vtabstruct_5cymem_5cymem_PyFree {
  void (*_set)(struct __pyx_obj_5cymem_5cymem_PyFree *, __pyx_t_5cymem_5cymem_free_t);
};
static struct __pyx_vtabstruct_5cymem_5cymem_PyFree *__pyx_vtabptr_5cymem_5cymem_PyFree;


/* "cymem/cymem.pxd":16
 * cdef PyFree WrapFree(free_t free)
 * 
 * cdef class Pool:             # <<<<<<<<<<<<<<
 *     cdef readonly size_t size
 *     cdef readonly dict addresses
 */

struct __pyx_vtabstruct_5cymem_5cymem_Pool {
  void *(*alloc)(struct __pyx_obj_5cymem_5cymem_Pool *, size_t, size_t);
  void (*free)(struct __pyx_obj_5cymem_5cymem_Pool *, void *);
  void *(*realloc)(struct __pyx_obj_5cymem_5cymem_Pool *, void *, size_t);
};
static struct __pyx_vtabstruct_5cymem_5cymem_Pool *__pyx_vtabptr_5cymem_5cymem_Pool;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* SliceObject.proto */
#define __Pyx_PyObject_DelSlice(obj, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)\
    __Pyx_PyObject_SetSlice(obj, (PyObject*)NULL, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(
        PyObject* obj, PyObject* value, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* py_abs.proto */
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *num);
#define __Pyx_PyNumber_Absolute(x)\
    ((likely(PyLong_CheckExact(x))) ?\
         (likely(Py_SIZE(x) >= 0) ? (Py_INCREF(x), (x)) : __Pyx_PyLong_AbsNeg(x)) :\
         PyNumber_Absolute(x))
#else
#define __Pyx_PyNumber_Absolute(x)  PyNumber_Absolute(x)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_0_29_36
#define __PYX_HAVE_RT_ImportType_proto_0_29_36
#if __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if __STDC_VERSION__ >= 201112L || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_0_29_36(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_0_29_36(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_0_29_36 {
   __Pyx_ImportType_CheckSize_Error_0_29_36 = 0,
   __Pyx_ImportType_CheckSize_Warn_0_29_36 = 1,
   __Pyx_ImportType_CheckSize_Ignore_0_29_36 = 2
};
static PyTypeObject *__Pyx_ImportType_0_29_36(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_0_29_36 check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_long__double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_long__double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_long__double(a, b) ((a)-(b))
    #define __Pyx_c_prod_long__double(a, b) ((a)*(b))
    #define __Pyx_c_quot_long__double(a, b) ((a)/(b))
    #define __Pyx_c_neg_long__double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_long__double(z) ((z)==(long double)0)
    #define __Pyx_c_conj_long__double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_long__double(z)     (::std::abs(z))
        #define __Pyx_c_pow_long__double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_long__double(z) ((z)==0)
    #define __Pyx_c_conj_long__double(z)    (conjl(z))
    #if 1
        #define __Pyx_c_abs_long__double(z)     (cabsl(z))
        #define __Pyx_c_pow_long__double(a, b)  (cpowl(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_sum_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_diff_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_prod_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_quot_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_neg_long__double(__pyx_t_long_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_long__double(__pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_conj_long__double(__pyx_t_long_double_complex);
    #if 1
        static CYTHON_INLINE long double __Pyx_c_abs_long__double(__pyx_t_long_double_complex);
        static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_pow_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    #endif
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_generic = 0;
static PyTypeObject *__pyx_ptype_5numpy_number = 0;
static PyTypeObject *__pyx_ptype_5numpy_integer = 0;
static PyTypeObject *__pyx_ptype_5numpy_signedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_unsignedinteger = 0;
static PyTypeObject *__pyx_ptype_5numpy_inexact = 0;
static PyTypeObject *__pyx_ptype_5numpy_floating = 0;
static PyTypeObject *__pyx_ptype_5numpy_complexfloating = 0;
static PyTypeObject *__pyx_ptype_5numpy_flexible = 0;
static PyTypeObject *__pyx_ptype_5numpy_character = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;

/* Module declarations from 'libc.math' */

/* Module declarations from 'cymem.cymem' */
static PyTypeObject *__pyx_ptype_5cymem_5cymem_PyMalloc = 0;
static PyTypeObject *__pyx_ptype_5cymem_5cymem_PyFree = 0;
static PyTypeObject *__pyx_ptype_5cymem_5cymem_Pool = 0;
static PyTypeObject *__pyx_ptype_5cymem_5cymem_Address = 0;

/* Module declarations from 'cython' */

/* Module declarations from 'agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy' */
static __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_subtract(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point, __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point); /*proto*/
static __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_add(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point, __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point); /*proto*/
static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_norm(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point); /*proto*/
static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_euclidean(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point, __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point); /*proto*/
static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_dot(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point, __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point); /*proto*/
static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_get_angle(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point, __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point); /*proto*/
static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_f_cutoff(double, double, double); /*proto*/
static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_f_cutoff_grad(double, double, double); /*proto*/
#define __Pyx_MODULE_NAME "agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy"
extern int __pyx_module_is_main_agox__models__descriptors__fingerprint_cython__angular_fingerprintFeature_cy;
int __pyx_module_is_main_agox__models__descriptors__fingerprint_cython__angular_fingerprintFeature_cy = 0;

/* Implementation of 'agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy' */
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_sum;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_a[] = "a";
static const char __pyx_k_c[] = "c";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_j[] = "j";
static const char __pyx_k_k[] = "k";
static const char __pyx_k_m[] = "m";
static const char __pyx_k_n[] = "n";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_z[] = "z";
static const char __pyx_k_k1[] = "k1";
static const char __pyx_k_k2[] = "k2";
static const char __pyx_k_k3[] = "k3";
static const char __pyx_k_m1[] = "m1";
static const char __pyx_k_m2[] = "m2";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_os[] = "os";
static const char __pyx_k_pi[] = "pi";
static const char __pyx_k_t1[] = "t1";
static const char __pyx_k_t2[] = "t2";
static const char __pyx_k_Rc1[] = "Rc1";
static const char __pyx_k_Rc2[] = "Rc2";
static const char __pyx_k_Rij[] = "Rij";
static const char __pyx_k_Rik[] = "Rik";
static const char __pyx_k_cwd[] = "cwd";
static const char __pyx_k_dim[] = "dim";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_dot[] = "dot";
static const char __pyx_k_erf[] = "erf";
static const char __pyx_k_eta[] = "eta";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_mem[] = "mem";
static const char __pyx_k_num[] = "num";
static const char __pyx_k_pbc[] = "pbc";
static const char __pyx_k_pos[] = "pos";
static const char __pyx_k_sum[] = "sum";
static const char __pyx_k_sys[] = "sys";
static const char __pyx_k_tt1[] = "tt1";
static const char __pyx_k_tt2[] = "tt2";
static const char __pyx_k_xyz[] = "xyz";
static const char __pyx_k_argv[] = "argv";
static const char __pyx_k_axis[] = "axis";
static const char __pyx_k_ceil[] = "ceil";
static const char __pyx_k_cell[] = "cell";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_math[] = "math";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_norm[] = "norm";
static const char __pyx_k_ones[] = "ones";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_sqrt[] = "sqrt";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_angle[] = "angle";
static const char __pyx_k_array[] = "array";
static const char __pyx_k_atoms[] = "atoms";
static const char __pyx_k_cdist[] = "cdist";
static const char __pyx_k_count[] = "count";
static const char __pyx_k_fc_ij[] = "fc_ij";
static const char __pyx_k_fc_ik[] = "fc_ik";
static const char __pyx_k_gamma[] = "gamma";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_pos_i[] = "pos_i";
static const char __pyx_k_pos_j[] = "pos_j";
static const char __pyx_k_pos_k[] = "pos_k";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_type1[] = "type1";
static const char __pyx_k_type2[] = "type2";
static const char __pyx_k_type3[] = "type3";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_zeros[] = "zeros";
static const char __pyx_k_Natoms[] = "Natoms";
static const char __pyx_k_Nbins1[] = "Nbins1";
static const char __pyx_k_Nbins2[] = "Nbins2";
static const char __pyx_k_Ncells[] = "Ncells";
static const char __pyx_k_Ntypes[] = "Ntypes";
static const char __pyx_k_Rc_max[] = "Rc_max";
static const char __pyx_k_RijVec[] = "RijVec";
static const char __pyx_k_RikVec[] = "RikVec";
static const char __pyx_k_arg_up[] = "arg_up";
static const char __pyx_k_astype[] = "astype";
static const char __pyx_k_binpos[] = "binpos";
static const char __pyx_k_getcwd[] = "getcwd";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_linalg[] = "linalg";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_newbin[] = "newbin";
static const char __pyx_k_nsigma[] = "nsigma";
static const char __pyx_k_object[] = "object";
static const char __pyx_k_pos_np[] = "pos_np";
static const char __pyx_k_sigma1[] = "sigma1";
static const char __pyx_k_sigma2[] = "sigma2";
static const char __pyx_k_tolist[] = "tolist";
static const char __pyx_k_value1[] = "value1";
static const char __pyx_k_value2[] = "value2";
static const char __pyx_k_volume[] = "volume";
static const char __pyx_k_arg_low[] = "arg_low";
static const char __pyx_k_cond_ij[] = "cond_ij";
static const char __pyx_k_cond_ik[] = "cond_ik";
static const char __pyx_k_get_pbc[] = "get_pbc";
static const char __pyx_k_k_start[] = "k_start";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_product[] = "product";
static const char __pyx_k_reshape[] = "reshape";
static const char __pyx_k_feature1[] = "feature1";
static const char __pyx_k_feature2[] = "feature2";
static const char __pyx_k_get_cell[] = "get_cell";
static const char __pyx_k_ncellmax[] = "ncellmax";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_subarray[] = "subarray";
static const char __pyx_k_Nelements[] = "Nelements";
static const char __pyx_k_bin_index[] = "bin_index";
static const char __pyx_k_binwidth1[] = "binwidth1";
static const char __pyx_k_binwidth2[] = "binwidth2";
static const char __pyx_k_cos_angle[] = "cos_angle";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_erfarg_up[] = "erfarg_up";
static const char __pyx_k_itertools[] = "itertools";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_num_pairs[] = "num_pairs";
static const char __pyx_k_atoms_list[] = "atoms_list";
static const char __pyx_k_cell_index[] = "cell_index";
static const char __pyx_k_center_bin[] = "center_bin";
static const char __pyx_k_erfarg_low[] = "erfarg_low";
static const char __pyx_k_fc_grad_ij[] = "fc_grad_ij";
static const char __pyx_k_fc_grad_ik[] = "fc_grad_ik";
static const char __pyx_k_featureMat[] = "featureMat";
static const char __pyx_k_feature_np[] = "feature_np";
static const char __pyx_k_get_volume[] = "get_volume";
static const char __pyx_k_grad_index[] = "grad_index";
static const char __pyx_k_maxbin_lim[] = "maxbin_lim";
static const char __pyx_k_minbin_lim[] = "minbin_lim";
static const char __pyx_k_neighbours[] = "neighbours";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_cell_index1[] = "cell_index1";
static const char __pyx_k_cell_index2[] = "cell_index2";
static const char __pyx_k_feature1_np[] = "feature1_np";
static const char __pyx_k_feature2_np[] = "feature2_np";
static const char __pyx_k_get_feature[] = "get_feature";
static const char __pyx_k_use_angular[] = "use_angular";
static const char __pyx_k_angle_grad_i[] = "angle_grad_i";
static const char __pyx_k_angle_grad_j[] = "angle_grad_j";
static const char __pyx_k_angle_grad_k[] = "angle_grad_k";
static const char __pyx_k_atomic_count[] = "atomic_count";
static const char __pyx_k_atomic_types[] = "atomic_types";
static const char __pyx_k_displacement[] = "displacement";
static const char __pyx_k_displacement1[] = "displacement1";
static const char __pyx_k_displacement2[] = "displacement2";
static const char __pyx_k_feature_grad1[] = "feature_grad1";
static const char __pyx_k_feature_grad2[] = "feature_grad2";
static const char __pyx_k_feature_grads[] = "feature_grads";
static const char __pyx_k_get_positions[] = "get_positions";
static const char __pyx_k_normalization[] = "normalization";
static const char __pyx_k_num_converted[] = "num_converted";
static const char __pyx_k_bondtype_index[] = "bondtype_index";
static const char __pyx_k_cell_vec_norms[] = "cell_vec_norms";
static const char __pyx_k_get_featureMat[] = "get_featureMat";
static const char __pyx_k_smearing_norm1[] = "smearing_norm1";
static const char __pyx_k_smearing_norm2[] = "smearing_norm2";
static const char __pyx_k_type_converter[] = "type_converter";
static const char __pyx_k_Nelements_2body[] = "Nelements_2body";
static const char __pyx_k_Nelements_3body[] = "Nelements_3body";
static const char __pyx_k_bondtypes_2body[] = "bondtypes_2body";
static const char __pyx_k_bondtypes_3body[] = "bondtypes_3body";
static const char __pyx_k_feature_grad_np[] = "feature_grad_np";
static const char __pyx_k_Nbondtypes_2body[] = "Nbondtypes_2body";
static const char __pyx_k_Nbondtypes_3body[] = "Nbondtypes_3body";
static const char __pyx_k_feature_grad1_np[] = "feature_grad1_np";
static const char __pyx_k_feature_grad2_np[] = "feature_grad2_np";
static const char __pyx_k_num_converted_old[] = "num_converted_old";
static const char __pyx_k_cell_displacements[] = "cell_displacements";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_convert_atom_types[] = "convert_atom_types";
static const char __pyx_k_get_atomic_numbers[] = "get_atomic_numbers";
static const char __pyx_k_Angular_Fingerprint[] = "Angular_Fingerprint";
static const char __pyx_k_bondtypes_2body_old[] = "bondtypes_2body_old";
static const char __pyx_k_bondtypes_3body_old[] = "bondtypes_3body_old";
static const char __pyx_k_get_featureGradient[] = "get_featureGradient";
static const char __pyx_k_neighbourcells_disp[] = "neighbourcells_disp";
static const char __pyx_k_bondtypes_3body_keys[] = "bondtypes_3body_keys";
static const char __pyx_k_cell_displacements_old[] = "cell_displacements_old";
static const char __pyx_k_scipy_spatial_distance[] = "scipy.spatial.distance";
static const char __pyx_k_get_all_featureGradients[] = "get_all_featureGradients";
static const char __pyx_k_Angular_Fingerprint___init[] = "Angular_Fingerprint.__init__";
static const char __pyx_k_comparator_for_construction_of[] = " comparator for construction of angular fingerprints\n    ";
static const char __pyx_k_get_neighbour_cells_displaceme[] = "__get_neighbour_cells_displacement";
static const char __pyx_k_Angular_Fingerprint__get_neighb[] = "_Angular_Fingerprint__get_neighbour_cells_displacement";
static const char __pyx_k_Angular_Fingerprint_get_feature[] = "Angular_Fingerprint.get_feature";
static const char __pyx_k_Angular_Fingerprint___get_neighb[] = "Angular_Fingerprint.__get_neighbour_cells_displacement";
static const char __pyx_k_Angular_Fingerprint_get_all_feat[] = "Angular_Fingerprint.get_all_featureGradients";
static const char __pyx_k_Angular_Fingerprint_get_featureG[] = "Angular_Fingerprint.get_featureGradient";
static const char __pyx_k_Angular_Fingerprint_get_featureM[] = "Angular_Fingerprint.get_featureMat";
static const char __pyx_k_agox_models_descriptors_fingerpr[] = "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx";
static const char __pyx_k_numpy__core_multiarray_failed_to[] = "numpy._core.multiarray failed to import";
static const char __pyx_k_numpy__core_umath_failed_to_impo[] = "numpy._core.umath failed to import";
static const char __pyx_k_agox_models_descriptors_fingerpr_2[] = "agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy";
static PyObject *__pyx_n_s_Angular_Fingerprint;
static PyObject *__pyx_n_s_Angular_Fingerprint___get_neighb;
static PyObject *__pyx_n_s_Angular_Fingerprint___init;
static PyObject *__pyx_n_s_Angular_Fingerprint__get_neighb;
static PyObject *__pyx_n_s_Angular_Fingerprint_get_all_feat;
static PyObject *__pyx_n_s_Angular_Fingerprint_get_feature;
static PyObject *__pyx_n_s_Angular_Fingerprint_get_featureG;
static PyObject *__pyx_n_s_Angular_Fingerprint_get_featureM;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_n_s_Natoms;
static PyObject *__pyx_n_s_Nbins1;
static PyObject *__pyx_n_s_Nbins2;
static PyObject *__pyx_n_s_Nbondtypes_2body;
static PyObject *__pyx_n_s_Nbondtypes_3body;
static PyObject *__pyx_n_s_Ncells;
static PyObject *__pyx_n_s_Nelements;
static PyObject *__pyx_n_s_Nelements_2body;
static PyObject *__pyx_n_s_Nelements_3body;
static PyObject *__pyx_n_s_Ntypes;
static PyObject *__pyx_n_s_Rc1;
static PyObject *__pyx_n_s_Rc2;
static PyObject *__pyx_n_s_Rc_max;
static PyObject *__pyx_n_s_Rij;
static PyObject *__pyx_n_s_RijVec;
static PyObject *__pyx_n_s_Rik;
static PyObject *__pyx_n_s_RikVec;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_kp_s_agox_models_descriptors_fingerpr;
static PyObject *__pyx_n_s_agox_models_descriptors_fingerpr_2;
static PyObject *__pyx_n_s_angle;
static PyObject *__pyx_n_s_angle_grad_i;
static PyObject *__pyx_n_s_angle_grad_j;
static PyObject *__pyx_n_s_angle_grad_k;
static PyObject *__pyx_n_s_arg_low;
static PyObject *__pyx_n_s_arg_up;
static PyObject *__pyx_n_s_argv;
static PyObject *__pyx_n_s_array;
static PyObject *__pyx_n_s_astype;
static PyObject *__pyx_n_s_atomic_count;
static PyObject *__pyx_n_s_atomic_types;
static PyObject *__pyx_n_s_atoms;
static PyObject *__pyx_n_s_atoms_list;
static PyObject *__pyx_n_s_axis;
static PyObject *__pyx_n_s_bin_index;
static PyObject *__pyx_n_s_binpos;
static PyObject *__pyx_n_s_binwidth1;
static PyObject *__pyx_n_s_binwidth2;
static PyObject *__pyx_n_s_bondtype_index;
static PyObject *__pyx_n_s_bondtypes_2body;
static PyObject *__pyx_n_s_bondtypes_2body_old;
static PyObject *__pyx_n_s_bondtypes_3body;
static PyObject *__pyx_n_s_bondtypes_3body_keys;
static PyObject *__pyx_n_s_bondtypes_3body_old;
static PyObject *__pyx_n_s_c;
static PyObject *__pyx_n_s_cdist;
static PyObject *__pyx_n_s_ceil;
static PyObject *__pyx_n_s_cell;
static PyObject *__pyx_n_s_cell_displacements;
static PyObject *__pyx_n_s_cell_displacements_old;
static PyObject *__pyx_n_s_cell_index;
static PyObject *__pyx_n_s_cell_index1;
static PyObject *__pyx_n_s_cell_index2;
static PyObject *__pyx_n_s_cell_vec_norms;
static PyObject *__pyx_n_s_center_bin;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_kp_s_comparator_for_construction_of;
static PyObject *__pyx_n_s_cond_ij;
static PyObject *__pyx_n_s_cond_ik;
static PyObject *__pyx_n_s_convert_atom_types;
static PyObject *__pyx_n_s_cos_angle;
static PyObject *__pyx_n_s_count;
static PyObject *__pyx_n_s_cwd;
static PyObject *__pyx_n_s_dim;
static PyObject *__pyx_n_s_displacement;
static PyObject *__pyx_n_s_displacement1;
static PyObject *__pyx_n_s_displacement2;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_dot;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_erf;
static PyObject *__pyx_n_s_erfarg_low;
static PyObject *__pyx_n_s_erfarg_up;
static PyObject *__pyx_n_s_eta;
static PyObject *__pyx_n_s_fc_grad_ij;
static PyObject *__pyx_n_s_fc_grad_ik;
static PyObject *__pyx_n_s_fc_ij;
static PyObject *__pyx_n_s_fc_ik;
static PyObject *__pyx_n_s_feature1;
static PyObject *__pyx_n_s_feature1_np;
static PyObject *__pyx_n_s_feature2;
static PyObject *__pyx_n_s_feature2_np;
static PyObject *__pyx_n_s_featureMat;
static PyObject *__pyx_n_s_feature_grad1;
static PyObject *__pyx_n_s_feature_grad1_np;
static PyObject *__pyx_n_s_feature_grad2;
static PyObject *__pyx_n_s_feature_grad2_np;
static PyObject *__pyx_n_s_feature_grad_np;
static PyObject *__pyx_n_s_feature_grads;
static PyObject *__pyx_n_s_feature_np;
static PyObject *__pyx_n_s_gamma;
static PyObject *__pyx_n_s_get_all_featureGradients;
static PyObject *__pyx_n_s_get_atomic_numbers;
static PyObject *__pyx_n_s_get_cell;
static PyObject *__pyx_n_s_get_feature;
static PyObject *__pyx_n_s_get_featureGradient;
static PyObject *__pyx_n_s_get_featureMat;
static PyObject *__pyx_n_s_get_neighbour_cells_displaceme;
static PyObject *__pyx_n_s_get_pbc;
static PyObject *__pyx_n_s_get_positions;
static PyObject *__pyx_n_s_get_volume;
static PyObject *__pyx_n_s_getcwd;
static PyObject *__pyx_n_s_grad_index;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_itertools;
static PyObject *__pyx_n_s_j;
static PyObject *__pyx_n_s_k;
static PyObject *__pyx_n_s_k1;
static PyObject *__pyx_n_s_k2;
static PyObject *__pyx_n_s_k3;
static PyObject *__pyx_n_s_k_start;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_linalg;
static PyObject *__pyx_n_s_m;
static PyObject *__pyx_n_s_m1;
static PyObject *__pyx_n_s_m2;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_math;
static PyObject *__pyx_n_s_maxbin_lim;
static PyObject *__pyx_n_s_mem;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_minbin_lim;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_n;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_ncellmax;
static PyObject *__pyx_n_s_neighbourcells_disp;
static PyObject *__pyx_n_s_neighbours;
static PyObject *__pyx_n_s_newbin;
static PyObject *__pyx_n_s_norm;
static PyObject *__pyx_n_s_normalization;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_nsigma;
static PyObject *__pyx_n_s_num;
static PyObject *__pyx_n_s_num_converted;
static PyObject *__pyx_n_s_num_converted_old;
static PyObject *__pyx_n_s_num_pairs;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_s_numpy__core_multiarray_failed_to;
static PyObject *__pyx_kp_s_numpy__core_umath_failed_to_impo;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_n_s_ones;
static PyObject *__pyx_n_s_os;
static PyObject *__pyx_n_s_pbc;
static PyObject *__pyx_n_s_pi;
static PyObject *__pyx_n_s_pos;
static PyObject *__pyx_n_s_pos_i;
static PyObject *__pyx_n_s_pos_j;
static PyObject *__pyx_n_s_pos_k;
static PyObject *__pyx_n_s_pos_np;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_product;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reshape;
static PyObject *__pyx_n_s_scipy_spatial_distance;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_sigma1;
static PyObject *__pyx_n_s_sigma2;
static PyObject *__pyx_n_s_smearing_norm1;
static PyObject *__pyx_n_s_smearing_norm2;
static PyObject *__pyx_n_s_sqrt;
static PyObject *__pyx_n_s_subarray;
static PyObject *__pyx_n_s_sum;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_t1;
static PyObject *__pyx_n_s_t2;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_n_s_tolist;
static PyObject *__pyx_n_s_tt1;
static PyObject *__pyx_n_s_tt2;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_n_s_type1;
static PyObject *__pyx_n_s_type2;
static PyObject *__pyx_n_s_type3;
static PyObject *__pyx_n_s_type_converter;
static PyObject *__pyx_n_s_use_angular;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_value1;
static PyObject *__pyx_n_s_value2;
static PyObject *__pyx_n_s_volume;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_xyz;
static PyObject *__pyx_n_s_y;
static PyObject *__pyx_n_s_z;
static PyObject *__pyx_n_s_zeros;
static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_convert_atom_types(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_num); /* proto */
static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_atoms, PyObject *__pyx_v_Rc1, PyObject *__pyx_v_Rc2, PyObject *__pyx_v_binwidth1, PyObject *__pyx_v_Nbins2, PyObject *__pyx_v_sigma1, PyObject *__pyx_v_sigma2, PyObject *__pyx_v_nsigma, PyObject *__pyx_v_eta, PyObject *__pyx_v_gamma, PyObject *__pyx_v_use_angular); /* proto */
static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_2get_feature(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_atoms); /* proto */
static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_4get_featureMat(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_atoms_list); /* proto */
static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_6get_featureGradient(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_atoms); /* proto */
static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_8get_all_featureGradients(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_atoms_list); /* proto */
static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_10__get_neighbour_cells_displacement(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_pbc, PyObject *__pyx_v_cell); /* proto */
static PyObject *__pyx_float_0_1;
static PyObject *__pyx_float_0_2;
static PyObject *__pyx_float_4_0;
static PyObject *__pyx_float_0_10;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_30;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_slice_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_codeobj__5;
static PyObject *__pyx_codeobj__8;
static PyObject *__pyx_codeobj__11;
static PyObject *__pyx_codeobj__13;
static PyObject *__pyx_codeobj__15;
static PyObject *__pyx_codeobj__17;
static PyObject *__pyx_codeobj__19;
/* Late includes */

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":26
 *     double coord[3]
 * 
 * cdef Point subtract(Point p1, Point p2):             # <<<<<<<<<<<<<<
 *     cdef Point p
 *     p.coord[0] = p1.coord[0] - p2.coord[0]
 */

static __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_subtract(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_p1, __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_p2) {
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_p;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("subtract", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":28
 * cdef Point subtract(Point p1, Point p2):
 *     cdef Point p
 *     p.coord[0] = p1.coord[0] - p2.coord[0]             # <<<<<<<<<<<<<<
 *     p.coord[1] = p1.coord[1] - p2.coord[1]
 *     p.coord[2] = p1.coord[2] - p2.coord[2]
 */
  (__pyx_v_p.coord[0]) = ((__pyx_v_p1.coord[0]) - (__pyx_v_p2.coord[0]));

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":29
 *     cdef Point p
 *     p.coord[0] = p1.coord[0] - p2.coord[0]
 *     p.coord[1] = p1.coord[1] - p2.coord[1]             # <<<<<<<<<<<<<<
 *     p.coord[2] = p1.coord[2] - p2.coord[2]
 *     return p
 */
  (__pyx_v_p.coord[1]) = ((__pyx_v_p1.coord[1]) - (__pyx_v_p2.coord[1]));

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":30
 *     p.coord[0] = p1.coord[0] - p2.coord[0]
 *     p.coord[1] = p1.coord[1] - p2.coord[1]
 *     p.coord[2] = p1.coord[2] - p2.coord[2]             # <<<<<<<<<<<<<<
 *     return p
 * 
 */
  (__pyx_v_p.coord[2]) = ((__pyx_v_p1.coord[2]) - (__pyx_v_p2.coord[2]));

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":31
 *     p.coord[1] = p1.coord[1] - p2.coord[1]
 *     p.coord[2] = p1.coord[2] - p2.coord[2]
 *     return p             # <<<<<<<<<<<<<<
 * 
 * cdef Point add(Point p1, Point p2):
 */
  __pyx_r = __pyx_v_p;
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":26
 *     double coord[3]
 * 
 * cdef Point subtract(Point p1, Point p2):             # <<<<<<<<<<<<<<
 *     cdef Point p
 *     p.coord[0] = p1.coord[0] - p2.coord[0]
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":33
 *     return p
 * 
 * cdef Point add(Point p1, Point p2):             # <<<<<<<<<<<<<<
 *     cdef Point p
 *     p.coord[0] = p1.coord[0] + p2.coord[0]
 */

static __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_add(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_p1, __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_p2) {
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_p;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":35
 * cdef Point add(Point p1, Point p2):
 *     cdef Point p
 *     p.coord[0] = p1.coord[0] + p2.coord[0]             # <<<<<<<<<<<<<<
 *     p.coord[1] = p1.coord[1] + p2.coord[1]
 *     p.coord[2] = p1.coord[2] + p2.coord[2]
 */
  (__pyx_v_p.coord[0]) = ((__pyx_v_p1.coord[0]) + (__pyx_v_p2.coord[0]));

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":36
 *     cdef Point p
 *     p.coord[0] = p1.coord[0] + p2.coord[0]
 *     p.coord[1] = p1.coord[1] + p2.coord[1]             # <<<<<<<<<<<<<<
 *     p.coord[2] = p1.coord[2] + p2.coord[2]
 *     return p
 */
  (__pyx_v_p.coord[1]) = ((__pyx_v_p1.coord[1]) + (__pyx_v_p2.coord[1]));

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":37
 *     p.coord[0] = p1.coord[0] + p2.coord[0]
 *     p.coord[1] = p1.coord[1] + p2.coord[1]
 *     p.coord[2] = p1.coord[2] + p2.coord[2]             # <<<<<<<<<<<<<<
 *     return p
 * 
 */
  (__pyx_v_p.coord[2]) = ((__pyx_v_p1.coord[2]) + (__pyx_v_p2.coord[2]));

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":38
 *     p.coord[1] = p1.coord[1] + p2.coord[1]
 *     p.coord[2] = p1.coord[2] + p2.coord[2]
 *     return p             # <<<<<<<<<<<<<<
 * 
 * cdef double norm(Point p):
 */
  __pyx_r = __pyx_v_p;
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":33
 *     return p
 * 
 * cdef Point add(Point p1, Point p2):             # <<<<<<<<<<<<<<
 *     cdef Point p
 *     p.coord[0] = p1.coord[0] + p2.coord[0]
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":40
 *     return p
 * 
 * cdef double norm(Point p):             # <<<<<<<<<<<<<<
 *     return sqrt(p.coord[0]*p.coord[0] + p.coord[1]*p.coord[1] + p.coord[2]*p.coord[2])
 * 
 */

static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_norm(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_p) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("norm", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":41
 * 
 * cdef double norm(Point p):
 *     return sqrt(p.coord[0]*p.coord[0] + p.coord[1]*p.coord[1] + p.coord[2]*p.coord[2])             # <<<<<<<<<<<<<<
 * 
 * cdef double euclidean(Point p1, Point p2):
 */
  __pyx_r = sqrt(((((__pyx_v_p.coord[0]) * (__pyx_v_p.coord[0])) + ((__pyx_v_p.coord[1]) * (__pyx_v_p.coord[1]))) + ((__pyx_v_p.coord[2]) * (__pyx_v_p.coord[2]))));
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":40
 *     return p
 * 
 * cdef double norm(Point p):             # <<<<<<<<<<<<<<
 *     return sqrt(p.coord[0]*p.coord[0] + p.coord[1]*p.coord[1] + p.coord[2]*p.coord[2])
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":43
 *     return sqrt(p.coord[0]*p.coord[0] + p.coord[1]*p.coord[1] + p.coord[2]*p.coord[2])
 * 
 * cdef double euclidean(Point p1, Point p2):             # <<<<<<<<<<<<<<
 *     return norm(subtract(p1,p2))
 * 
 */

static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_euclidean(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_p1, __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_p2) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("euclidean", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":44
 * 
 * cdef double euclidean(Point p1, Point p2):
 *     return norm(subtract(p1,p2))             # <<<<<<<<<<<<<<
 * 
 * cdef double dot(Point v1, Point v2):
 */
  __pyx_r = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_norm(__pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_subtract(__pyx_v_p1, __pyx_v_p2));
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":43
 *     return sqrt(p.coord[0]*p.coord[0] + p.coord[1]*p.coord[1] + p.coord[2]*p.coord[2])
 * 
 * cdef double euclidean(Point p1, Point p2):             # <<<<<<<<<<<<<<
 *     return norm(subtract(p1,p2))
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":46
 *     return norm(subtract(p1,p2))
 * 
 * cdef double dot(Point v1, Point v2):             # <<<<<<<<<<<<<<
 *     return v1.coord[0]*v2.coord[0] + v1.coord[1]*v2.coord[1] + v1.coord[2]*v2.coord[2]
 * 
 */

static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_dot(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_v1, __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_v2) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("dot", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":47
 * 
 * cdef double dot(Point v1, Point v2):
 *     return v1.coord[0]*v2.coord[0] + v1.coord[1]*v2.coord[1] + v1.coord[2]*v2.coord[2]             # <<<<<<<<<<<<<<
 * 
 * cdef double get_angle(Point v1, Point v2):
 */
  __pyx_r = ((((__pyx_v_v1.coord[0]) * (__pyx_v_v2.coord[0])) + ((__pyx_v_v1.coord[1]) * (__pyx_v_v2.coord[1]))) + ((__pyx_v_v1.coord[2]) * (__pyx_v_v2.coord[2])));
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":46
 *     return norm(subtract(p1,p2))
 * 
 * cdef double dot(Point v1, Point v2):             # <<<<<<<<<<<<<<
 *     return v1.coord[0]*v2.coord[0] + v1.coord[1]*v2.coord[1] + v1.coord[2]*v2.coord[2]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":49
 *     return v1.coord[0]*v2.coord[0] + v1.coord[1]*v2.coord[1] + v1.coord[2]*v2.coord[2]
 * 
 * cdef double get_angle(Point v1, Point v2):             # <<<<<<<<<<<<<<
 *     """
 *     Returns angle with convention [0,pi]
 */

static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_get_angle(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_v1, __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_v2) {
  double __pyx_v_norm1;
  double __pyx_v_norm2;
  double __pyx_v_arg;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  double __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_angle", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":53
 *     Returns angle with convention [0,pi]
 *     """
 *     norm1 = norm(v1)             # <<<<<<<<<<<<<<
 *     norm2 = norm(v2)
 *     arg = dot(v1,v2)/(norm1*norm2)
 */
  __pyx_v_norm1 = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_norm(__pyx_v_v1);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":54
 *     """
 *     norm1 = norm(v1)
 *     norm2 = norm(v2)             # <<<<<<<<<<<<<<
 *     arg = dot(v1,v2)/(norm1*norm2)
 *     # This is added to correct for numerical errors
 */
  __pyx_v_norm2 = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_norm(__pyx_v_v2);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":55
 *     norm1 = norm(v1)
 *     norm2 = norm(v2)
 *     arg = dot(v1,v2)/(norm1*norm2)             # <<<<<<<<<<<<<<
 *     # This is added to correct for numerical errors
 *     if arg < -1:
 */
  __pyx_t_1 = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_dot(__pyx_v_v1, __pyx_v_v2);
  __pyx_t_2 = (__pyx_v_norm1 * __pyx_v_norm2);
  if (unlikely(__pyx_t_2 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 55, __pyx_L1_error)
  }
  __pyx_v_arg = (__pyx_t_1 / __pyx_t_2);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":57
 *     arg = dot(v1,v2)/(norm1*norm2)
 *     # This is added to correct for numerical errors
 *     if arg < -1:             # <<<<<<<<<<<<<<
 *         arg = -1.
 *     elif arg > 1:
 */
  __pyx_t_3 = ((__pyx_v_arg < -1.0) != 0);
  if (__pyx_t_3) {

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":58
 *     # This is added to correct for numerical errors
 *     if arg < -1:
 *         arg = -1.             # <<<<<<<<<<<<<<
 *     elif arg > 1:
 *         arg = 1.
 */
    __pyx_v_arg = -1.;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":57
 *     arg = dot(v1,v2)/(norm1*norm2)
 *     # This is added to correct for numerical errors
 *     if arg < -1:             # <<<<<<<<<<<<<<
 *         arg = -1.
 *     elif arg > 1:
 */
    goto __pyx_L3;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":59
 *     if arg < -1:
 *         arg = -1.
 *     elif arg > 1:             # <<<<<<<<<<<<<<
 *         arg = 1.
 *     return acos(arg)
 */
  __pyx_t_3 = ((__pyx_v_arg > 1.0) != 0);
  if (__pyx_t_3) {

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":60
 *         arg = -1.
 *     elif arg > 1:
 *         arg = 1.             # <<<<<<<<<<<<<<
 *     return acos(arg)
 * 
 */
    __pyx_v_arg = 1.;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":59
 *     if arg < -1:
 *         arg = -1.
 *     elif arg > 1:             # <<<<<<<<<<<<<<
 *         arg = 1.
 *     return acos(arg)
 */
  }
  __pyx_L3:;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":61
 *     elif arg > 1:
 *         arg = 1.
 *     return acos(arg)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = acos(__pyx_v_arg);
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":49
 *     return v1.coord[0]*v2.coord[0] + v1.coord[1]*v2.coord[1] + v1.coord[2]*v2.coord[2]
 * 
 * cdef double get_angle(Point v1, Point v2):             # <<<<<<<<<<<<<<
 *     """
 *     Returns angle with convention [0,pi]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.get_angle", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":64
 * 
 * @cython.cdivision(True)
 * cdef double f_cutoff(double r, double gamma, double Rcut):             # <<<<<<<<<<<<<<
 *     """
 *     Polinomial cutoff function in the, with the steepness determined by "gamma"
 */

static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_f_cutoff(double __pyx_v_r, double __pyx_v_gamma, double __pyx_v_Rcut) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("f_cutoff", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":70
 *     For large gamma, the function goes towards a step function at Rc.
 *     """
 *     if not gamma == 0:             # <<<<<<<<<<<<<<
 *         return 1 + gamma*pow(r/Rcut, gamma+1) - (gamma+1)*pow(r/Rcut, gamma)
 *     else:
 */
  __pyx_t_1 = ((!((__pyx_v_gamma == 0.0) != 0)) != 0);
  if (__pyx_t_1) {

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":71
 *     """
 *     if not gamma == 0:
 *         return 1 + gamma*pow(r/Rcut, gamma+1) - (gamma+1)*pow(r/Rcut, gamma)             # <<<<<<<<<<<<<<
 *     else:
 *         return 1
 */
    __pyx_r = ((1.0 + (__pyx_v_gamma * pow((__pyx_v_r / __pyx_v_Rcut), (__pyx_v_gamma + 1.0)))) - ((__pyx_v_gamma + 1.0) * pow((__pyx_v_r / __pyx_v_Rcut), __pyx_v_gamma)));
    goto __pyx_L0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":70
 *     For large gamma, the function goes towards a step function at Rc.
 *     """
 *     if not gamma == 0:             # <<<<<<<<<<<<<<
 *         return 1 + gamma*pow(r/Rcut, gamma+1) - (gamma+1)*pow(r/Rcut, gamma)
 *     else:
 */
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":73
 *         return 1 + gamma*pow(r/Rcut, gamma+1) - (gamma+1)*pow(r/Rcut, gamma)
 *     else:
 *         return 1             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  /*else*/ {
    __pyx_r = 1.0;
    goto __pyx_L0;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":64
 * 
 * @cython.cdivision(True)
 * cdef double f_cutoff(double r, double gamma, double Rcut):             # <<<<<<<<<<<<<<
 *     """
 *     Polinomial cutoff function in the, with the steepness determined by "gamma"
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":76
 * 
 * @cython.cdivision(True)
 * cdef double f_cutoff_grad(double r, double gamma, double Rcut):             # <<<<<<<<<<<<<<
 *     if not gamma == 0:
 *         return gamma*(gamma+1)/Rcut * (pow(r/Rcut, gamma) - pow(r/Rcut, gamma-1))
 */

static double __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_f_cutoff_grad(double __pyx_v_r, double __pyx_v_gamma, double __pyx_v_Rcut) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("f_cutoff_grad", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":77
 * @cython.cdivision(True)
 * cdef double f_cutoff_grad(double r, double gamma, double Rcut):
 *     if not gamma == 0:             # <<<<<<<<<<<<<<
 *         return gamma*(gamma+1)/Rcut * (pow(r/Rcut, gamma) - pow(r/Rcut, gamma-1))
 *     else:
 */
  __pyx_t_1 = ((!((__pyx_v_gamma == 0.0) != 0)) != 0);
  if (__pyx_t_1) {

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":78
 * cdef double f_cutoff_grad(double r, double gamma, double Rcut):
 *     if not gamma == 0:
 *         return gamma*(gamma+1)/Rcut * (pow(r/Rcut, gamma) - pow(r/Rcut, gamma-1))             # <<<<<<<<<<<<<<
 *     else:
 *         return 0
 */
    __pyx_r = (((__pyx_v_gamma * (__pyx_v_gamma + 1.0)) / __pyx_v_Rcut) * (pow((__pyx_v_r / __pyx_v_Rcut), __pyx_v_gamma) - pow((__pyx_v_r / __pyx_v_Rcut), (__pyx_v_gamma - 1.0))));
    goto __pyx_L0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":77
 * @cython.cdivision(True)
 * cdef double f_cutoff_grad(double r, double gamma, double Rcut):
 *     if not gamma == 0:             # <<<<<<<<<<<<<<
 *         return gamma*(gamma+1)/Rcut * (pow(r/Rcut, gamma) - pow(r/Rcut, gamma-1))
 *     else:
 */
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":80
 *         return gamma*(gamma+1)/Rcut * (pow(r/Rcut, gamma) - pow(r/Rcut, gamma-1))
 *     else:
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * def convert_atom_types(num):
 */
  /*else*/ {
    __pyx_r = 0.0;
    goto __pyx_L0;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":76
 * 
 * @cython.cdivision(True)
 * cdef double f_cutoff_grad(double r, double gamma, double Rcut):             # <<<<<<<<<<<<<<
 *     if not gamma == 0:
 *         return gamma*(gamma+1)/Rcut * (pow(r/Rcut, gamma) - pow(r/Rcut, gamma-1))
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":82
 *         return 0
 * 
 * def convert_atom_types(num):             # <<<<<<<<<<<<<<
 *     cdef int Natoms = len(num)
 *     cdef list atomic_types = sorted(list(set(num)))
 */

/* Python wrapper */
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_1convert_atom_types(PyObject *__pyx_self, PyObject *__pyx_v_num); /*proto*/
static PyMethodDef __pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_1convert_atom_types = {"convert_atom_types", (PyCFunction)__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_1convert_atom_types, METH_O, 0};
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_1convert_atom_types(PyObject *__pyx_self, PyObject *__pyx_v_num) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("convert_atom_types (wrapper)", 0);
  __pyx_r = __pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_convert_atom_types(__pyx_self, ((PyObject *)__pyx_v_num));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_convert_atom_types(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_num) {
  int __pyx_v_Natoms;
  PyObject *__pyx_v_atomic_types = 0;
  int __pyx_v_Ntypes;
  PyObject *__pyx_v_num_converted = 0;
  int __pyx_v_i;
  int __pyx_v_j;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert_atom_types", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":83
 * 
 * def convert_atom_types(num):
 *     cdef int Natoms = len(num)             # <<<<<<<<<<<<<<
 *     cdef list atomic_types = sorted(list(set(num)))
 *     cdef int Ntypes = len(atomic_types)
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_num); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 83, __pyx_L1_error)
  __pyx_v_Natoms = __pyx_t_1;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":84
 * def convert_atom_types(num):
 *     cdef int Natoms = len(num)
 *     cdef list atomic_types = sorted(list(set(num)))             # <<<<<<<<<<<<<<
 *     cdef int Ntypes = len(atomic_types)
 *     cdef list num_converted = [0]*Natoms
 */
  __pyx_t_3 = PySet_New(__pyx_v_num); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PySequence_List(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_List(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_2 = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_5 = PyList_Sort(__pyx_t_2); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 84, __pyx_L1_error)
  __pyx_v_atomic_types = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":85
 *     cdef int Natoms = len(num)
 *     cdef list atomic_types = sorted(list(set(num)))
 *     cdef int Ntypes = len(atomic_types)             # <<<<<<<<<<<<<<
 *     cdef list num_converted = [0]*Natoms
 *     cdef int i, j
 */
  if (unlikely(__pyx_v_atomic_types == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 85, __pyx_L1_error)
  }
  __pyx_t_1 = PyList_GET_SIZE(__pyx_v_atomic_types); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 85, __pyx_L1_error)
  __pyx_v_Ntypes = __pyx_t_1;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":86
 *     cdef list atomic_types = sorted(list(set(num)))
 *     cdef int Ntypes = len(atomic_types)
 *     cdef list num_converted = [0]*Natoms             # <<<<<<<<<<<<<<
 *     cdef int i, j
 *     for i in range(Natoms):
 */
  __pyx_t_2 = PyList_New(1 * ((__pyx_v_Natoms<0) ? 0:__pyx_v_Natoms)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  { Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < __pyx_v_Natoms; __pyx_temp++) {
      __Pyx_INCREF(__pyx_int_0);
      __Pyx_GIVEREF(__pyx_int_0);
      PyList_SET_ITEM(__pyx_t_2, __pyx_temp, __pyx_int_0);
    }
  }
  __pyx_v_num_converted = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":88
 *     cdef list num_converted = [0]*Natoms
 *     cdef int i, j
 *     for i in range(Natoms):             # <<<<<<<<<<<<<<
 *         for j in range(Ntypes):
 *             if num[i] == atomic_types[j]:
 */
  __pyx_t_6 = __pyx_v_Natoms;
  __pyx_t_7 = __pyx_t_6;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":89
 *     cdef int i, j
 *     for i in range(Natoms):
 *         for j in range(Ntypes):             # <<<<<<<<<<<<<<
 *             if num[i] == atomic_types[j]:
 *                 num_converted[i] = j
 */
    __pyx_t_9 = __pyx_v_Ntypes;
    __pyx_t_10 = __pyx_t_9;
    for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
      __pyx_v_j = __pyx_t_11;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":90
 *     for i in range(Natoms):
 *         for j in range(Ntypes):
 *             if num[i] == atomic_types[j]:             # <<<<<<<<<<<<<<
 *                 num_converted[i] = j
 *     return num_converted
 */
      __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_num, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(__pyx_v_atomic_types == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 90, __pyx_L1_error)
      }
      __pyx_t_3 = __Pyx_GetItemInt_List(__pyx_v_atomic_types, __pyx_v_j, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 90, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 90, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 90, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_12) {

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":91
 *         for j in range(Ntypes):
 *             if num[i] == atomic_types[j]:
 *                 num_converted[i] = j             # <<<<<<<<<<<<<<
 *     return num_converted
 * 
 */
        __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_j); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 91, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_num_converted, __pyx_v_i, __pyx_t_4, int, 1, __Pyx_PyInt_From_int, 1, 1, 1) < 0)) __PYX_ERR(0, 91, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":90
 *     for i in range(Natoms):
 *         for j in range(Ntypes):
 *             if num[i] == atomic_types[j]:             # <<<<<<<<<<<<<<
 *                 num_converted[i] = j
 *     return num_converted
 */
      }
    }
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":92
 *             if num[i] == atomic_types[j]:
 *                 num_converted[i] = j
 *     return num_converted             # <<<<<<<<<<<<<<
 * 
 * class Angular_Fingerprint(object):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_num_converted);
  __pyx_r = __pyx_v_num_converted;
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":82
 *         return 0
 * 
 * def convert_atom_types(num):             # <<<<<<<<<<<<<<
 *     cdef int Natoms = len(num)
 *     cdef list atomic_types = sorted(list(set(num)))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.convert_atom_types", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_atomic_types);
  __Pyx_XDECREF(__pyx_v_num_converted);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":97
 *     """ comparator for construction of angular fingerprints
 *     """
 *     def __init__(self, atoms, Rc1=4.0, Rc2=4.0, binwidth1=0.1, Nbins2=30, sigma1=0.2, sigma2=0.10, nsigma=4, eta=1, gamma=3, use_angular=True):             # <<<<<<<<<<<<<<
 *         """ Set a common cut of radius
 *         """
 */

/* Python wrapper */
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint___init__[] = " Set a common cut of radius\n        ";
static PyMethodDef __pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_1__init__ = {"__init__", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_1__init__, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint___init__};
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_atoms = 0;
  PyObject *__pyx_v_Rc1 = 0;
  PyObject *__pyx_v_Rc2 = 0;
  PyObject *__pyx_v_binwidth1 = 0;
  PyObject *__pyx_v_Nbins2 = 0;
  PyObject *__pyx_v_sigma1 = 0;
  PyObject *__pyx_v_sigma2 = 0;
  PyObject *__pyx_v_nsigma = 0;
  PyObject *__pyx_v_eta = 0;
  PyObject *__pyx_v_gamma = 0;
  PyObject *__pyx_v_use_angular = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_atoms,&__pyx_n_s_Rc1,&__pyx_n_s_Rc2,&__pyx_n_s_binwidth1,&__pyx_n_s_Nbins2,&__pyx_n_s_sigma1,&__pyx_n_s_sigma2,&__pyx_n_s_nsigma,&__pyx_n_s_eta,&__pyx_n_s_gamma,&__pyx_n_s_use_angular,0};
    PyObject* values[12] = {0,0,0,0,0,0,0,0,0,0,0,0};
    values[2] = ((PyObject *)((PyObject*)__pyx_float_4_0));
    values[3] = ((PyObject *)((PyObject*)__pyx_float_4_0));
    values[4] = ((PyObject *)((PyObject*)__pyx_float_0_1));
    values[5] = ((PyObject *)((PyObject *)__pyx_int_30));
    values[6] = ((PyObject *)((PyObject*)__pyx_float_0_2));
    values[7] = ((PyObject *)((PyObject*)__pyx_float_0_10));
    values[8] = ((PyObject *)((PyObject *)__pyx_int_4));
    values[9] = ((PyObject *)((PyObject *)__pyx_int_1));
    values[10] = ((PyObject *)((PyObject *)__pyx_int_3));
    values[11] = ((PyObject *)((PyObject *)Py_True));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_atoms)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 12, 1); __PYX_ERR(0, 97, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_Rc1);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_Rc2);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_binwidth1);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_Nbins2);
          if (value) { values[5] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_sigma1);
          if (value) { values[6] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_sigma2);
          if (value) { values[7] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_nsigma);
          if (value) { values[8] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_eta);
          if (value) { values[9] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_gamma);
          if (value) { values[10] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case 11:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_angular);
          if (value) { values[11] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 97, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_atoms = values[1];
    __pyx_v_Rc1 = values[2];
    __pyx_v_Rc2 = values[3];
    __pyx_v_binwidth1 = values[4];
    __pyx_v_Nbins2 = values[5];
    __pyx_v_sigma1 = values[6];
    __pyx_v_sigma2 = values[7];
    __pyx_v_nsigma = values[8];
    __pyx_v_eta = values[9];
    __pyx_v_gamma = values[10];
    __pyx_v_use_angular = values[11];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 12, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 97, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint___init__(__pyx_self, __pyx_v_self, __pyx_v_atoms, __pyx_v_Rc1, __pyx_v_Rc2, __pyx_v_binwidth1, __pyx_v_Nbins2, __pyx_v_sigma1, __pyx_v_sigma2, __pyx_v_nsigma, __pyx_v_eta, __pyx_v_gamma, __pyx_v_use_angular);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_atoms, PyObject *__pyx_v_Rc1, PyObject *__pyx_v_Rc2, PyObject *__pyx_v_binwidth1, PyObject *__pyx_v_Nbins2, PyObject *__pyx_v_sigma1, PyObject *__pyx_v_sigma2, PyObject *__pyx_v_nsigma, PyObject *__pyx_v_eta, PyObject *__pyx_v_gamma, PyObject *__pyx_v_use_angular) {
  PyObject *__pyx_v_type_converter = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_type = NULL;
  PyObject *__pyx_v_count = NULL;
  PyObject *__pyx_v_tt1 = NULL;
  PyObject *__pyx_v_tt2 = NULL;
  PyObject *__pyx_v_type1 = NULL;
  PyObject *__pyx_v_type2 = NULL;
  PyObject *__pyx_v_t1 = NULL;
  PyObject *__pyx_v_t2 = NULL;
  PyObject *__pyx_v_bondtypes_3body_keys = NULL;
  PyObject *__pyx_v_type3 = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_k1 = NULL;
  PyObject *__pyx_v_k2 = NULL;
  PyObject *__pyx_v_k3 = NULL;
  PyObject *__pyx_7genexpr__pyx_v_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  double __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  PyObject *(*__pyx_t_12)(PyObject *);
  int __pyx_t_13;
  int __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  PyObject *(*__pyx_t_18)(PyObject *);
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":101
 *         """
 * 
 *         self.Rc1 = Rc1             # <<<<<<<<<<<<<<
 *         self.Rc2 = Rc2
 *         self.binwidth1 = binwidth1
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Rc1, __pyx_v_Rc1) < 0) __PYX_ERR(0, 101, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":102
 * 
 *         self.Rc1 = Rc1
 *         self.Rc2 = Rc2             # <<<<<<<<<<<<<<
 *         self.binwidth1 = binwidth1
 *         self.Nbins2 = Nbins2
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Rc2, __pyx_v_Rc2) < 0) __PYX_ERR(0, 102, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":103
 *         self.Rc1 = Rc1
 *         self.Rc2 = Rc2
 *         self.binwidth1 = binwidth1             # <<<<<<<<<<<<<<
 *         self.Nbins2 = Nbins2
 *         self.binwidth2 = np.pi / Nbins2
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_binwidth1, __pyx_v_binwidth1) < 0) __PYX_ERR(0, 103, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":104
 *         self.Rc2 = Rc2
 *         self.binwidth1 = binwidth1
 *         self.Nbins2 = Nbins2             # <<<<<<<<<<<<<<
 *         self.binwidth2 = np.pi / Nbins2
 *         self.sigma1 = sigma1
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Nbins2, __pyx_v_Nbins2) < 0) __PYX_ERR(0, 104, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":105
 *         self.binwidth1 = binwidth1
 *         self.Nbins2 = Nbins2
 *         self.binwidth2 = np.pi / Nbins2             # <<<<<<<<<<<<<<
 *         self.sigma1 = sigma1
 *         self.sigma2 = sigma2
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_pi); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_v_Nbins2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_binwidth2, __pyx_t_1) < 0) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":106
 *         self.Nbins2 = Nbins2
 *         self.binwidth2 = np.pi / Nbins2
 *         self.sigma1 = sigma1             # <<<<<<<<<<<<<<
 *         self.sigma2 = sigma2
 *         self.nsigma = nsigma
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_sigma1, __pyx_v_sigma1) < 0) __PYX_ERR(0, 106, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":107
 *         self.binwidth2 = np.pi / Nbins2
 *         self.sigma1 = sigma1
 *         self.sigma2 = sigma2             # <<<<<<<<<<<<<<
 *         self.nsigma = nsigma
 *         self.eta = eta
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_sigma2, __pyx_v_sigma2) < 0) __PYX_ERR(0, 107, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":108
 *         self.sigma1 = sigma1
 *         self.sigma2 = sigma2
 *         self.nsigma = nsigma             # <<<<<<<<<<<<<<
 *         self.eta = eta
 *         self.gamma = gamma
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_nsigma, __pyx_v_nsigma) < 0) __PYX_ERR(0, 108, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":109
 *         self.sigma2 = sigma2
 *         self.nsigma = nsigma
 *         self.eta = eta             # <<<<<<<<<<<<<<
 *         self.gamma = gamma
 *         self.use_angular = use_angular
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_eta, __pyx_v_eta) < 0) __PYX_ERR(0, 109, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":110
 *         self.nsigma = nsigma
 *         self.eta = eta
 *         self.gamma = gamma             # <<<<<<<<<<<<<<
 *         self.use_angular = use_angular
 * 
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_gamma, __pyx_v_gamma) < 0) __PYX_ERR(0, 110, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":111
 *         self.eta = eta
 *         self.gamma = gamma
 *         self.use_angular = use_angular             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_use_angular, __pyx_v_use_angular) < 0) __PYX_ERR(0, 111, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":114
 * 
 * 
 *         self.volume = atoms.get_volume()             # <<<<<<<<<<<<<<
 *         self.pbc = atoms.get_pbc()
 *         self.Natoms = len(atoms)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_volume); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_volume, __pyx_t_1) < 0) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":115
 * 
 *         self.volume = atoms.get_volume()
 *         self.pbc = atoms.get_pbc()             # <<<<<<<<<<<<<<
 *         self.Natoms = len(atoms)
 *         self.dim = 3
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_pbc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_pbc, __pyx_t_1) < 0) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":116
 *         self.volume = atoms.get_volume()
 *         self.pbc = atoms.get_pbc()
 *         self.Natoms = len(atoms)             # <<<<<<<<<<<<<<
 *         self.dim = 3
 * 
 */
  __pyx_t_4 = PyObject_Length(__pyx_v_atoms); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 116, __pyx_L1_error)
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Natoms, __pyx_t_1) < 0) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":117
 *         self.pbc = atoms.get_pbc()
 *         self.Natoms = len(atoms)
 *         self.dim = 3             # <<<<<<<<<<<<<<
 * 
 *         # parameters for the binning:
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_dim, __pyx_int_3) < 0) __PYX_ERR(0, 117, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":120
 * 
 *         # parameters for the binning:
 *         self.m1 = self.nsigma*self.sigma1/self.binwidth1  # number of neighbour bins included.             # <<<<<<<<<<<<<<
 *         self.smearing_norm1 = erf(1/np.sqrt(2) * self.m1 * self.binwidth1/self.sigma1)  # Integral of the included part of the gauss
 *         self.Nbins1 = int(np.ceil(self.Rc1/self.binwidth1))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_nsigma); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sigma1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_binwidth1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_m1, __pyx_t_1) < 0) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":121
 *         # parameters for the binning:
 *         self.m1 = self.nsigma*self.sigma1/self.binwidth1  # number of neighbour bins included.
 *         self.smearing_norm1 = erf(1/np.sqrt(2) * self.m1 * self.binwidth1/self.sigma1)  # Integral of the included part of the gauss             # <<<<<<<<<<<<<<
 *         self.Nbins1 = int(np.ceil(self.Rc1/self.binwidth1))
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_int_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_int_2);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_int_1, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_m1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_binwidth1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sigma1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyFloat_FromDouble(erf(__pyx_t_5)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_smearing_norm1, __pyx_t_2) < 0) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":122
 *         self.m1 = self.nsigma*self.sigma1/self.binwidth1  # number of neighbour bins included.
 *         self.smearing_norm1 = erf(1/np.sqrt(2) * self.m1 * self.binwidth1/self.sigma1)  # Integral of the included part of the gauss
 *         self.Nbins1 = int(np.ceil(self.Rc1/self.binwidth1))             # <<<<<<<<<<<<<<
 * 
 *         self.m2 = self.nsigma*self.sigma2/self.binwidth2  # number of neighbour bins included.
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_ceil); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Rc1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_binwidth1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyNumber_Divide(__pyx_t_1, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyNumber_Int(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Nbins1, __pyx_t_3) < 0) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":124
 *         self.Nbins1 = int(np.ceil(self.Rc1/self.binwidth1))
 * 
 *         self.m2 = self.nsigma*self.sigma2/self.binwidth2  # number of neighbour bins included.             # <<<<<<<<<<<<<<
 *         self.smearing_norm2 = erf(1/np.sqrt(2) * self.m2 * self.binwidth2/self.sigma2)  # Integral of the included part of the gauss
 *         self.binwidth2 = np.pi/Nbins2
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_nsigma); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sigma2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = PyNumber_Multiply(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_binwidth2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_t_7, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_m2, __pyx_t_3) < 0) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":125
 * 
 *         self.m2 = self.nsigma*self.sigma2/self.binwidth2  # number of neighbour bins included.
 *         self.smearing_norm2 = erf(1/np.sqrt(2) * self.m2 * self.binwidth2/self.sigma2)  # Integral of the included part of the gauss             # <<<<<<<<<<<<<<
 *         self.binwidth2 = np.pi/Nbins2
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_2, __pyx_int_2) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_int_2);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyNumber_Divide(__pyx_int_1, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_m2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_7, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_binwidth2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = PyNumber_Multiply(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sigma2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_t_7, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyFloat_FromDouble(erf(__pyx_t_5)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_smearing_norm2, __pyx_t_2) < 0) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":126
 *         self.m2 = self.nsigma*self.sigma2/self.binwidth2  # number of neighbour bins included.
 *         self.smearing_norm2 = erf(1/np.sqrt(2) * self.m2 * self.binwidth2/self.sigma2)  # Integral of the included part of the gauss
 *         self.binwidth2 = np.pi/Nbins2             # <<<<<<<<<<<<<<
 * 
 *         # Prepare stuff to handle multiple species
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_pi); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_v_Nbins2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_binwidth2, __pyx_t_2) < 0) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":129
 * 
 *         # Prepare stuff to handle multiple species
 *         self.num = atoms.get_atomic_numbers()             # <<<<<<<<<<<<<<
 *         self.atomic_types = sorted(list(set(self.num)))
 *         self.atomic_count = {type:list(self.num).count(type) for type in self.atomic_types}
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_atomic_numbers); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_num, __pyx_t_2) < 0) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":130
 *         # Prepare stuff to handle multiple species
 *         self.num = atoms.get_atomic_numbers()
 *         self.atomic_types = sorted(list(set(self.num)))             # <<<<<<<<<<<<<<
 *         self.atomic_count = {type:list(self.num).count(type) for type in self.atomic_types}
 *         self.Ntypes = len(self.atomic_types)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_num); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = PySet_New(__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_List(__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PySequence_List(__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_2 = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;
  __pyx_t_8 = PyList_Sort(__pyx_t_2); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 130, __pyx_L1_error)
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_atomic_types, __pyx_t_2) < 0) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":131
 *         self.num = atoms.get_atomic_numbers()
 *         self.atomic_types = sorted(list(set(self.num)))
 *         self.atomic_count = {type:list(self.num).count(type) for type in self.atomic_types}             # <<<<<<<<<<<<<<
 *         self.Ntypes = len(self.atomic_types)
 * 
 */
  { /* enter inner scope */
    __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 131, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_types); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 131, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
      __pyx_t_3 = __pyx_t_7; __Pyx_INCREF(__pyx_t_3); __pyx_t_4 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 131, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_9 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 131, __pyx_L5_error)
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_7); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 131, __pyx_L5_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 131, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        } else {
          if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_7); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 131, __pyx_L5_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 131, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        }
      } else {
        __pyx_t_7 = __pyx_t_9(__pyx_t_3);
        if (unlikely(!__pyx_t_7)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 131, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_type, __pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_num); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 131, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_1 = PySequence_List(__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 131, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_count); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 131, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_7 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_1, __pyx_7genexpr__pyx_v_type) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_7genexpr__pyx_v_type);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 131, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(PyDict_SetItem(__pyx_t_2, (PyObject*)__pyx_7genexpr__pyx_v_type, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 131, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_type); __pyx_7genexpr__pyx_v_type = 0;
    goto __pyx_L8_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_type); __pyx_7genexpr__pyx_v_type = 0;
    goto __pyx_L1_error;
    __pyx_L8_exit_scope:;
  } /* exit inner scope */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_atomic_count, __pyx_t_2) < 0) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":132
 *         self.atomic_types = sorted(list(set(self.num)))
 *         self.atomic_count = {type:list(self.num).count(type) for type in self.atomic_types}
 *         self.Ntypes = len(self.atomic_types)             # <<<<<<<<<<<<<<
 * 
 *         type_converter = {}
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_types); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Ntypes, __pyx_t_2) < 0) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":134
 *         self.Ntypes = len(self.atomic_types)
 * 
 *         type_converter = {}             # <<<<<<<<<<<<<<
 *         for i, type in enumerate(self.atomic_types):
 *             type_converter[type] = i
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_type_converter = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":135
 * 
 *         type_converter = {}
 *         for i, type in enumerate(self.atomic_types):             # <<<<<<<<<<<<<<
 *             type_converter[type] = i
 * 
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_t_2 = __pyx_int_0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_types); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_7 = __pyx_t_3; __Pyx_INCREF(__pyx_t_7); __pyx_t_4 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 135, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_7))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_7)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 135, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_7, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 135, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 135, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_7, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 135, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      }
    } else {
      __pyx_t_3 = __pyx_t_9(__pyx_t_7);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 135, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_type, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_AddObjC(__pyx_t_2, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":136
 *         type_converter = {}
 *         for i, type in enumerate(self.atomic_types):
 *             type_converter[type] = i             # <<<<<<<<<<<<<<
 * 
 *         # Bondtypes - 2body
 */
    if (unlikely(PyDict_SetItem(__pyx_v_type_converter, __pyx_v_type, __pyx_v_i) < 0)) __PYX_ERR(0, 136, __pyx_L1_error)

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":135
 * 
 *         type_converter = {}
 *         for i, type in enumerate(self.atomic_types):             # <<<<<<<<<<<<<<
 *             type_converter[type] = i
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":139
 * 
 *         # Bondtypes - 2body
 *         self.Nbondtypes_2body = 0             # <<<<<<<<<<<<<<
 *         self.bondtypes_2body = -np.ones((self.Ntypes, self.Ntypes)).astype(int)
 *         count = 0
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_2body, __pyx_int_0) < 0) __PYX_ERR(0, 139, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":140
 *         # Bondtypes - 2body
 *         self.Nbondtypes_2body = 0
 *         self.bondtypes_2body = -np.ones((self.Ntypes, self.Ntypes)).astype(int)             # <<<<<<<<<<<<<<
 *         count = 0
 *         for tt1 in self.atomic_types:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_ones); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Ntypes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Ntypes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_1);
  __pyx_t_3 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_7 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_1, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_10);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_astype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, ((PyObject *)(&PyInt_Type))) : __Pyx_PyObject_CallOneArg(__pyx_t_6, ((PyObject *)(&PyInt_Type)));
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyNumber_Negative(__pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_2body, __pyx_t_6) < 0) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":141
 *         self.Nbondtypes_2body = 0
 *         self.bondtypes_2body = -np.ones((self.Ntypes, self.Ntypes)).astype(int)
 *         count = 0             # <<<<<<<<<<<<<<
 *         for tt1 in self.atomic_types:
 *             for tt2 in self.atomic_types:
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_count = __pyx_int_0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":142
 *         self.bondtypes_2body = -np.ones((self.Ntypes, self.Ntypes)).astype(int)
 *         count = 0
 *         for tt1 in self.atomic_types:             # <<<<<<<<<<<<<<
 *             for tt2 in self.atomic_types:
 *                 type1, type2 = tuple(sorted([tt1, tt2]))
 */
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_types); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
    __pyx_t_2 = __pyx_t_6; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 142, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_6); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 142, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 142, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_6); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 142, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 142, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      }
    } else {
      __pyx_t_6 = __pyx_t_9(__pyx_t_2);
      if (unlikely(!__pyx_t_6)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 142, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_XDECREF_SET(__pyx_v_tt1, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":143
 *         count = 0
 *         for tt1 in self.atomic_types:
 *             for tt2 in self.atomic_types:             # <<<<<<<<<<<<<<
 *                 type1, type2 = tuple(sorted([tt1, tt2]))
 *                 t1 = type_converter[type1]
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_types); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
      __pyx_t_7 = __pyx_t_6; __Pyx_INCREF(__pyx_t_7); __pyx_t_11 = 0;
      __pyx_t_12 = NULL;
    } else {
      __pyx_t_11 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 143, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_12 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 143, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    for (;;) {
      if (likely(!__pyx_t_12)) {
        if (likely(PyList_CheckExact(__pyx_t_7))) {
          if (__pyx_t_11 >= PyList_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_11); __Pyx_INCREF(__pyx_t_6); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 143, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_7, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 143, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        } else {
          if (__pyx_t_11 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_11); __Pyx_INCREF(__pyx_t_6); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 143, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_7, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 143, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        }
      } else {
        __pyx_t_6 = __pyx_t_12(__pyx_t_7);
        if (unlikely(!__pyx_t_6)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 143, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_XDECREF_SET(__pyx_v_tt2, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":144
 *         for tt1 in self.atomic_types:
 *             for tt2 in self.atomic_types:
 *                 type1, type2 = tuple(sorted([tt1, tt2]))             # <<<<<<<<<<<<<<
 *                 t1 = type_converter[type1]
 *                 t2 = type_converter[type2]
 */
      __pyx_t_10 = PyList_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 144, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_v_tt1);
      __Pyx_GIVEREF(__pyx_v_tt1);
      PyList_SET_ITEM(__pyx_t_10, 0, __pyx_v_tt1);
      __Pyx_INCREF(__pyx_v_tt2);
      __Pyx_GIVEREF(__pyx_v_tt2);
      PyList_SET_ITEM(__pyx_t_10, 1, __pyx_v_tt2);
      __pyx_t_6 = ((PyObject*)__pyx_t_10);
      __pyx_t_10 = 0;
      __pyx_t_8 = PyList_Sort(__pyx_t_6); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 144, __pyx_L1_error)
      if (unlikely(__pyx_t_6 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
        __PYX_ERR(0, 144, __pyx_L1_error)
      }
      __pyx_t_10 = PyList_AsTuple(__pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 144, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (1) {
        PyObject* sequence = __pyx_t_10;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 144, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_1 = PyTuple_GET_ITEM(sequence, 1); 
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_1);
        #else
        __pyx_t_6 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 144, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_1 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
      __Pyx_XDECREF_SET(__pyx_v_type1, __pyx_t_6);
      __pyx_t_6 = 0;
      __Pyx_XDECREF_SET(__pyx_v_type2, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":145
 *             for tt2 in self.atomic_types:
 *                 type1, type2 = tuple(sorted([tt1, tt2]))
 *                 t1 = type_converter[type1]             # <<<<<<<<<<<<<<
 *                 t2 = type_converter[type2]
 *                 if type1 < type2:
 */
      __pyx_t_10 = __Pyx_PyDict_GetItem(__pyx_v_type_converter, __pyx_v_type1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_XDECREF_SET(__pyx_v_t1, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":146
 *                 type1, type2 = tuple(sorted([tt1, tt2]))
 *                 t1 = type_converter[type1]
 *                 t2 = type_converter[type2]             # <<<<<<<<<<<<<<
 *                 if type1 < type2:
 *                     if self.bondtypes_2body[t1,t2] == -1:
 */
      __pyx_t_10 = __Pyx_PyDict_GetItem(__pyx_v_type_converter, __pyx_v_type2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 146, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_XDECREF_SET(__pyx_v_t2, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":147
 *                 t1 = type_converter[type1]
 *                 t2 = type_converter[type2]
 *                 if type1 < type2:             # <<<<<<<<<<<<<<
 *                     if self.bondtypes_2body[t1,t2] == -1:
 *                         self.bondtypes_2body[t1,t2] = count
 */
      __pyx_t_10 = PyObject_RichCompare(__pyx_v_type1, __pyx_v_type2, Py_LT); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 147, __pyx_L1_error)
      __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 147, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (__pyx_t_13) {

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":148
 *                 t2 = type_converter[type2]
 *                 if type1 < type2:
 *                     if self.bondtypes_2body[t1,t2] == -1:             # <<<<<<<<<<<<<<
 *                         self.bondtypes_2body[t1,t2] = count
 *                         self.Nbondtypes_2body += 1
 */
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_2body); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 148, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_v_t1);
        __Pyx_GIVEREF(__pyx_v_t1);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_t1);
        __Pyx_INCREF(__pyx_v_t2);
        __Pyx_GIVEREF(__pyx_v_t2);
        PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_t2);
        __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_t_10, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 148, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_t_6, __pyx_int_neg_1, -1L, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 148, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_13) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":149
 *                 if type1 < type2:
 *                     if self.bondtypes_2body[t1,t2] == -1:
 *                         self.bondtypes_2body[t1,t2] = count             # <<<<<<<<<<<<<<
 *                         self.Nbondtypes_2body += 1
 *                         count += 1
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_2body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 149, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_INCREF(__pyx_v_t1);
          __Pyx_GIVEREF(__pyx_v_t1);
          PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_t1);
          __Pyx_INCREF(__pyx_v_t2);
          __Pyx_GIVEREF(__pyx_v_t2);
          PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_t2);
          if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_t_6, __pyx_v_count) < 0)) __PYX_ERR(0, 149, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":150
 *                     if self.bondtypes_2body[t1,t2] == -1:
 *                         self.bondtypes_2body[t1,t2] = count
 *                         self.Nbondtypes_2body += 1             # <<<<<<<<<<<<<<
 *                         count += 1
 *                 elif type1 == type2 and (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 */
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_2body); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 150, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_t_6, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_2body, __pyx_t_1) < 0) __PYX_ERR(0, 150, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":151
 *                         self.bondtypes_2body[t1,t2] = count
 *                         self.Nbondtypes_2body += 1
 *                         count += 1             # <<<<<<<<<<<<<<
 *                 elif type1 == type2 and (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                     if self.bondtypes_2body[t1,t2] == -1:
 */
          __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_count, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 151, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF_SET(__pyx_v_count, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":148
 *                 t2 = type_converter[type2]
 *                 if type1 < type2:
 *                     if self.bondtypes_2body[t1,t2] == -1:             # <<<<<<<<<<<<<<
 *                         self.bondtypes_2body[t1,t2] = count
 *                         self.Nbondtypes_2body += 1
 */
        }

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":147
 *                 t1 = type_converter[type1]
 *                 t2 = type_converter[type2]
 *                 if type1 < type2:             # <<<<<<<<<<<<<<
 *                     if self.bondtypes_2body[t1,t2] == -1:
 *                         self.bondtypes_2body[t1,t2] = count
 */
        goto __pyx_L15;
      }

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":152
 *                         self.Nbondtypes_2body += 1
 *                         count += 1
 *                 elif type1 == type2 and (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):             # <<<<<<<<<<<<<<
 *                     if self.bondtypes_2body[t1,t2] == -1:
 *                         self.bondtypes_2body[t1,t2] = count
 */
      __pyx_t_1 = PyObject_RichCompare(__pyx_v_type1, __pyx_v_type2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
      __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__pyx_t_14) {
      } else {
        __pyx_t_13 = __pyx_t_14;
        goto __pyx_L17_bool_binop_done;
      }
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_type1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyObject_RichCompare(__pyx_t_6, __pyx_int_1, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_14) {
      } else {
        __pyx_t_13 = __pyx_t_14;
        goto __pyx_L17_bool_binop_done;
      }
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_pbc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_sum, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyObject_RichCompare(__pyx_t_6, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 152, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_13 = __pyx_t_14;
      __pyx_L17_bool_binop_done:;
      if (__pyx_t_13) {

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":153
 *                         count += 1
 *                 elif type1 == type2 and (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                     if self.bondtypes_2body[t1,t2] == -1:             # <<<<<<<<<<<<<<
 *                         self.bondtypes_2body[t1,t2] = count
 *                         self.Nbondtypes_2body += 1
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_2body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 153, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 153, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_v_t1);
        __Pyx_GIVEREF(__pyx_v_t1);
        PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_t1);
        __Pyx_INCREF(__pyx_v_t2);
        __Pyx_GIVEREF(__pyx_v_t2);
        PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_t2);
        __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 153, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = __Pyx_PyInt_EqObjC(__pyx_t_10, __pyx_int_neg_1, -1L, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 153, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 153, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (__pyx_t_13) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":154
 *                 elif type1 == type2 and (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                     if self.bondtypes_2body[t1,t2] == -1:
 *                         self.bondtypes_2body[t1,t2] = count             # <<<<<<<<<<<<<<
 *                         self.Nbondtypes_2body += 1
 *                         count += 1
 */
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_2body); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 154, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 154, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_INCREF(__pyx_v_t1);
          __Pyx_GIVEREF(__pyx_v_t1);
          PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_t1);
          __Pyx_INCREF(__pyx_v_t2);
          __Pyx_GIVEREF(__pyx_v_t2);
          PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_v_t2);
          if (unlikely(PyObject_SetItem(__pyx_t_6, __pyx_t_10, __pyx_v_count) < 0)) __PYX_ERR(0, 154, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":155
 *                     if self.bondtypes_2body[t1,t2] == -1:
 *                         self.bondtypes_2body[t1,t2] = count
 *                         self.Nbondtypes_2body += 1             # <<<<<<<<<<<<<<
 *                         count += 1
 *         self.bondtypes_2body = self.bondtypes_2body.reshape(-1).tolist()
 */
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_2body); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 155, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_6 = __Pyx_PyInt_AddObjC(__pyx_t_10, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 155, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_2body, __pyx_t_6) < 0) __PYX_ERR(0, 155, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":156
 *                         self.bondtypes_2body[t1,t2] = count
 *                         self.Nbondtypes_2body += 1
 *                         count += 1             # <<<<<<<<<<<<<<
 *         self.bondtypes_2body = self.bondtypes_2body.reshape(-1).tolist()
 * 
 */
          __pyx_t_6 = __Pyx_PyInt_AddObjC(__pyx_v_count, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 156, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF_SET(__pyx_v_count, __pyx_t_6);
          __pyx_t_6 = 0;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":153
 *                         count += 1
 *                 elif type1 == type2 and (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                     if self.bondtypes_2body[t1,t2] == -1:             # <<<<<<<<<<<<<<
 *                         self.bondtypes_2body[t1,t2] = count
 *                         self.Nbondtypes_2body += 1
 */
        }

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":152
 *                         self.Nbondtypes_2body += 1
 *                         count += 1
 *                 elif type1 == type2 and (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):             # <<<<<<<<<<<<<<
 *                     if self.bondtypes_2body[t1,t2] == -1:
 *                         self.bondtypes_2body[t1,t2] = count
 */
      }
      __pyx_L15:;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":143
 *         count = 0
 *         for tt1 in self.atomic_types:
 *             for tt2 in self.atomic_types:             # <<<<<<<<<<<<<<
 *                 type1, type2 = tuple(sorted([tt1, tt2]))
 *                 t1 = type_converter[type1]
 */
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":142
 *         self.bondtypes_2body = -np.ones((self.Ntypes, self.Ntypes)).astype(int)
 *         count = 0
 *         for tt1 in self.atomic_types:             # <<<<<<<<<<<<<<
 *             for tt2 in self.atomic_types:
 *                 type1, type2 = tuple(sorted([tt1, tt2]))
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":157
 *                         self.Nbondtypes_2body += 1
 *                         count += 1
 *         self.bondtypes_2body = self.bondtypes_2body.reshape(-1).tolist()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_2body); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_reshape); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_7 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_6, __pyx_int_neg_1) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_int_neg_1);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_tolist); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_2body, __pyx_t_2) < 0) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":161
 * 
 *         # Bondtypes - 3body
 *         self.bondtypes_3body = -np.ones((self.Ntypes, self.Ntypes, self.Ntypes)).astype(int)             # <<<<<<<<<<<<<<
 *         bondtypes_3body_keys = []
 *         for type1 in self.atomic_types:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_ones); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Ntypes); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Ntypes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Ntypes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_15 = PyTuple_New(3); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_15, 2, __pyx_t_3);
  __pyx_t_7 = 0;
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_10 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_3, __pyx_t_15) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_15);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_astype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_2 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_10, ((PyObject *)(&PyInt_Type))) : __Pyx_PyObject_CallOneArg(__pyx_t_6, ((PyObject *)(&PyInt_Type)));
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyNumber_Negative(__pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_3body, __pyx_t_6) < 0) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":162
 *         # Bondtypes - 3body
 *         self.bondtypes_3body = -np.ones((self.Ntypes, self.Ntypes, self.Ntypes)).astype(int)
 *         bondtypes_3body_keys = []             # <<<<<<<<<<<<<<
 *         for type1 in self.atomic_types:
 *             for type2 in self.atomic_types:
 */
  __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v_bondtypes_3body_keys = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":163
 *         self.bondtypes_3body = -np.ones((self.Ntypes, self.Ntypes, self.Ntypes)).astype(int)
 *         bondtypes_3body_keys = []
 *         for type1 in self.atomic_types:             # <<<<<<<<<<<<<<
 *             for type2 in self.atomic_types:
 *                 if type1 == type2 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 */
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_types); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
    __pyx_t_2 = __pyx_t_6; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 163, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_6); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 163, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_6); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 163, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      }
    } else {
      __pyx_t_6 = __pyx_t_9(__pyx_t_2);
      if (unlikely(!__pyx_t_6)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 163, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_XDECREF_SET(__pyx_v_type1, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":164
 *         bondtypes_3body_keys = []
 *         for type1 in self.atomic_types:
 *             for type2 in self.atomic_types:             # <<<<<<<<<<<<<<
 *                 if type1 == type2 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                     continue
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_types); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
      __pyx_t_10 = __pyx_t_6; __Pyx_INCREF(__pyx_t_10); __pyx_t_11 = 0;
      __pyx_t_12 = NULL;
    } else {
      __pyx_t_11 = -1; __pyx_t_10 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 164, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_12 = Py_TYPE(__pyx_t_10)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 164, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    for (;;) {
      if (likely(!__pyx_t_12)) {
        if (likely(PyList_CheckExact(__pyx_t_10))) {
          if (__pyx_t_11 >= PyList_GET_SIZE(__pyx_t_10)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyList_GET_ITEM(__pyx_t_10, __pyx_t_11); __Pyx_INCREF(__pyx_t_6); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 164, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_10, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 164, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        } else {
          if (__pyx_t_11 >= PyTuple_GET_SIZE(__pyx_t_10)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_10, __pyx_t_11); __Pyx_INCREF(__pyx_t_6); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 164, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_10, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 164, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        }
      } else {
        __pyx_t_6 = __pyx_t_12(__pyx_t_10);
        if (unlikely(!__pyx_t_6)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 164, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_XDECREF_SET(__pyx_v_type2, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":165
 *         for type1 in self.atomic_types:
 *             for type2 in self.atomic_types:
 *                 if type1 == type2 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):             # <<<<<<<<<<<<<<
 *                     continue
 *                 for type3 in self.atomic_types:
 */
      __pyx_t_6 = PyObject_RichCompare(__pyx_v_type1, __pyx_v_type2, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 165, __pyx_L1_error)
      __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (__pyx_t_14) {
      } else {
        __pyx_t_13 = __pyx_t_14;
        goto __pyx_L26_bool_binop_done;
      }
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_count); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_t_6, __pyx_v_type1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyObject_RichCompare(__pyx_t_15, __pyx_int_1, Py_GT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_16 < 0)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (!__pyx_t_16) {
      } else {
        __pyx_t_14 = __pyx_t_16;
        goto __pyx_L28_bool_binop_done;
      }
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_pbc); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_15 = __Pyx_PyObject_CallOneArg(__pyx_builtin_sum, __pyx_t_6); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyObject_RichCompare(__pyx_t_15, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_16 < 0)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_14 = __pyx_t_16;
      __pyx_L28_bool_binop_done:;
      __pyx_t_16 = ((!__pyx_t_14) != 0);
      __pyx_t_13 = __pyx_t_16;
      __pyx_L26_bool_binop_done:;
      if (__pyx_t_13) {

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":166
 *             for type2 in self.atomic_types:
 *                 if type1 == type2 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                     continue             # <<<<<<<<<<<<<<
 *                 for type3 in self.atomic_types:
 *                     if type1 == type3 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 */
        goto __pyx_L23_continue;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":165
 *         for type1 in self.atomic_types:
 *             for type2 in self.atomic_types:
 *                 if type1 == type2 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):             # <<<<<<<<<<<<<<
 *                     continue
 *                 for type3 in self.atomic_types:
 */
      }

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":167
 *                 if type1 == type2 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                     continue
 *                 for type3 in self.atomic_types:             # <<<<<<<<<<<<<<
 *                     if type1 == type3 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                         continue
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_types); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 167, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
        __pyx_t_15 = __pyx_t_6; __Pyx_INCREF(__pyx_t_15); __pyx_t_17 = 0;
        __pyx_t_18 = NULL;
      } else {
        __pyx_t_17 = -1; __pyx_t_15 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 167, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_18 = Py_TYPE(__pyx_t_15)->tp_iternext; if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 167, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      for (;;) {
        if (likely(!__pyx_t_18)) {
          if (likely(PyList_CheckExact(__pyx_t_15))) {
            if (__pyx_t_17 >= PyList_GET_SIZE(__pyx_t_15)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_6 = PyList_GET_ITEM(__pyx_t_15, __pyx_t_17); __Pyx_INCREF(__pyx_t_6); __pyx_t_17++; if (unlikely(0 < 0)) __PYX_ERR(0, 167, __pyx_L1_error)
            #else
            __pyx_t_6 = PySequence_ITEM(__pyx_t_15, __pyx_t_17); __pyx_t_17++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 167, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            #endif
          } else {
            if (__pyx_t_17 >= PyTuple_GET_SIZE(__pyx_t_15)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_15, __pyx_t_17); __Pyx_INCREF(__pyx_t_6); __pyx_t_17++; if (unlikely(0 < 0)) __PYX_ERR(0, 167, __pyx_L1_error)
            #else
            __pyx_t_6 = PySequence_ITEM(__pyx_t_15, __pyx_t_17); __pyx_t_17++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 167, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            #endif
          }
        } else {
          __pyx_t_6 = __pyx_t_18(__pyx_t_15);
          if (unlikely(!__pyx_t_6)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 167, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_XDECREF_SET(__pyx_v_type3, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":168
 *                     continue
 *                 for type3 in self.atomic_types:
 *                     if type1 == type3 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):             # <<<<<<<<<<<<<<
 *                         continue
 *                     key = tuple([type1] + sorted([type2, type3]))
 */
        __pyx_t_6 = PyObject_RichCompare(__pyx_v_type1, __pyx_v_type3, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 168, __pyx_L1_error)
        __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_16 < 0)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (__pyx_t_16) {
        } else {
          __pyx_t_13 = __pyx_t_16;
          goto __pyx_L33_bool_binop_done;
        }
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_count); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_t_6, __pyx_v_type1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = PyObject_RichCompare(__pyx_t_3, __pyx_int_1, Py_GT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (!__pyx_t_14) {
        } else {
          __pyx_t_16 = __pyx_t_14;
          goto __pyx_L35_bool_binop_done;
        }
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_pbc); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_sum, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = PyObject_RichCompare(__pyx_t_3, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_16 = __pyx_t_14;
        __pyx_L35_bool_binop_done:;
        __pyx_t_14 = ((!__pyx_t_16) != 0);
        __pyx_t_13 = __pyx_t_14;
        __pyx_L33_bool_binop_done:;
        if (__pyx_t_13) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":169
 *                 for type3 in self.atomic_types:
 *                     if type1 == type3 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                         continue             # <<<<<<<<<<<<<<
 *                     key = tuple([type1] + sorted([type2, type3]))
 *                     if type2 < type3:
 */
          goto __pyx_L30_continue;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":168
 *                     continue
 *                 for type3 in self.atomic_types:
 *                     if type1 == type3 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):             # <<<<<<<<<<<<<<
 *                         continue
 *                     key = tuple([type1] + sorted([type2, type3]))
 */
        }

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":170
 *                     if type1 == type3 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                         continue
 *                     key = tuple([type1] + sorted([type2, type3]))             # <<<<<<<<<<<<<<
 *                     if type2 < type3:
 *                         bondtypes_3body_keys.append(key)
 */
        __pyx_t_6 = PyList_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_v_type1);
        __Pyx_GIVEREF(__pyx_v_type1);
        PyList_SET_ITEM(__pyx_t_6, 0, __pyx_v_type1);
        __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_v_type2);
        __Pyx_GIVEREF(__pyx_v_type2);
        PyList_SET_ITEM(__pyx_t_1, 0, __pyx_v_type2);
        __Pyx_INCREF(__pyx_v_type3);
        __Pyx_GIVEREF(__pyx_v_type3);
        PyList_SET_ITEM(__pyx_t_1, 1, __pyx_v_type3);
        __pyx_t_3 = ((PyObject*)__pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_8 = PyList_Sort(__pyx_t_3); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 170, __pyx_L1_error)
        __pyx_t_1 = PyNumber_Add(__pyx_t_6, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = PyList_AsTuple(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_3);
        __pyx_t_3 = 0;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":171
 *                         continue
 *                     key = tuple([type1] + sorted([type2, type3]))
 *                     if type2 < type3:             # <<<<<<<<<<<<<<
 *                         bondtypes_3body_keys.append(key)
 *                     elif type2 == type3:
 */
        __pyx_t_3 = PyObject_RichCompare(__pyx_v_type2, __pyx_v_type3, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
        __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 171, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (__pyx_t_13) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":172
 *                     key = tuple([type1] + sorted([type2, type3]))
 *                     if type2 < type3:
 *                         bondtypes_3body_keys.append(key)             # <<<<<<<<<<<<<<
 *                     elif type2 == type3:
 *                         if type2 == type1 and (self.atomic_count[type1] > 2 or sum(self.pbc) > 0):
 */
          __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_bondtypes_3body_keys, __pyx_v_key); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 172, __pyx_L1_error)

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":171
 *                         continue
 *                     key = tuple([type1] + sorted([type2, type3]))
 *                     if type2 < type3:             # <<<<<<<<<<<<<<
 *                         bondtypes_3body_keys.append(key)
 *                     elif type2 == type3:
 */
          goto __pyx_L37;
        }

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":173
 *                     if type2 < type3:
 *                         bondtypes_3body_keys.append(key)
 *                     elif type2 == type3:             # <<<<<<<<<<<<<<
 *                         if type2 == type1 and (self.atomic_count[type1] > 2 or sum(self.pbc) > 0):
 *                             bondtypes_3body_keys.append(key)
 */
        __pyx_t_3 = PyObject_RichCompare(__pyx_v_type2, __pyx_v_type3, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 173, __pyx_L1_error)
        __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 173, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (__pyx_t_13) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":174
 *                         bondtypes_3body_keys.append(key)
 *                     elif type2 == type3:
 *                         if type2 == type1 and (self.atomic_count[type1] > 2 or sum(self.pbc) > 0):             # <<<<<<<<<<<<<<
 *                             bondtypes_3body_keys.append(key)
 *                         elif type2 != type1 and (self.atomic_count[type2] > 1 or sum(self.pbc) > 0):
 */
          __pyx_t_3 = PyObject_RichCompare(__pyx_v_type2, __pyx_v_type1, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 174, __pyx_L1_error)
          __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 174, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (__pyx_t_14) {
          } else {
            __pyx_t_13 = __pyx_t_14;
            goto __pyx_L39_bool_binop_done;
          }
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 174, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_3, __pyx_v_type1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 174, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_int_2, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 174, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 174, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (!__pyx_t_14) {
          } else {
            __pyx_t_13 = __pyx_t_14;
            goto __pyx_L39_bool_binop_done;
          }
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_pbc); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 174, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_sum, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 174, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 174, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 174, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_13 = __pyx_t_14;
          __pyx_L39_bool_binop_done:;
          if (__pyx_t_13) {

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":175
 *                     elif type2 == type3:
 *                         if type2 == type1 and (self.atomic_count[type1] > 2 or sum(self.pbc) > 0):
 *                             bondtypes_3body_keys.append(key)             # <<<<<<<<<<<<<<
 *                         elif type2 != type1 and (self.atomic_count[type2] > 1 or sum(self.pbc) > 0):
 *                             bondtypes_3body_keys.append(key)
 */
            __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_bondtypes_3body_keys, __pyx_v_key); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 175, __pyx_L1_error)

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":174
 *                         bondtypes_3body_keys.append(key)
 *                     elif type2 == type3:
 *                         if type2 == type1 and (self.atomic_count[type1] > 2 or sum(self.pbc) > 0):             # <<<<<<<<<<<<<<
 *                             bondtypes_3body_keys.append(key)
 *                         elif type2 != type1 and (self.atomic_count[type2] > 1 or sum(self.pbc) > 0):
 */
            goto __pyx_L38;
          }

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":176
 *                         if type2 == type1 and (self.atomic_count[type1] > 2 or sum(self.pbc) > 0):
 *                             bondtypes_3body_keys.append(key)
 *                         elif type2 != type1 and (self.atomic_count[type2] > 1 or sum(self.pbc) > 0):             # <<<<<<<<<<<<<<
 *                             bondtypes_3body_keys.append(key)
 *         self.Nbondtypes_3body = len(bondtypes_3body_keys)
 */
          __pyx_t_3 = PyObject_RichCompare(__pyx_v_type2, __pyx_v_type1, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L1_error)
          __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 176, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (__pyx_t_14) {
          } else {
            __pyx_t_13 = __pyx_t_14;
            goto __pyx_L42_bool_binop_done;
          }
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_atomic_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_3, __pyx_v_type2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_int_1, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 176, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (!__pyx_t_14) {
          } else {
            __pyx_t_13 = __pyx_t_14;
            goto __pyx_L42_bool_binop_done;
          }
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_pbc); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_sum, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 176, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_13 = __pyx_t_14;
          __pyx_L42_bool_binop_done:;
          if (__pyx_t_13) {

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":177
 *                             bondtypes_3body_keys.append(key)
 *                         elif type2 != type1 and (self.atomic_count[type2] > 1 or sum(self.pbc) > 0):
 *                             bondtypes_3body_keys.append(key)             # <<<<<<<<<<<<<<
 *         self.Nbondtypes_3body = len(bondtypes_3body_keys)
 *         for i, key in enumerate(bondtypes_3body_keys):
 */
            __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_bondtypes_3body_keys, __pyx_v_key); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 177, __pyx_L1_error)

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":176
 *                         if type2 == type1 and (self.atomic_count[type1] > 2 or sum(self.pbc) > 0):
 *                             bondtypes_3body_keys.append(key)
 *                         elif type2 != type1 and (self.atomic_count[type2] > 1 or sum(self.pbc) > 0):             # <<<<<<<<<<<<<<
 *                             bondtypes_3body_keys.append(key)
 *         self.Nbondtypes_3body = len(bondtypes_3body_keys)
 */
          }
          __pyx_L38:;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":173
 *                     if type2 < type3:
 *                         bondtypes_3body_keys.append(key)
 *                     elif type2 == type3:             # <<<<<<<<<<<<<<
 *                         if type2 == type1 and (self.atomic_count[type1] > 2 or sum(self.pbc) > 0):
 *                             bondtypes_3body_keys.append(key)
 */
        }
        __pyx_L37:;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":167
 *                 if type1 == type2 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                     continue
 *                 for type3 in self.atomic_types:             # <<<<<<<<<<<<<<
 *                     if type1 == type3 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                         continue
 */
        __pyx_L30_continue:;
      }
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":164
 *         bondtypes_3body_keys = []
 *         for type1 in self.atomic_types:
 *             for type2 in self.atomic_types:             # <<<<<<<<<<<<<<
 *                 if type1 == type2 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 *                     continue
 */
      __pyx_L23_continue:;
    }
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":163
 *         self.bondtypes_3body = -np.ones((self.Ntypes, self.Ntypes, self.Ntypes)).astype(int)
 *         bondtypes_3body_keys = []
 *         for type1 in self.atomic_types:             # <<<<<<<<<<<<<<
 *             for type2 in self.atomic_types:
 *                 if type1 == type2 and not (self.atomic_count[type1] > 1 or sum(self.pbc) > 0):
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":178
 *                         elif type2 != type1 and (self.atomic_count[type2] > 1 or sum(self.pbc) > 0):
 *                             bondtypes_3body_keys.append(key)
 *         self.Nbondtypes_3body = len(bondtypes_3body_keys)             # <<<<<<<<<<<<<<
 *         for i, key in enumerate(bondtypes_3body_keys):
 *             k1, k2, k3 = key
 */
  __pyx_t_4 = PyList_GET_SIZE(__pyx_v_bondtypes_3body_keys); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 178, __pyx_L1_error)
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_3body, __pyx_t_2) < 0) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":179
 *                             bondtypes_3body_keys.append(key)
 *         self.Nbondtypes_3body = len(bondtypes_3body_keys)
 *         for i, key in enumerate(bondtypes_3body_keys):             # <<<<<<<<<<<<<<
 *             k1, k2, k3 = key
 *             self.bondtypes_3body[type_converter[k1],type_converter[k2],type_converter[k3]] = i
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_t_2 = __pyx_int_0;
  __pyx_t_10 = __pyx_v_bondtypes_3body_keys; __Pyx_INCREF(__pyx_t_10); __pyx_t_4 = 0;
  for (;;) {
    if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_10)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_15 = PyList_GET_ITEM(__pyx_t_10, __pyx_t_4); __Pyx_INCREF(__pyx_t_15); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 179, __pyx_L1_error)
    #else
    __pyx_t_15 = PySequence_ITEM(__pyx_t_10, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_15);
    __pyx_t_15 = 0;
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_2);
    __pyx_t_15 = __Pyx_PyInt_AddObjC(__pyx_t_2, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_2);
    __pyx_t_2 = __pyx_t_15;
    __pyx_t_15 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":180
 *         self.Nbondtypes_3body = len(bondtypes_3body_keys)
 *         for i, key in enumerate(bondtypes_3body_keys):
 *             k1, k2, k3 = key             # <<<<<<<<<<<<<<
 *             self.bondtypes_3body[type_converter[k1],type_converter[k2],type_converter[k3]] = i
 *         self.bondtypes_3body = self.bondtypes_3body.reshape(-1).tolist()
 */
    if ((likely(PyTuple_CheckExact(__pyx_v_key))) || (PyList_CheckExact(__pyx_v_key))) {
      PyObject* sequence = __pyx_v_key;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 180, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_15 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_1 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_15 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_1 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_15);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      #else
      __pyx_t_15 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #endif
    } else {
      Py_ssize_t index = -1;
      __pyx_t_6 = PyObject_GetIter(__pyx_v_key); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_19 = Py_TYPE(__pyx_t_6)->tp_iternext;
      index = 0; __pyx_t_15 = __pyx_t_19(__pyx_t_6); if (unlikely(!__pyx_t_15)) goto __pyx_L47_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_15);
      index = 1; __pyx_t_3 = __pyx_t_19(__pyx_t_6); if (unlikely(!__pyx_t_3)) goto __pyx_L47_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 2; __pyx_t_1 = __pyx_t_19(__pyx_t_6); if (unlikely(!__pyx_t_1)) goto __pyx_L47_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_6), 3) < 0) __PYX_ERR(0, 180, __pyx_L1_error)
      __pyx_t_19 = NULL;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L48_unpacking_done;
      __pyx_L47_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_19 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 180, __pyx_L1_error)
      __pyx_L48_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_k1, __pyx_t_15);
    __pyx_t_15 = 0;
    __Pyx_XDECREF_SET(__pyx_v_k2, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_k3, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":181
 *         for i, key in enumerate(bondtypes_3body_keys):
 *             k1, k2, k3 = key
 *             self.bondtypes_3body[type_converter[k1],type_converter[k2],type_converter[k3]] = i             # <<<<<<<<<<<<<<
 *         self.bondtypes_3body = self.bondtypes_3body.reshape(-1).tolist()
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_3body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_type_converter, __pyx_v_k1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_15 = __Pyx_PyDict_GetItem(__pyx_v_type_converter, __pyx_v_k2); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_v_type_converter, __pyx_v_k3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_15);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_15);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_6);
    __pyx_t_3 = 0;
    __pyx_t_15 = 0;
    __pyx_t_6 = 0;
    if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_t_7, __pyx_v_i) < 0)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":179
 *                             bondtypes_3body_keys.append(key)
 *         self.Nbondtypes_3body = len(bondtypes_3body_keys)
 *         for i, key in enumerate(bondtypes_3body_keys):             # <<<<<<<<<<<<<<
 *             k1, k2, k3 = key
 *             self.bondtypes_3body[type_converter[k1],type_converter[k2],type_converter[k3]] = i
 */
  }
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":182
 *             k1, k2, k3 = key
 *             self.bondtypes_3body[type_converter[k1],type_converter[k2],type_converter[k3]] = i
 *         self.bondtypes_3body = self.bondtypes_3body.reshape(-1).tolist()             # <<<<<<<<<<<<<<
 * 
 *         self.Nelements_2body = self.Nbondtypes_2body * self.Nbins1
 */
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_3body); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_reshape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_10 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_7, __pyx_int_neg_1) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_int_neg_1);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_tolist); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_3body, __pyx_t_2) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":184
 *         self.bondtypes_3body = self.bondtypes_3body.reshape(-1).tolist()
 * 
 *         self.Nelements_2body = self.Nbondtypes_2body * self.Nbins1             # <<<<<<<<<<<<<<
 *         self.Nelements_3body = self.Nbondtypes_3body * self.Nbins2
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_2body); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbins1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = PyNumber_Multiply(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Nelements_2body, __pyx_t_10) < 0) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":185
 * 
 *         self.Nelements_2body = self.Nbondtypes_2body * self.Nbins1
 *         self.Nelements_3body = self.Nbondtypes_3body * self.Nbins2             # <<<<<<<<<<<<<<
 * 
 *         if use_angular:
 */
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_3body); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbins2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_10, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Nelements_3body, __pyx_t_2) < 0) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":187
 *         self.Nelements_3body = self.Nbondtypes_3body * self.Nbins2
 * 
 *         if use_angular:             # <<<<<<<<<<<<<<
 *             self.Nelements = self.Nelements_2body + self.Nelements_3body
 *         else:
 */
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_v_use_angular); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 187, __pyx_L1_error)
  if (__pyx_t_13) {

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":188
 * 
 *         if use_angular:
 *             self.Nelements = self.Nelements_2body + self.Nelements_3body             # <<<<<<<<<<<<<<
 *         else:
 *             self.Nelements = self.Nelements_2body
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nelements_2body); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nelements_3body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = PyNumber_Add(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Nelements, __pyx_t_10) < 0) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":187
 *         self.Nelements_3body = self.Nbondtypes_3body * self.Nbins2
 * 
 *         if use_angular:             # <<<<<<<<<<<<<<
 *             self.Nelements = self.Nelements_2body + self.Nelements_3body
 *         else:
 */
    goto __pyx_L49;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":190
 *             self.Nelements = self.Nelements_2body + self.Nelements_3body
 *         else:
 *             self.Nelements = self.Nelements_2body             # <<<<<<<<<<<<<<
 * 
 *         # Get relevant neighbour unit-cells
 */
  /*else*/ {
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nelements_2body); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_Nelements, __pyx_t_10) < 0) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __pyx_L49:;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":193
 * 
 *         # Get relevant neighbour unit-cells
 *         self.pbc = atoms.get_pbc()             # <<<<<<<<<<<<<<
 *         self.cell = atoms.get_cell()
 *         self.neighbourcells_disp = self.__get_neighbour_cells_displacement(self.pbc, self.cell)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_pbc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_10 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_pbc, __pyx_t_10) < 0) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":194
 *         # Get relevant neighbour unit-cells
 *         self.pbc = atoms.get_pbc()
 *         self.cell = atoms.get_cell()             # <<<<<<<<<<<<<<
 *         self.neighbourcells_disp = self.__get_neighbour_cells_displacement(self.pbc, self.cell)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_cell); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_10 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_cell, __pyx_t_10) < 0) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":195
 *         self.pbc = atoms.get_pbc()
 *         self.cell = atoms.get_cell()
 *         self.neighbourcells_disp = self.__get_neighbour_cells_displacement(self.pbc, self.cell)             # <<<<<<<<<<<<<<
 * 
 *     def get_feature(self, atoms):
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Angular_Fingerprint__get_neighb); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_pbc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_cell); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = NULL;
  __pyx_t_20 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_20 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_2, __pyx_t_7};
    __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_20, 2+__pyx_t_20); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_2, __pyx_t_7};
    __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_20, 2+__pyx_t_20); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_15 = PyTuple_New(2+__pyx_t_20); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_20, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_20, __pyx_t_7);
    __pyx_t_2 = 0;
    __pyx_t_7 = 0;
    __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_15, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_neighbourcells_disp, __pyx_t_10) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":97
 *     """ comparator for construction of angular fingerprints
 *     """
 *     def __init__(self, atoms, Rc1=4.0, Rc2=4.0, binwidth1=0.1, Nbins2=30, sigma1=0.2, sigma2=0.10, nsigma=4, eta=1, gamma=3, use_angular=True):             # <<<<<<<<<<<<<<
 *         """ Set a common cut of radius
 *         """
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_type_converter);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_type);
  __Pyx_XDECREF(__pyx_v_count);
  __Pyx_XDECREF(__pyx_v_tt1);
  __Pyx_XDECREF(__pyx_v_tt2);
  __Pyx_XDECREF(__pyx_v_type1);
  __Pyx_XDECREF(__pyx_v_type2);
  __Pyx_XDECREF(__pyx_v_t1);
  __Pyx_XDECREF(__pyx_v_t2);
  __Pyx_XDECREF(__pyx_v_bondtypes_3body_keys);
  __Pyx_XDECREF(__pyx_v_type3);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_k1);
  __Pyx_XDECREF(__pyx_v_k2);
  __Pyx_XDECREF(__pyx_v_k3);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_type);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":197
 *         self.neighbourcells_disp = self.__get_neighbour_cells_displacement(self.pbc, self.cell)
 * 
 *     def get_feature(self, atoms):             # <<<<<<<<<<<<<<
 *         """
 *         """
 */

/* Python wrapper */
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_3get_feature(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_2get_feature[] = "\n        ";
static PyMethodDef __pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_3get_feature = {"get_feature", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_3get_feature, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_2get_feature};
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_3get_feature(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_atoms = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_feature (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_atoms,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_atoms)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_feature", 1, 2, 2, 1); __PYX_ERR(0, 197, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_feature") < 0)) __PYX_ERR(0, 197, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_atoms = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_feature", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 197, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.get_feature", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_2get_feature(__pyx_self, __pyx_v_self, __pyx_v_atoms);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_2get_feature(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_atoms) {
  double __pyx_v_Rc1;
  double __pyx_v_Rc2;
  double __pyx_v_binwidth1;
  double __pyx_v_binwidth2;
  int __pyx_v_Nbins1;
  int __pyx_v_Nbins2;
  double __pyx_v_sigma1;
  double __pyx_v_sigma2;
  int __pyx_v_nsigma;
  double __pyx_v_eta;
  double __pyx_v_gamma;
  PyObject *__pyx_v_use_angular = 0;
  double __pyx_v_volume;
  CYTHON_UNUSED int __pyx_v_dim;
  double __pyx_v_m1;
  double __pyx_v_m2;
  double __pyx_v_smearing_norm1;
  double __pyx_v_smearing_norm2;
  int __pyx_v_Nelements_2body;
  int __pyx_v_Nelements_3body;
  int __pyx_v_Nelements;
  struct __pyx_obj_5cymem_5cymem_Pool *__pyx_v_mem = 0;
  CYTHON_UNUSED PyObject *__pyx_v_cell = NULL;
  int __pyx_v_Natoms;
  PyObject *__pyx_v_pos_np = 0;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point *__pyx_v_pos;
  int __pyx_v_m;
  int __pyx_v_Ncells;
  PyObject *__pyx_v_cell_displacements_old = 0;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point *__pyx_v_cell_displacements;
  int __pyx_v_Ntypes;
  CYTHON_UNUSED int __pyx_v_Nbondtypes_2body;
  PyObject *__pyx_v_bondtypes_2body_old = 0;
  int *__pyx_v_bondtypes_2body;
  PyObject *__pyx_v_num_converted_old = 0;
  int *__pyx_v_num_converted;
  double *__pyx_v_feature1;
  int __pyx_v_num_pairs;
  int __pyx_v_center_bin;
  int __pyx_v_minbin_lim;
  int __pyx_v_maxbin_lim;
  int __pyx_v_newbin;
  int __pyx_v_bondtype_index;
  int __pyx_v_type1;
  int __pyx_v_type2;
  double __pyx_v_Rij;
  double __pyx_v_normalization;
  double __pyx_v_binpos;
  double __pyx_v_c;
  double __pyx_v_erfarg_low;
  double __pyx_v_erfarg_up;
  double __pyx_v_value;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_n;
  int __pyx_v_cell_index;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_displacement;
  PyObject *__pyx_v_feature1_np = NULL;
  CYTHON_UNUSED int __pyx_v_Nbondtypes_3body;
  PyObject *__pyx_v_bondtypes_3body_old = 0;
  int *__pyx_v_bondtypes_3body;
  double *__pyx_v_feature2;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_RijVec;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_RikVec;
  double __pyx_v_angle;
  int __pyx_v_k;
  int __pyx_v_type3;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_pos_i;
  int __pyx_v_cell_index1;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_displacement1;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_pos_j;
  int __pyx_v_cell_index2;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_displacement2;
  long __pyx_v_k_start;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_pos_k;
  double __pyx_v_Rik;
  PyObject *__pyx_v_feature2_np = NULL;
  PyObject *__pyx_v_feature_np = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  void *__pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  int __pyx_t_18;
  int __pyx_t_19;
  double __pyx_t_20;
  long __pyx_t_21;
  long __pyx_t_22;
  int __pyx_t_23;
  double __pyx_t_24;
  int __pyx_t_25;
  int __pyx_t_26;
  int __pyx_t_27;
  int __pyx_t_28;
  int __pyx_t_29;
  int __pyx_t_30;
  int __pyx_t_31;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_feature", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":200
 *         """
 *         """
 *         cdef double Rc1 = self.Rc1             # <<<<<<<<<<<<<<
 *         cdef double Rc2 = self.Rc2
 *         cdef double binwidth1 = self.binwidth1
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Rc1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Rc1 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":201
 *         """
 *         cdef double Rc1 = self.Rc1
 *         cdef double Rc2 = self.Rc2             # <<<<<<<<<<<<<<
 *         cdef double binwidth1 = self.binwidth1
 *         cdef double binwidth2 = self.binwidth2
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Rc2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Rc2 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":202
 *         cdef double Rc1 = self.Rc1
 *         cdef double Rc2 = self.Rc2
 *         cdef double binwidth1 = self.binwidth1             # <<<<<<<<<<<<<<
 *         cdef double binwidth2 = self.binwidth2
 *         cdef int Nbins1 = self.Nbins1
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_binwidth1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_binwidth1 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":203
 *         cdef double Rc2 = self.Rc2
 *         cdef double binwidth1 = self.binwidth1
 *         cdef double binwidth2 = self.binwidth2             # <<<<<<<<<<<<<<
 *         cdef int Nbins1 = self.Nbins1
 *         cdef int Nbins2 = self.Nbins2
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_binwidth2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_binwidth2 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":204
 *         cdef double binwidth1 = self.binwidth1
 *         cdef double binwidth2 = self.binwidth2
 *         cdef int Nbins1 = self.Nbins1             # <<<<<<<<<<<<<<
 *         cdef int Nbins2 = self.Nbins2
 *         cdef double sigma1 = self.sigma1
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbins1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nbins1 = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":205
 *         cdef double binwidth2 = self.binwidth2
 *         cdef int Nbins1 = self.Nbins1
 *         cdef int Nbins2 = self.Nbins2             # <<<<<<<<<<<<<<
 *         cdef double sigma1 = self.sigma1
 *         cdef double sigma2 = self.sigma2
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbins2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nbins2 = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":206
 *         cdef int Nbins1 = self.Nbins1
 *         cdef int Nbins2 = self.Nbins2
 *         cdef double sigma1 = self.sigma1             # <<<<<<<<<<<<<<
 *         cdef double sigma2 = self.sigma2
 *         cdef int nsigma = self.nsigma
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sigma1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_sigma1 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":207
 *         cdef int Nbins2 = self.Nbins2
 *         cdef double sigma1 = self.sigma1
 *         cdef double sigma2 = self.sigma2             # <<<<<<<<<<<<<<
 *         cdef int nsigma = self.nsigma
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sigma2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_sigma2 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":208
 *         cdef double sigma1 = self.sigma1
 *         cdef double sigma2 = self.sigma2
 *         cdef int nsigma = self.nsigma             # <<<<<<<<<<<<<<
 * 
 *         cdef double eta = self.eta
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_nsigma); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_nsigma = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":210
 *         cdef int nsigma = self.nsigma
 * 
 *         cdef double eta = self.eta             # <<<<<<<<<<<<<<
 *         cdef double gamma = self.gamma
 *         cdef use_angular = self.use_angular
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_eta); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_eta = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":211
 * 
 *         cdef double eta = self.eta
 *         cdef double gamma = self.gamma             # <<<<<<<<<<<<<<
 *         cdef use_angular = self.use_angular
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_gamma); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_gamma = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":212
 *         cdef double eta = self.eta
 *         cdef double gamma = self.gamma
 *         cdef use_angular = self.use_angular             # <<<<<<<<<<<<<<
 * 
 *         cdef double volume = self.volume
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_use_angular); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_use_angular = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":214
 *         cdef use_angular = self.use_angular
 * 
 *         cdef double volume = self.volume             # <<<<<<<<<<<<<<
 *         cdef int dim = self.dim
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_volume); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_volume = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":215
 * 
 *         cdef double volume = self.volume
 *         cdef int dim = self.dim             # <<<<<<<<<<<<<<
 * 
 *         cdef double m1 = self.m1
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_dim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dim = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":217
 *         cdef int dim = self.dim
 * 
 *         cdef double m1 = self.m1             # <<<<<<<<<<<<<<
 *         cdef double m2 = self.m2
 *         cdef double smearing_norm1 = self.smearing_norm1
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_m1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_m1 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":218
 * 
 *         cdef double m1 = self.m1
 *         cdef double m2 = self.m2             # <<<<<<<<<<<<<<
 *         cdef double smearing_norm1 = self.smearing_norm1
 *         cdef double smearing_norm2 = self.smearing_norm2
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_m2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 218, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_m2 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":219
 *         cdef double m1 = self.m1
 *         cdef double m2 = self.m2
 *         cdef double smearing_norm1 = self.smearing_norm1             # <<<<<<<<<<<<<<
 *         cdef double smearing_norm2 = self.smearing_norm2
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_smearing_norm1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_smearing_norm1 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":220
 *         cdef double m2 = self.m2
 *         cdef double smearing_norm1 = self.smearing_norm1
 *         cdef double smearing_norm2 = self.smearing_norm2             # <<<<<<<<<<<<<<
 * 
 *         cdef int Nelements_2body = self.Nelements_2body
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_smearing_norm2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_smearing_norm2 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":222
 *         cdef double smearing_norm2 = self.smearing_norm2
 * 
 *         cdef int Nelements_2body = self.Nelements_2body             # <<<<<<<<<<<<<<
 *         cdef int Nelements_3body = self.Nelements_3body
 *         cdef int Nelements = self.Nelements
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nelements_2body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nelements_2body = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":223
 * 
 *         cdef int Nelements_2body = self.Nelements_2body
 *         cdef int Nelements_3body = self.Nelements_3body             # <<<<<<<<<<<<<<
 *         cdef int Nelements = self.Nelements
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nelements_3body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nelements_3body = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":224
 *         cdef int Nelements_2body = self.Nelements_2body
 *         cdef int Nelements_3body = self.Nelements_3body
 *         cdef int Nelements = self.Nelements             # <<<<<<<<<<<<<<
 * 
 *         # Memory allocation pool
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nelements); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nelements = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":228
 *         # Memory allocation pool
 *         cdef Pool mem
 *         mem = Pool()             # <<<<<<<<<<<<<<
 * 
 *         cell = atoms.get_cell()
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5cymem_5cymem_Pool)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_mem = ((struct __pyx_obj_5cymem_5cymem_Pool *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":230
 *         mem = Pool()
 * 
 *         cell = atoms.get_cell()             # <<<<<<<<<<<<<<
 *         cdef int Natoms = self.Natoms
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_cell); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_cell = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":231
 * 
 *         cell = atoms.get_cell()
 *         cdef int Natoms = self.Natoms             # <<<<<<<<<<<<<<
 * 
 *         # Get positions and convert to Point-struct
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Natoms); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Natoms = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":234
 * 
 *         # Get positions and convert to Point-struct
 *         cdef list pos_np = atoms.get_positions().tolist()             # <<<<<<<<<<<<<<
 *         cdef Point *pos
 *         pos = <Point*>mem.alloc(Natoms, sizeof(Point))
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_positions); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_tolist); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 234, __pyx_L1_error)
  __pyx_v_pos_np = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":236
 *         cdef list pos_np = atoms.get_positions().tolist()
 *         cdef Point *pos
 *         pos = <Point*>mem.alloc(Natoms, sizeof(Point))             # <<<<<<<<<<<<<<
 *         cdef int m
 *         for m in range(Natoms):
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, __pyx_v_Natoms, (sizeof(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 236, __pyx_L1_error)
  __pyx_v_pos = ((__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":238
 *         pos = <Point*>mem.alloc(Natoms, sizeof(Point))
 *         cdef int m
 *         for m in range(Natoms):             # <<<<<<<<<<<<<<
 *             pos[m].coord[0] = pos_np[m][0]
 *             pos[m].coord[1] = pos_np[m][1]
 */
  __pyx_t_3 = __pyx_v_Natoms;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":239
 *         cdef int m
 *         for m in range(Natoms):
 *             pos[m].coord[0] = pos_np[m][0]             # <<<<<<<<<<<<<<
 *             pos[m].coord[1] = pos_np[m][1]
 *             pos[m].coord[2] = pos_np[m][2]
 */
    if (unlikely(__pyx_v_pos_np == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 239, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_pos_np, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 239, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 239, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    ((__pyx_v_pos[__pyx_v_m]).coord[0]) = __pyx_t_2;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":240
 *         for m in range(Natoms):
 *             pos[m].coord[0] = pos_np[m][0]
 *             pos[m].coord[1] = pos_np[m][1]             # <<<<<<<<<<<<<<
 *             pos[m].coord[2] = pos_np[m][2]
 * 
 */
    if (unlikely(__pyx_v_pos_np == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 240, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_pos_np, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_5, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    ((__pyx_v_pos[__pyx_v_m]).coord[1]) = __pyx_t_2;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":241
 *             pos[m].coord[0] = pos_np[m][0]
 *             pos[m].coord[1] = pos_np[m][1]
 *             pos[m].coord[2] = pos_np[m][2]             # <<<<<<<<<<<<<<
 * 
 *         # Get neighbourcells and convert to Point-struct
 */
    if (unlikely(__pyx_v_pos_np == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 241, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_pos_np, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 241, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_1, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 241, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 241, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    ((__pyx_v_pos[__pyx_v_m]).coord[2]) = __pyx_t_2;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":244
 * 
 *         # Get neighbourcells and convert to Point-struct
 *         cdef int Ncells = len(self.neighbourcells_disp)             # <<<<<<<<<<<<<<
 *         cdef list cell_displacements_old = self.neighbourcells_disp
 *         cdef Point *cell_displacements
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_neighbourcells_disp); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = PyObject_Length(__pyx_t_5); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_Ncells = __pyx_t_10;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":245
 *         # Get neighbourcells and convert to Point-struct
 *         cdef int Ncells = len(self.neighbourcells_disp)
 *         cdef list cell_displacements_old = self.neighbourcells_disp             # <<<<<<<<<<<<<<
 *         cdef Point *cell_displacements
 *         cell_displacements = <Point*>mem.alloc(Ncells, sizeof(Point))
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_neighbourcells_disp); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (!(likely(PyList_CheckExact(__pyx_t_5))||((__pyx_t_5) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_5)->tp_name), 0))) __PYX_ERR(0, 245, __pyx_L1_error)
  __pyx_v_cell_displacements_old = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":247
 *         cdef list cell_displacements_old = self.neighbourcells_disp
 *         cdef Point *cell_displacements
 *         cell_displacements = <Point*>mem.alloc(Ncells, sizeof(Point))             # <<<<<<<<<<<<<<
 *         for m in range(Ncells):
 *             cell_displacements[m].coord[0] = cell_displacements_old[m][0]
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, __pyx_v_Ncells, (sizeof(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 247, __pyx_L1_error)
  __pyx_v_cell_displacements = ((__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":248
 *         cdef Point *cell_displacements
 *         cell_displacements = <Point*>mem.alloc(Ncells, sizeof(Point))
 *         for m in range(Ncells):             # <<<<<<<<<<<<<<
 *             cell_displacements[m].coord[0] = cell_displacements_old[m][0]
 *             cell_displacements[m].coord[1] = cell_displacements_old[m][1]
 */
  __pyx_t_3 = __pyx_v_Ncells;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":249
 *         cell_displacements = <Point*>mem.alloc(Ncells, sizeof(Point))
 *         for m in range(Ncells):
 *             cell_displacements[m].coord[0] = cell_displacements_old[m][0]             # <<<<<<<<<<<<<<
 *             cell_displacements[m].coord[1] = cell_displacements_old[m][1]
 *             cell_displacements[m].coord[2] = cell_displacements_old[m][2]
 */
    if (unlikely(__pyx_v_cell_displacements_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 249, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_cell_displacements_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_5, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 249, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    ((__pyx_v_cell_displacements[__pyx_v_m]).coord[0]) = __pyx_t_2;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":250
 *         for m in range(Ncells):
 *             cell_displacements[m].coord[0] = cell_displacements_old[m][0]
 *             cell_displacements[m].coord[1] = cell_displacements_old[m][1]             # <<<<<<<<<<<<<<
 *             cell_displacements[m].coord[2] = cell_displacements_old[m][2]
 * 
 */
    if (unlikely(__pyx_v_cell_displacements_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 250, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_cell_displacements_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    ((__pyx_v_cell_displacements[__pyx_v_m]).coord[1]) = __pyx_t_2;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":251
 *             cell_displacements[m].coord[0] = cell_displacements_old[m][0]
 *             cell_displacements[m].coord[1] = cell_displacements_old[m][1]
 *             cell_displacements[m].coord[2] = cell_displacements_old[m][2]             # <<<<<<<<<<<<<<
 * 
 *         # Convert 2body bondtype list into c-array
 */
    if (unlikely(__pyx_v_cell_displacements_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 251, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_cell_displacements_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_5, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 251, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    ((__pyx_v_cell_displacements[__pyx_v_m]).coord[2]) = __pyx_t_2;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":254
 * 
 *         # Convert 2body bondtype list into c-array
 *         cdef int Ntypes = self.Ntypes             # <<<<<<<<<<<<<<
 *         cdef int Nbondtypes_2body = self.Nbondtypes_2body
 *         cdef list bondtypes_2body_old = self.bondtypes_2body
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Ntypes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Ntypes = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":255
 *         # Convert 2body bondtype list into c-array
 *         cdef int Ntypes = self.Ntypes
 *         cdef int Nbondtypes_2body = self.Nbondtypes_2body             # <<<<<<<<<<<<<<
 *         cdef list bondtypes_2body_old = self.bondtypes_2body
 *         cdef int *bondtypes_2body
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_2body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nbondtypes_2body = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":256
 *         cdef int Ntypes = self.Ntypes
 *         cdef int Nbondtypes_2body = self.Nbondtypes_2body
 *         cdef list bondtypes_2body_old = self.bondtypes_2body             # <<<<<<<<<<<<<<
 *         cdef int *bondtypes_2body
 *         bondtypes_2body = <int*>mem.alloc(Ntypes*Ntypes, sizeof(int))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_2body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 256, __pyx_L1_error)
  __pyx_v_bondtypes_2body_old = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":258
 *         cdef list bondtypes_2body_old = self.bondtypes_2body
 *         cdef int *bondtypes_2body
 *         bondtypes_2body = <int*>mem.alloc(Ntypes*Ntypes, sizeof(int))             # <<<<<<<<<<<<<<
 *         for m in range(Ntypes*Ntypes):
 *             bondtypes_2body[m] = bondtypes_2body_old[m]
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, (__pyx_v_Ntypes * __pyx_v_Ntypes), (sizeof(int))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 258, __pyx_L1_error)
  __pyx_v_bondtypes_2body = ((int *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":259
 *         cdef int *bondtypes_2body
 *         bondtypes_2body = <int*>mem.alloc(Ntypes*Ntypes, sizeof(int))
 *         for m in range(Ntypes*Ntypes):             # <<<<<<<<<<<<<<
 *             bondtypes_2body[m] = bondtypes_2body_old[m]
 * 
 */
  __pyx_t_3 = (__pyx_v_Ntypes * __pyx_v_Ntypes);
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":260
 *         bondtypes_2body = <int*>mem.alloc(Ntypes*Ntypes, sizeof(int))
 *         for m in range(Ntypes*Ntypes):
 *             bondtypes_2body[m] = bondtypes_2body_old[m]             # <<<<<<<<<<<<<<
 * 
 *         # Get converted atom Ntypes
 */
    if (unlikely(__pyx_v_bondtypes_2body_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 260, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_bondtypes_2body_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    (__pyx_v_bondtypes_2body[__pyx_v_m]) = __pyx_t_11;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":263
 * 
 *         # Get converted atom Ntypes
 *         cdef list num_converted_old = convert_atom_types(atoms.get_atomic_numbers())             # <<<<<<<<<<<<<<
 *         cdef int *num_converted
 *         num_converted = <int*>mem.alloc(Natoms, sizeof(int))
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_convert_atom_types); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_atomic_numbers); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_12)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_4 = (__pyx_t_12) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 263, __pyx_L1_error)
  __pyx_v_num_converted_old = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":265
 *         cdef list num_converted_old = convert_atom_types(atoms.get_atomic_numbers())
 *         cdef int *num_converted
 *         num_converted = <int*>mem.alloc(Natoms, sizeof(int))             # <<<<<<<<<<<<<<
 *         for m in range(Natoms):
 *             num_converted[m] = num_converted_old[m]
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, __pyx_v_Natoms, (sizeof(int))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 265, __pyx_L1_error)
  __pyx_v_num_converted = ((int *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":266
 *         cdef int *num_converted
 *         num_converted = <int*>mem.alloc(Natoms, sizeof(int))
 *         for m in range(Natoms):             # <<<<<<<<<<<<<<
 *             num_converted[m] = num_converted_old[m]
 * 
 */
  __pyx_t_3 = __pyx_v_Natoms;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":267
 *         num_converted = <int*>mem.alloc(Natoms, sizeof(int))
 *         for m in range(Natoms):
 *             num_converted[m] = num_converted_old[m]             # <<<<<<<<<<<<<<
 * 
 *         # RADIAL FEATURE
 */
    if (unlikely(__pyx_v_num_converted_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 267, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_num_converted_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    (__pyx_v_num_converted[__pyx_v_m]) = __pyx_t_11;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":273
 *         # Initialize radial feature
 *         cdef double *feature1
 *         feature1 = <double*>mem.alloc(Nelements_2body, sizeof(double))             # <<<<<<<<<<<<<<
 * 
 *         cdef int num_pairs, center_bin, minbin_lim, maxbin_lim, newbin, bondtype_index, type1, type2
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, __pyx_v_Nelements_2body, (sizeof(double))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 273, __pyx_L1_error)
  __pyx_v_feature1 = ((double *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":278
 *         cdef double Rij, normalization, binpos, c, erfarg_low, erfarg_up, value
 *         cdef int i, j, n
 *         for i in range(Natoms):             # <<<<<<<<<<<<<<
 *             for cell_index in range(Ncells):
 *                 displacement = cell_displacements[cell_index]
 */
  __pyx_t_3 = __pyx_v_Natoms;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":279
 *         cdef int i, j, n
 *         for i in range(Natoms):
 *             for cell_index in range(Ncells):             # <<<<<<<<<<<<<<
 *                 displacement = cell_displacements[cell_index]
 *                 for j in range(Natoms):
 */
    __pyx_t_11 = __pyx_v_Ncells;
    __pyx_t_13 = __pyx_t_11;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_cell_index = __pyx_t_14;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":280
 *         for i in range(Natoms):
 *             for cell_index in range(Ncells):
 *                 displacement = cell_displacements[cell_index]             # <<<<<<<<<<<<<<
 *                 for j in range(Natoms):
 *                     Rij = euclidean(pos[i], add(pos[j], displacement))
 */
      __pyx_v_displacement = (__pyx_v_cell_displacements[__pyx_v_cell_index]);

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":281
 *             for cell_index in range(Ncells):
 *                 displacement = cell_displacements[cell_index]
 *                 for j in range(Natoms):             # <<<<<<<<<<<<<<
 *                     Rij = euclidean(pos[i], add(pos[j], displacement))
 * 
 */
      __pyx_t_15 = __pyx_v_Natoms;
      __pyx_t_16 = __pyx_t_15;
      for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
        __pyx_v_j = __pyx_t_17;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":282
 *                 displacement = cell_displacements[cell_index]
 *                 for j in range(Natoms):
 *                     Rij = euclidean(pos[i], add(pos[j], displacement))             # <<<<<<<<<<<<<<
 * 
 *                     # Stop if distance too long or atoms are the same one.
 */
        __pyx_v_Rij = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_euclidean((__pyx_v_pos[__pyx_v_i]), __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_add((__pyx_v_pos[__pyx_v_j]), __pyx_v_displacement));

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":285
 * 
 *                     # Stop if distance too long or atoms are the same one.
 *                     if Rij > Rc1+nsigma*sigma1 or Rij < 1e-6:             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
        __pyx_t_19 = ((__pyx_v_Rij > (__pyx_v_Rc1 + (__pyx_v_nsigma * __pyx_v_sigma1))) != 0);
        if (!__pyx_t_19) {
        } else {
          __pyx_t_18 = __pyx_t_19;
          goto __pyx_L18_bool_binop_done;
        }
        __pyx_t_19 = ((__pyx_v_Rij < 1e-6) != 0);
        __pyx_t_18 = __pyx_t_19;
        __pyx_L18_bool_binop_done:;
        if (__pyx_t_18) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":286
 *                     # Stop if distance too long or atoms are the same one.
 *                     if Rij > Rc1+nsigma*sigma1 or Rij < 1e-6:
 *                         continue             # <<<<<<<<<<<<<<
 * 
 *                     # determine bondtype
 */
          goto __pyx_L15_continue;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":285
 * 
 *                     # Stop if distance too long or atoms are the same one.
 *                     if Rij > Rc1+nsigma*sigma1 or Rij < 1e-6:             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
        }

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":289
 * 
 *                     # determine bondtype
 *                     if num_converted[i] <= num_converted[j]:             # <<<<<<<<<<<<<<
 *                         type1 = num_converted[i]
 *                         type2 = num_converted[j]
 */
        __pyx_t_18 = (((__pyx_v_num_converted[__pyx_v_i]) <= (__pyx_v_num_converted[__pyx_v_j])) != 0);
        if (__pyx_t_18) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":290
 *                     # determine bondtype
 *                     if num_converted[i] <= num_converted[j]:
 *                         type1 = num_converted[i]             # <<<<<<<<<<<<<<
 *                         type2 = num_converted[j]
 *                     else:
 */
          __pyx_v_type1 = (__pyx_v_num_converted[__pyx_v_i]);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":291
 *                     if num_converted[i] <= num_converted[j]:
 *                         type1 = num_converted[i]
 *                         type2 = num_converted[j]             # <<<<<<<<<<<<<<
 *                     else:
 *                         type1 = num_converted[j]
 */
          __pyx_v_type2 = (__pyx_v_num_converted[__pyx_v_j]);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":289
 * 
 *                     # determine bondtype
 *                     if num_converted[i] <= num_converted[j]:             # <<<<<<<<<<<<<<
 *                         type1 = num_converted[i]
 *                         type2 = num_converted[j]
 */
          goto __pyx_L20;
        }

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":293
 *                         type2 = num_converted[j]
 *                     else:
 *                         type1 = num_converted[j]             # <<<<<<<<<<<<<<
 *                         type2 = num_converted[i]
 *                     bondtype_index = Nbins1*bondtypes_2body[Ntypes*type1+type2]
 */
        /*else*/ {
          __pyx_v_type1 = (__pyx_v_num_converted[__pyx_v_j]);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":294
 *                     else:
 *                         type1 = num_converted[j]
 *                         type2 = num_converted[i]             # <<<<<<<<<<<<<<
 *                     bondtype_index = Nbins1*bondtypes_2body[Ntypes*type1+type2]
 * 
 */
          __pyx_v_type2 = (__pyx_v_num_converted[__pyx_v_i]);
        }
        __pyx_L20:;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":295
 *                         type1 = num_converted[j]
 *                         type2 = num_converted[i]
 *                     bondtype_index = Nbins1*bondtypes_2body[Ntypes*type1+type2]             # <<<<<<<<<<<<<<
 * 
 *                     # Calculate normalization
 */
        __pyx_v_bondtype_index = (__pyx_v_Nbins1 * (__pyx_v_bondtypes_2body[((__pyx_v_Ntypes * __pyx_v_type1) + __pyx_v_type2)]));

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":298
 * 
 *                     # Calculate normalization
 *                     num_pairs = Natoms*Natoms             # <<<<<<<<<<<<<<
 *                     normalization = 1./smearing_norm1
 *                     normalization /= 4*M_PI*Rij*Rij * binwidth1 * num_pairs/volume
 */
        __pyx_v_num_pairs = (__pyx_v_Natoms * __pyx_v_Natoms);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":299
 *                     # Calculate normalization
 *                     num_pairs = Natoms*Natoms
 *                     normalization = 1./smearing_norm1             # <<<<<<<<<<<<<<
 *                     normalization /= 4*M_PI*Rij*Rij * binwidth1 * num_pairs/volume
 * 
 */
        if (unlikely(__pyx_v_smearing_norm1 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 299, __pyx_L1_error)
        }
        __pyx_v_normalization = (1. / __pyx_v_smearing_norm1);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":300
 *                     num_pairs = Natoms*Natoms
 *                     normalization = 1./smearing_norm1
 *                     normalization /= 4*M_PI*Rij*Rij * binwidth1 * num_pairs/volume             # <<<<<<<<<<<<<<
 * 
 *                     # Identify what bin 'Rij' belongs to + it's position in this bin
 */
        __pyx_t_2 = (((((4.0 * M_PI) * __pyx_v_Rij) * __pyx_v_Rij) * __pyx_v_binwidth1) * __pyx_v_num_pairs);
        if (unlikely(__pyx_v_volume == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 300, __pyx_L1_error)
        }
        __pyx_t_20 = (__pyx_t_2 / __pyx_v_volume);
        if (unlikely(__pyx_t_20 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 300, __pyx_L1_error)
        }
        __pyx_v_normalization = (__pyx_v_normalization / __pyx_t_20);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":303
 * 
 *                     # Identify what bin 'Rij' belongs to + it's position in this bin
 *                     center_bin = <int> floor(Rij/binwidth1)             # <<<<<<<<<<<<<<
 *                     binpos = Rij/binwidth1 - center_bin
 * 
 */
        if (unlikely(__pyx_v_binwidth1 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 303, __pyx_L1_error)
        }
        __pyx_v_center_bin = ((int)floor((__pyx_v_Rij / __pyx_v_binwidth1)));

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":304
 *                     # Identify what bin 'Rij' belongs to + it's position in this bin
 *                     center_bin = <int> floor(Rij/binwidth1)
 *                     binpos = Rij/binwidth1 - center_bin             # <<<<<<<<<<<<<<
 * 
 *                     # Lower and upper range of bins affected by the current atomic distance deltaR.
 */
        if (unlikely(__pyx_v_binwidth1 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 304, __pyx_L1_error)
        }
        __pyx_v_binpos = ((__pyx_v_Rij / __pyx_v_binwidth1) - __pyx_v_center_bin);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":307
 * 
 *                     # Lower and upper range of bins affected by the current atomic distance deltaR.
 *                     minbin_lim = <int> -ceil(m1 - binpos)             # <<<<<<<<<<<<<<
 *                     maxbin_lim = <int> ceil(m1 - (1-binpos))
 * 
 */
        __pyx_v_minbin_lim = ((int)(-ceil((__pyx_v_m1 - __pyx_v_binpos))));

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":308
 *                     # Lower and upper range of bins affected by the current atomic distance deltaR.
 *                     minbin_lim = <int> -ceil(m1 - binpos)
 *                     maxbin_lim = <int> ceil(m1 - (1-binpos))             # <<<<<<<<<<<<<<
 * 
 *                     for n in range(minbin_lim, maxbin_lim + 1):
 */
        __pyx_v_maxbin_lim = ((int)ceil((__pyx_v_m1 - (1.0 - __pyx_v_binpos))));

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":310
 *                     maxbin_lim = <int> ceil(m1 - (1-binpos))
 * 
 *                     for n in range(minbin_lim, maxbin_lim + 1):             # <<<<<<<<<<<<<<
 *                         newbin = center_bin + n
 *                         if newbin < 0 or newbin >= Nbins1:
 */
        __pyx_t_21 = (__pyx_v_maxbin_lim + 1);
        __pyx_t_22 = __pyx_t_21;
        for (__pyx_t_23 = __pyx_v_minbin_lim; __pyx_t_23 < __pyx_t_22; __pyx_t_23+=1) {
          __pyx_v_n = __pyx_t_23;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":311
 * 
 *                     for n in range(minbin_lim, maxbin_lim + 1):
 *                         newbin = center_bin + n             # <<<<<<<<<<<<<<
 *                         if newbin < 0 or newbin >= Nbins1:
 *                             continue
 */
          __pyx_v_newbin = (__pyx_v_center_bin + __pyx_v_n);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":312
 *                     for n in range(minbin_lim, maxbin_lim + 1):
 *                         newbin = center_bin + n
 *                         if newbin < 0 or newbin >= Nbins1:             # <<<<<<<<<<<<<<
 *                             continue
 * 
 */
          __pyx_t_19 = ((__pyx_v_newbin < 0) != 0);
          if (!__pyx_t_19) {
          } else {
            __pyx_t_18 = __pyx_t_19;
            goto __pyx_L24_bool_binop_done;
          }
          __pyx_t_19 = ((__pyx_v_newbin >= __pyx_v_Nbins1) != 0);
          __pyx_t_18 = __pyx_t_19;
          __pyx_L24_bool_binop_done:;
          if (__pyx_t_18) {

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":313
 *                         newbin = center_bin + n
 *                         if newbin < 0 or newbin >= Nbins1:
 *                             continue             # <<<<<<<<<<<<<<
 * 
 *                         # Calculate gauss contribution to current bin
 */
            goto __pyx_L21_continue;

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":312
 *                     for n in range(minbin_lim, maxbin_lim + 1):
 *                         newbin = center_bin + n
 *                         if newbin < 0 or newbin >= Nbins1:             # <<<<<<<<<<<<<<
 *                             continue
 * 
 */
          }

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":316
 * 
 *                         # Calculate gauss contribution to current bin
 *                         c = 1./sqrt(2)*binwidth1/sigma1             # <<<<<<<<<<<<<<
 *                         erfarg_low = max(-m1, n-binpos)
 *                         erfarg_up = min(m1, n+(1-binpos))
 */
          __pyx_t_20 = sqrt(2.0);
          if (unlikely(__pyx_t_20 == 0)) {
            PyErr_SetString(PyExc_ZeroDivisionError, "float division");
            __PYX_ERR(0, 316, __pyx_L1_error)
          }
          __pyx_t_2 = ((1. / __pyx_t_20) * __pyx_v_binwidth1);
          if (unlikely(__pyx_v_sigma1 == 0)) {
            PyErr_SetString(PyExc_ZeroDivisionError, "float division");
            __PYX_ERR(0, 316, __pyx_L1_error)
          }
          __pyx_v_c = (__pyx_t_2 / __pyx_v_sigma1);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":317
 *                         # Calculate gauss contribution to current bin
 *                         c = 1./sqrt(2)*binwidth1/sigma1
 *                         erfarg_low = max(-m1, n-binpos)             # <<<<<<<<<<<<<<
 *                         erfarg_up = min(m1, n+(1-binpos))
 *                         value = 0.5*erf(c*erfarg_up)-0.5*erf(c*erfarg_low)
 */
          __pyx_t_2 = (__pyx_v_n - __pyx_v_binpos);
          __pyx_t_20 = (-__pyx_v_m1);
          if (((__pyx_t_2 > __pyx_t_20) != 0)) {
            __pyx_t_24 = __pyx_t_2;
          } else {
            __pyx_t_24 = __pyx_t_20;
          }
          __pyx_v_erfarg_low = __pyx_t_24;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":318
 *                         c = 1./sqrt(2)*binwidth1/sigma1
 *                         erfarg_low = max(-m1, n-binpos)
 *                         erfarg_up = min(m1, n+(1-binpos))             # <<<<<<<<<<<<<<
 *                         value = 0.5*erf(c*erfarg_up)-0.5*erf(c*erfarg_low)
 * 
 */
          __pyx_t_24 = (__pyx_v_n + (1.0 - __pyx_v_binpos));
          __pyx_t_2 = __pyx_v_m1;
          if (((__pyx_t_24 < __pyx_t_2) != 0)) {
            __pyx_t_20 = __pyx_t_24;
          } else {
            __pyx_t_20 = __pyx_t_2;
          }
          __pyx_v_erfarg_up = __pyx_t_20;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":319
 *                         erfarg_low = max(-m1, n-binpos)
 *                         erfarg_up = min(m1, n+(1-binpos))
 *                         value = 0.5*erf(c*erfarg_up)-0.5*erf(c*erfarg_low)             # <<<<<<<<<<<<<<
 * 
 *                         # Apply normalization
 */
          __pyx_v_value = ((0.5 * erf((__pyx_v_c * __pyx_v_erfarg_up))) - (0.5 * erf((__pyx_v_c * __pyx_v_erfarg_low))));

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":322
 * 
 *                         # Apply normalization
 *                         value *= normalization             # <<<<<<<<<<<<<<
 * 
 *                         feature1[bondtype_index + newbin] += value
 */
          __pyx_v_value = (__pyx_v_value * __pyx_v_normalization);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":324
 *                         value *= normalization
 * 
 *                         feature1[bondtype_index + newbin] += value             # <<<<<<<<<<<<<<
 *                         #feature1[newbin] += value
 * 
 */
          __pyx_t_25 = (__pyx_v_bondtype_index + __pyx_v_newbin);
          (__pyx_v_feature1[__pyx_t_25]) = ((__pyx_v_feature1[__pyx_t_25]) + __pyx_v_value);
          __pyx_L21_continue:;
        }
        __pyx_L15_continue:;
      }
    }
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":328
 * 
 *         # Convert radial feature to numpy array
 *         feature1_np = np.zeros(Nelements_2body)             # <<<<<<<<<<<<<<
 *         for m in range(Nelements_2body):
 *             feature1_np[m] = feature1[m]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_Nelements_2body); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_feature1_np = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":329
 *         # Convert radial feature to numpy array
 *         feature1_np = np.zeros(Nelements_2body)
 *         for m in range(Nelements_2body):             # <<<<<<<<<<<<<<
 *             feature1_np[m] = feature1[m]
 * 
 */
  __pyx_t_3 = __pyx_v_Nelements_2body;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":330
 *         feature1_np = np.zeros(Nelements_2body)
 *         for m in range(Nelements_2body):
 *             feature1_np[m] = feature1[m]             # <<<<<<<<<<<<<<
 * 
 *         # Return feature if only radial part is desired
 */
    __pyx_t_1 = PyFloat_FromDouble((__pyx_v_feature1[__pyx_v_m])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(__Pyx_SetItemInt(__pyx_v_feature1_np, __pyx_v_m, __pyx_t_1, int, 1, __Pyx_PyInt_From_int, 0, 1, 1) < 0)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":333
 * 
 *         # Return feature if only radial part is desired
 *         if not use_angular:             # <<<<<<<<<<<<<<
 *             return feature1_np
 * 
 */
  __pyx_t_18 = __Pyx_PyObject_IsTrue(__pyx_v_use_angular); if (unlikely(__pyx_t_18 < 0)) __PYX_ERR(0, 333, __pyx_L1_error)
  __pyx_t_19 = ((!__pyx_t_18) != 0);
  if (__pyx_t_19) {

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":334
 *         # Return feature if only radial part is desired
 *         if not use_angular:
 *             return feature1_np             # <<<<<<<<<<<<<<
 * 
 *         # ANGULAR FEATURE
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_feature1_np);
    __pyx_r = __pyx_v_feature1_np;
    goto __pyx_L0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":333
 * 
 *         # Return feature if only radial part is desired
 *         if not use_angular:             # <<<<<<<<<<<<<<
 *             return feature1_np
 * 
 */
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":339
 * 
 *         # Convert 3body bondtype list into c-array
 *         cdef int Nbondtypes_3body = self.Nbondtypes_3body             # <<<<<<<<<<<<<<
 *         cdef list bondtypes_3body_old = self.bondtypes_3body
 *         cdef int *bondtypes_3body
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_3body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 339, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nbondtypes_3body = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":340
 *         # Convert 3body bondtype list into c-array
 *         cdef int Nbondtypes_3body = self.Nbondtypes_3body
 *         cdef list bondtypes_3body_old = self.bondtypes_3body             # <<<<<<<<<<<<<<
 *         cdef int *bondtypes_3body
 *         bondtypes_3body = <int*>mem.alloc(Ntypes*Ntypes*Ntypes, sizeof(int))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_3body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 340, __pyx_L1_error)
  __pyx_v_bondtypes_3body_old = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":342
 *         cdef list bondtypes_3body_old = self.bondtypes_3body
 *         cdef int *bondtypes_3body
 *         bondtypes_3body = <int*>mem.alloc(Ntypes*Ntypes*Ntypes, sizeof(int))             # <<<<<<<<<<<<<<
 *         for m in range(Ntypes*Ntypes*Ntypes):
 *             bondtypes_3body[m] = bondtypes_3body_old[m]
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, ((__pyx_v_Ntypes * __pyx_v_Ntypes) * __pyx_v_Ntypes), (sizeof(int))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 342, __pyx_L1_error)
  __pyx_v_bondtypes_3body = ((int *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":343
 *         cdef int *bondtypes_3body
 *         bondtypes_3body = <int*>mem.alloc(Ntypes*Ntypes*Ntypes, sizeof(int))
 *         for m in range(Ntypes*Ntypes*Ntypes):             # <<<<<<<<<<<<<<
 *             bondtypes_3body[m] = bondtypes_3body_old[m]
 * 
 */
  __pyx_t_3 = ((__pyx_v_Ntypes * __pyx_v_Ntypes) * __pyx_v_Ntypes);
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":344
 *         bondtypes_3body = <int*>mem.alloc(Ntypes*Ntypes*Ntypes, sizeof(int))
 *         for m in range(Ntypes*Ntypes*Ntypes):
 *             bondtypes_3body[m] = bondtypes_3body_old[m]             # <<<<<<<<<<<<<<
 * 
 *         # Initialize angular feature
 */
    if (unlikely(__pyx_v_bondtypes_3body_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 344, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_bondtypes_3body_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 344, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 344, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    (__pyx_v_bondtypes_3body[__pyx_v_m]) = __pyx_t_11;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":348
 *         # Initialize angular feature
 *         cdef double *feature2
 *         feature2 = <double*>mem.alloc(Nelements_3body, sizeof(double))             # <<<<<<<<<<<<<<
 * 
 *         cdef Point RijVec, RikVec
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, __pyx_v_Nelements_3body, (sizeof(double))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 348, __pyx_L1_error)
  __pyx_v_feature2 = ((double *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":353
 *         cdef double angle
 *         cdef int k, cond_ij, cond_ik, type3
 *         for i in range(Natoms):             # <<<<<<<<<<<<<<
 *             pos_i = pos[i]
 *             for cell_index1 in range(Ncells):
 */
  __pyx_t_3 = __pyx_v_Natoms;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":354
 *         cdef int k, cond_ij, cond_ik, type3
 *         for i in range(Natoms):
 *             pos_i = pos[i]             # <<<<<<<<<<<<<<
 *             for cell_index1 in range(Ncells):
 *                 displacement1 = cell_displacements[cell_index1]
 */
    __pyx_v_pos_i = (__pyx_v_pos[__pyx_v_i]);

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":355
 *         for i in range(Natoms):
 *             pos_i = pos[i]
 *             for cell_index1 in range(Ncells):             # <<<<<<<<<<<<<<
 *                 displacement1 = cell_displacements[cell_index1]
 *                 for j in range(Natoms):
 */
    __pyx_t_11 = __pyx_v_Ncells;
    __pyx_t_13 = __pyx_t_11;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_cell_index1 = __pyx_t_14;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":356
 *             pos_i = pos[i]
 *             for cell_index1 in range(Ncells):
 *                 displacement1 = cell_displacements[cell_index1]             # <<<<<<<<<<<<<<
 *                 for j in range(Natoms):
 *                     pos_j = add(pos[j], displacement1)
 */
      __pyx_v_displacement1 = (__pyx_v_cell_displacements[__pyx_v_cell_index1]);

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":357
 *             for cell_index1 in range(Ncells):
 *                 displacement1 = cell_displacements[cell_index1]
 *                 for j in range(Natoms):             # <<<<<<<<<<<<<<
 *                     pos_j = add(pos[j], displacement1)
 *                     Rij = euclidean(pos[i], pos_j)
 */
      __pyx_t_15 = __pyx_v_Natoms;
      __pyx_t_16 = __pyx_t_15;
      for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
        __pyx_v_j = __pyx_t_17;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":358
 *                 displacement1 = cell_displacements[cell_index1]
 *                 for j in range(Natoms):
 *                     pos_j = add(pos[j], displacement1)             # <<<<<<<<<<<<<<
 *                     Rij = euclidean(pos[i], pos_j)
 *                     if Rij > Rc2 or Rij < 1e-6:
 */
        __pyx_v_pos_j = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_add((__pyx_v_pos[__pyx_v_j]), __pyx_v_displacement1);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":359
 *                 for j in range(Natoms):
 *                     pos_j = add(pos[j], displacement1)
 *                     Rij = euclidean(pos[i], pos_j)             # <<<<<<<<<<<<<<
 *                     if Rij > Rc2 or Rij < 1e-6:
 *                         continue
 */
        __pyx_v_Rij = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_euclidean((__pyx_v_pos[__pyx_v_i]), __pyx_v_pos_j);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":360
 *                     pos_j = add(pos[j], displacement1)
 *                     Rij = euclidean(pos[i], pos_j)
 *                     if Rij > Rc2 or Rij < 1e-6:             # <<<<<<<<<<<<<<
 *                         continue
 *                     for cell_index2 in range(cell_index1, Ncells):
 */
        __pyx_t_18 = ((__pyx_v_Rij > __pyx_v_Rc2) != 0);
        if (!__pyx_t_18) {
        } else {
          __pyx_t_19 = __pyx_t_18;
          goto __pyx_L38_bool_binop_done;
        }
        __pyx_t_18 = ((__pyx_v_Rij < 1e-6) != 0);
        __pyx_t_19 = __pyx_t_18;
        __pyx_L38_bool_binop_done:;
        if (__pyx_t_19) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":361
 *                     Rij = euclidean(pos[i], pos_j)
 *                     if Rij > Rc2 or Rij < 1e-6:
 *                         continue             # <<<<<<<<<<<<<<
 *                     for cell_index2 in range(cell_index1, Ncells):
 *                         displacement2 = cell_displacements[cell_index2]
 */
          goto __pyx_L35_continue;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":360
 *                     pos_j = add(pos[j], displacement1)
 *                     Rij = euclidean(pos[i], pos_j)
 *                     if Rij > Rc2 or Rij < 1e-6:             # <<<<<<<<<<<<<<
 *                         continue
 *                     for cell_index2 in range(cell_index1, Ncells):
 */
        }

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":362
 *                     if Rij > Rc2 or Rij < 1e-6:
 *                         continue
 *                     for cell_index2 in range(cell_index1, Ncells):             # <<<<<<<<<<<<<<
 *                         displacement2 = cell_displacements[cell_index2]
 *                         if cell_index1 == cell_index2:
 */
        __pyx_t_23 = __pyx_v_Ncells;
        __pyx_t_25 = __pyx_t_23;
        for (__pyx_t_26 = __pyx_v_cell_index1; __pyx_t_26 < __pyx_t_25; __pyx_t_26+=1) {
          __pyx_v_cell_index2 = __pyx_t_26;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":363
 *                         continue
 *                     for cell_index2 in range(cell_index1, Ncells):
 *                         displacement2 = cell_displacements[cell_index2]             # <<<<<<<<<<<<<<
 *                         if cell_index1 == cell_index2:
 *                             k_start = j+1
 */
          __pyx_v_displacement2 = (__pyx_v_cell_displacements[__pyx_v_cell_index2]);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":364
 *                     for cell_index2 in range(cell_index1, Ncells):
 *                         displacement2 = cell_displacements[cell_index2]
 *                         if cell_index1 == cell_index2:             # <<<<<<<<<<<<<<
 *                             k_start = j+1
 *                         else:
 */
          __pyx_t_19 = ((__pyx_v_cell_index1 == __pyx_v_cell_index2) != 0);
          if (__pyx_t_19) {

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":365
 *                         displacement2 = cell_displacements[cell_index2]
 *                         if cell_index1 == cell_index2:
 *                             k_start = j+1             # <<<<<<<<<<<<<<
 *                         else:
 *                             k_start = 0
 */
            __pyx_v_k_start = (__pyx_v_j + 1);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":364
 *                     for cell_index2 in range(cell_index1, Ncells):
 *                         displacement2 = cell_displacements[cell_index2]
 *                         if cell_index1 == cell_index2:             # <<<<<<<<<<<<<<
 *                             k_start = j+1
 *                         else:
 */
            goto __pyx_L42;
          }

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":367
 *                             k_start = j+1
 *                         else:
 *                             k_start = 0             # <<<<<<<<<<<<<<
 *                         for k in range(k_start, Natoms):
 *                             pos_k = add(pos[k], displacement2)
 */
          /*else*/ {
            __pyx_v_k_start = 0;
          }
          __pyx_L42:;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":368
 *                         else:
 *                             k_start = 0
 *                         for k in range(k_start, Natoms):             # <<<<<<<<<<<<<<
 *                             pos_k = add(pos[k], displacement2)
 *                             Rik = euclidean(pos_i, pos_k)
 */
          __pyx_t_27 = __pyx_v_Natoms;
          __pyx_t_28 = __pyx_t_27;
          for (__pyx_t_29 = __pyx_v_k_start; __pyx_t_29 < __pyx_t_28; __pyx_t_29+=1) {
            __pyx_v_k = __pyx_t_29;

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":369
 *                             k_start = 0
 *                         for k in range(k_start, Natoms):
 *                             pos_k = add(pos[k], displacement2)             # <<<<<<<<<<<<<<
 *                             Rik = euclidean(pos_i, pos_k)
 *                             if Rik > Rc2 or Rik < 1e-6:
 */
            __pyx_v_pos_k = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_add((__pyx_v_pos[__pyx_v_k]), __pyx_v_displacement2);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":370
 *                         for k in range(k_start, Natoms):
 *                             pos_k = add(pos[k], displacement2)
 *                             Rik = euclidean(pos_i, pos_k)             # <<<<<<<<<<<<<<
 *                             if Rik > Rc2 or Rik < 1e-6:
 *                                 continue
 */
            __pyx_v_Rik = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_euclidean(__pyx_v_pos_i, __pyx_v_pos_k);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":371
 *                             pos_k = add(pos[k], displacement2)
 *                             Rik = euclidean(pos_i, pos_k)
 *                             if Rik > Rc2 or Rik < 1e-6:             # <<<<<<<<<<<<<<
 *                                 continue
 * 
 */
            __pyx_t_18 = ((__pyx_v_Rik > __pyx_v_Rc2) != 0);
            if (!__pyx_t_18) {
            } else {
              __pyx_t_19 = __pyx_t_18;
              goto __pyx_L46_bool_binop_done;
            }
            __pyx_t_18 = ((__pyx_v_Rik < 1e-6) != 0);
            __pyx_t_19 = __pyx_t_18;
            __pyx_L46_bool_binop_done:;
            if (__pyx_t_19) {

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":372
 *                             Rik = euclidean(pos_i, pos_k)
 *                             if Rik > Rc2 or Rik < 1e-6:
 *                                 continue             # <<<<<<<<<<<<<<
 * 
 *                             # determine bondtype
 */
              goto __pyx_L43_continue;

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":371
 *                             pos_k = add(pos[k], displacement2)
 *                             Rik = euclidean(pos_i, pos_k)
 *                             if Rik > Rc2 or Rik < 1e-6:             # <<<<<<<<<<<<<<
 *                                 continue
 * 
 */
            }

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":375
 * 
 *                             # determine bondtype
 *                             type1 = num_converted[i]             # <<<<<<<<<<<<<<
 *                             if num_converted[j] <= num_converted[k]:
 *                                 type2 = num_converted[j]
 */
            __pyx_v_type1 = (__pyx_v_num_converted[__pyx_v_i]);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":376
 *                             # determine bondtype
 *                             type1 = num_converted[i]
 *                             if num_converted[j] <= num_converted[k]:             # <<<<<<<<<<<<<<
 *                                 type2 = num_converted[j]
 *                                 type3 = num_converted[k]
 */
            __pyx_t_19 = (((__pyx_v_num_converted[__pyx_v_j]) <= (__pyx_v_num_converted[__pyx_v_k])) != 0);
            if (__pyx_t_19) {

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":377
 *                             type1 = num_converted[i]
 *                             if num_converted[j] <= num_converted[k]:
 *                                 type2 = num_converted[j]             # <<<<<<<<<<<<<<
 *                                 type3 = num_converted[k]
 *                             else:
 */
              __pyx_v_type2 = (__pyx_v_num_converted[__pyx_v_j]);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":378
 *                             if num_converted[j] <= num_converted[k]:
 *                                 type2 = num_converted[j]
 *                                 type3 = num_converted[k]             # <<<<<<<<<<<<<<
 *                             else:
 *                                 type2 = num_converted[k]
 */
              __pyx_v_type3 = (__pyx_v_num_converted[__pyx_v_k]);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":376
 *                             # determine bondtype
 *                             type1 = num_converted[i]
 *                             if num_converted[j] <= num_converted[k]:             # <<<<<<<<<<<<<<
 *                                 type2 = num_converted[j]
 *                                 type3 = num_converted[k]
 */
              goto __pyx_L48;
            }

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":380
 *                                 type3 = num_converted[k]
 *                             else:
 *                                 type2 = num_converted[k]             # <<<<<<<<<<<<<<
 *                                 type3 = num_converted[j]
 *                             bondtype_index = Nbins2*bondtypes_3body[Ntypes*Ntypes*type1 + Ntypes*type2 + type3]
 */
            /*else*/ {
              __pyx_v_type2 = (__pyx_v_num_converted[__pyx_v_k]);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":381
 *                             else:
 *                                 type2 = num_converted[k]
 *                                 type3 = num_converted[j]             # <<<<<<<<<<<<<<
 *                             bondtype_index = Nbins2*bondtypes_3body[Ntypes*Ntypes*type1 + Ntypes*type2 + type3]
 * 
 */
              __pyx_v_type3 = (__pyx_v_num_converted[__pyx_v_j]);
            }
            __pyx_L48:;

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":382
 *                                 type2 = num_converted[k]
 *                                 type3 = num_converted[j]
 *                             bondtype_index = Nbins2*bondtypes_3body[Ntypes*Ntypes*type1 + Ntypes*type2 + type3]             # <<<<<<<<<<<<<<
 * 
 *                             # Calculate angle
 */
            __pyx_v_bondtype_index = (__pyx_v_Nbins2 * (__pyx_v_bondtypes_3body[((((__pyx_v_Ntypes * __pyx_v_Ntypes) * __pyx_v_type1) + (__pyx_v_Ntypes * __pyx_v_type2)) + __pyx_v_type3)]));

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":385
 * 
 *                             # Calculate angle
 *                             RijVec = subtract(pos_j,pos_i)             # <<<<<<<<<<<<<<
 *                             RikVec = subtract(pos_k, pos_i)
 *                             angle = get_angle(RijVec, RikVec)
 */
            __pyx_v_RijVec = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_subtract(__pyx_v_pos_j, __pyx_v_pos_i);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":386
 *                             # Calculate angle
 *                             RijVec = subtract(pos_j,pos_i)
 *                             RikVec = subtract(pos_k, pos_i)             # <<<<<<<<<<<<<<
 *                             angle = get_angle(RijVec, RikVec)
 * 
 */
            __pyx_v_RikVec = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_subtract(__pyx_v_pos_k, __pyx_v_pos_i);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":387
 *                             RijVec = subtract(pos_j,pos_i)
 *                             RikVec = subtract(pos_k, pos_i)
 *                             angle = get_angle(RijVec, RikVec)             # <<<<<<<<<<<<<<
 * 
 *                             # Calculate normalization
 */
            __pyx_v_angle = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_get_angle(__pyx_v_RijVec, __pyx_v_RikVec);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":390
 * 
 *                             # Calculate normalization
 *                             num_pairs = Natoms*Natoms*Natoms             # <<<<<<<<<<<<<<
 *                             normalization = 1./smearing_norm2
 *                             normalization /= num_pairs/volume
 */
            __pyx_v_num_pairs = ((__pyx_v_Natoms * __pyx_v_Natoms) * __pyx_v_Natoms);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":391
 *                             # Calculate normalization
 *                             num_pairs = Natoms*Natoms*Natoms
 *                             normalization = 1./smearing_norm2             # <<<<<<<<<<<<<<
 *                             normalization /= num_pairs/volume
 * 
 */
            if (unlikely(__pyx_v_smearing_norm2 == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 391, __pyx_L1_error)
            }
            __pyx_v_normalization = (1. / __pyx_v_smearing_norm2);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":392
 *                             num_pairs = Natoms*Natoms*Natoms
 *                             normalization = 1./smearing_norm2
 *                             normalization /= num_pairs/volume             # <<<<<<<<<<<<<<
 * 
 *                             # Identify what bin 'Rij' belongs to + it's position in this bin
 */
            if (unlikely(__pyx_v_volume == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 392, __pyx_L1_error)
            }
            __pyx_t_20 = (__pyx_v_num_pairs / __pyx_v_volume);
            if (unlikely(__pyx_t_20 == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 392, __pyx_L1_error)
            }
            __pyx_v_normalization = (__pyx_v_normalization / __pyx_t_20);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":395
 * 
 *                             # Identify what bin 'Rij' belongs to + it's position in this bin
 *                             center_bin = <int> floor(angle/binwidth1)             # <<<<<<<<<<<<<<
 *                             binpos = angle/binwidth2 - center_bin
 * 
 */
            if (unlikely(__pyx_v_binwidth1 == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 395, __pyx_L1_error)
            }
            __pyx_v_center_bin = ((int)floor((__pyx_v_angle / __pyx_v_binwidth1)));

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":396
 *                             # Identify what bin 'Rij' belongs to + it's position in this bin
 *                             center_bin = <int> floor(angle/binwidth1)
 *                             binpos = angle/binwidth2 - center_bin             # <<<<<<<<<<<<<<
 * 
 *                             # Lower and upper range of bins affected by the current atomic distance deltaR.
 */
            if (unlikely(__pyx_v_binwidth2 == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 396, __pyx_L1_error)
            }
            __pyx_v_binpos = ((__pyx_v_angle / __pyx_v_binwidth2) - __pyx_v_center_bin);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":399
 * 
 *                             # Lower and upper range of bins affected by the current atomic distance deltaR.
 *                             minbin_lim = <int> -ceil(m2 - binpos)             # <<<<<<<<<<<<<<
 *                             maxbin_lim = <int> ceil(m2 - (1-binpos))
 * 
 */
            __pyx_v_minbin_lim = ((int)(-ceil((__pyx_v_m2 - __pyx_v_binpos))));

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":400
 *                             # Lower and upper range of bins affected by the current atomic distance deltaR.
 *                             minbin_lim = <int> -ceil(m2 - binpos)
 *                             maxbin_lim = <int> ceil(m2 - (1-binpos))             # <<<<<<<<<<<<<<
 * 
 *                             for n in range(minbin_lim, maxbin_lim + 1):
 */
            __pyx_v_maxbin_lim = ((int)ceil((__pyx_v_m2 - (1.0 - __pyx_v_binpos))));

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":402
 *                             maxbin_lim = <int> ceil(m2 - (1-binpos))
 * 
 *                             for n in range(minbin_lim, maxbin_lim + 1):             # <<<<<<<<<<<<<<
 *                                 newbin = center_bin + n
 * 
 */
            __pyx_t_21 = (__pyx_v_maxbin_lim + 1);
            __pyx_t_22 = __pyx_t_21;
            for (__pyx_t_30 = __pyx_v_minbin_lim; __pyx_t_30 < __pyx_t_22; __pyx_t_30+=1) {
              __pyx_v_n = __pyx_t_30;

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":403
 * 
 *                             for n in range(minbin_lim, maxbin_lim + 1):
 *                                 newbin = center_bin + n             # <<<<<<<<<<<<<<
 * 
 *                                 # Wrap current bin into correct bin-range
 */
              __pyx_v_newbin = (__pyx_v_center_bin + __pyx_v_n);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":406
 * 
 *                                 # Wrap current bin into correct bin-range
 *                                 if newbin < 0:             # <<<<<<<<<<<<<<
 *                                     newbin = abs(newbin)
 *                                 if newbin > Nbins2-1:
 */
              __pyx_t_19 = ((__pyx_v_newbin < 0) != 0);
              if (__pyx_t_19) {

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":407
 *                                 # Wrap current bin into correct bin-range
 *                                 if newbin < 0:
 *                                     newbin = abs(newbin)             # <<<<<<<<<<<<<<
 *                                 if newbin > Nbins2-1:
 *                                     newbin = 2*Nbins2 - newbin - 1
 */
                __pyx_t_31 = abs(__pyx_v_newbin); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 407, __pyx_L1_error)
                __pyx_v_newbin = __pyx_t_31;

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":406
 * 
 *                                 # Wrap current bin into correct bin-range
 *                                 if newbin < 0:             # <<<<<<<<<<<<<<
 *                                     newbin = abs(newbin)
 *                                 if newbin > Nbins2-1:
 */
              }

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":408
 *                                 if newbin < 0:
 *                                     newbin = abs(newbin)
 *                                 if newbin > Nbins2-1:             # <<<<<<<<<<<<<<
 *                                     newbin = 2*Nbins2 - newbin - 1
 * 
 */
              __pyx_t_19 = ((__pyx_v_newbin > (__pyx_v_Nbins2 - 1)) != 0);
              if (__pyx_t_19) {

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":409
 *                                     newbin = abs(newbin)
 *                                 if newbin > Nbins2-1:
 *                                     newbin = 2*Nbins2 - newbin - 1             # <<<<<<<<<<<<<<
 * 
 *                                 # Calculate gauss contribution to current bin
 */
                __pyx_v_newbin = (((2 * __pyx_v_Nbins2) - __pyx_v_newbin) - 1);

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":408
 *                                 if newbin < 0:
 *                                     newbin = abs(newbin)
 *                                 if newbin > Nbins2-1:             # <<<<<<<<<<<<<<
 *                                     newbin = 2*Nbins2 - newbin - 1
 * 
 */
              }

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":412
 * 
 *                                 # Calculate gauss contribution to current bin
 *                                 c = 1./sqrt(2)*binwidth2/sigma2             # <<<<<<<<<<<<<<
 *                                 erfarg_low = max(-m2, n-binpos)
 *                                 erfarg_up = min(m2, n+(1-binpos))
 */
              __pyx_t_20 = sqrt(2.0);
              if (unlikely(__pyx_t_20 == 0)) {
                PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                __PYX_ERR(0, 412, __pyx_L1_error)
              }
              __pyx_t_24 = ((1. / __pyx_t_20) * __pyx_v_binwidth2);
              if (unlikely(__pyx_v_sigma2 == 0)) {
                PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                __PYX_ERR(0, 412, __pyx_L1_error)
              }
              __pyx_v_c = (__pyx_t_24 / __pyx_v_sigma2);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":413
 *                                 # Calculate gauss contribution to current bin
 *                                 c = 1./sqrt(2)*binwidth2/sigma2
 *                                 erfarg_low = max(-m2, n-binpos)             # <<<<<<<<<<<<<<
 *                                 erfarg_up = min(m2, n+(1-binpos))
 *                                 value = 0.5*erf(c*erfarg_up)-0.5*erf(c*erfarg_low)
 */
              __pyx_t_24 = (__pyx_v_n - __pyx_v_binpos);
              __pyx_t_20 = (-__pyx_v_m2);
              if (((__pyx_t_24 > __pyx_t_20) != 0)) {
                __pyx_t_2 = __pyx_t_24;
              } else {
                __pyx_t_2 = __pyx_t_20;
              }
              __pyx_v_erfarg_low = __pyx_t_2;

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":414
 *                                 c = 1./sqrt(2)*binwidth2/sigma2
 *                                 erfarg_low = max(-m2, n-binpos)
 *                                 erfarg_up = min(m2, n+(1-binpos))             # <<<<<<<<<<<<<<
 *                                 value = 0.5*erf(c*erfarg_up)-0.5*erf(c*erfarg_low)
 *                                 value *= f_cutoff(Rij, gamma, Rc2) * f_cutoff(Rik, gamma, Rc2)
 */
              __pyx_t_2 = (__pyx_v_n + (1.0 - __pyx_v_binpos));
              __pyx_t_24 = __pyx_v_m2;
              if (((__pyx_t_2 < __pyx_t_24) != 0)) {
                __pyx_t_20 = __pyx_t_2;
              } else {
                __pyx_t_20 = __pyx_t_24;
              }
              __pyx_v_erfarg_up = __pyx_t_20;

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":415
 *                                 erfarg_low = max(-m2, n-binpos)
 *                                 erfarg_up = min(m2, n+(1-binpos))
 *                                 value = 0.5*erf(c*erfarg_up)-0.5*erf(c*erfarg_low)             # <<<<<<<<<<<<<<
 *                                 value *= f_cutoff(Rij, gamma, Rc2) * f_cutoff(Rik, gamma, Rc2)
 *                                 # Apply normalization
 */
              __pyx_v_value = ((0.5 * erf((__pyx_v_c * __pyx_v_erfarg_up))) - (0.5 * erf((__pyx_v_c * __pyx_v_erfarg_low))));

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":416
 *                                 erfarg_up = min(m2, n+(1-binpos))
 *                                 value = 0.5*erf(c*erfarg_up)-0.5*erf(c*erfarg_low)
 *                                 value *= f_cutoff(Rij, gamma, Rc2) * f_cutoff(Rik, gamma, Rc2)             # <<<<<<<<<<<<<<
 *                                 # Apply normalization
 *                                 value *= normalization
 */
              __pyx_v_value = (__pyx_v_value * (__pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_f_cutoff(__pyx_v_Rij, __pyx_v_gamma, __pyx_v_Rc2) * __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_f_cutoff(__pyx_v_Rik, __pyx_v_gamma, __pyx_v_Rc2)));

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":418
 *                                 value *= f_cutoff(Rij, gamma, Rc2) * f_cutoff(Rik, gamma, Rc2)
 *                                 # Apply normalization
 *                                 value *= normalization             # <<<<<<<<<<<<<<
 * 
 *                                 feature2[bondtype_index + newbin] += value
 */
              __pyx_v_value = (__pyx_v_value * __pyx_v_normalization);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":420
 *                                 value *= normalization
 * 
 *                                 feature2[bondtype_index + newbin] += value             # <<<<<<<<<<<<<<
 *                                 #feature2[newbin] += value
 * 
 */
              __pyx_t_31 = (__pyx_v_bondtype_index + __pyx_v_newbin);
              (__pyx_v_feature2[__pyx_t_31]) = ((__pyx_v_feature2[__pyx_t_31]) + __pyx_v_value);
            }
            __pyx_L43_continue:;
          }
        }
        __pyx_L35_continue:;
      }
    }
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":424
 * 
 *         # Convert angular feature to numpy array
 *         feature2_np = np.zeros(Nelements_3body)             # <<<<<<<<<<<<<<
 *         for m in range(Nelements_3body):
 *             feature2_np[m] = eta * feature2[m]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_Nelements_3body); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_feature2_np = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":425
 *         # Convert angular feature to numpy array
 *         feature2_np = np.zeros(Nelements_3body)
 *         for m in range(Nelements_3body):             # <<<<<<<<<<<<<<
 *             feature2_np[m] = eta * feature2[m]
 * 
 */
  __pyx_t_3 = __pyx_v_Nelements_3body;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":426
 *         feature2_np = np.zeros(Nelements_3body)
 *         for m in range(Nelements_3body):
 *             feature2_np[m] = eta * feature2[m]             # <<<<<<<<<<<<<<
 * 
 *         feature_np = np.zeros(Nelements)
 */
    __pyx_t_1 = PyFloat_FromDouble((__pyx_v_eta * (__pyx_v_feature2[__pyx_v_m]))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(__Pyx_SetItemInt(__pyx_v_feature2_np, __pyx_v_m, __pyx_t_1, int, 1, __Pyx_PyInt_From_int, 0, 1, 1) < 0)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":428
 *             feature2_np[m] = eta * feature2[m]
 * 
 *         feature_np = np.zeros(Nelements)             # <<<<<<<<<<<<<<
 *         feature_np[:Nelements_2body] = feature1_np
 *         feature_np[Nelements_2body:] = feature2_np
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_Nelements); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_feature_np = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":429
 * 
 *         feature_np = np.zeros(Nelements)
 *         feature_np[:Nelements_2body] = feature1_np             # <<<<<<<<<<<<<<
 *         feature_np[Nelements_2body:] = feature2_np
 *         return feature_np
 */
  if (__Pyx_PyObject_SetSlice(__pyx_v_feature_np, __pyx_v_feature1_np, 0, __pyx_v_Nelements_2body, NULL, NULL, NULL, 0, 1, 1) < 0) __PYX_ERR(0, 429, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":430
 *         feature_np = np.zeros(Nelements)
 *         feature_np[:Nelements_2body] = feature1_np
 *         feature_np[Nelements_2body:] = feature2_np             # <<<<<<<<<<<<<<
 *         return feature_np
 * 
 */
  if (__Pyx_PyObject_SetSlice(__pyx_v_feature_np, __pyx_v_feature2_np, __pyx_v_Nelements_2body, 0, NULL, NULL, NULL, 1, 0, 1) < 0) __PYX_ERR(0, 430, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":431
 *         feature_np[:Nelements_2body] = feature1_np
 *         feature_np[Nelements_2body:] = feature2_np
 *         return feature_np             # <<<<<<<<<<<<<<
 * 
 *     def get_featureMat(self, atoms_list):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_feature_np);
  __pyx_r = __pyx_v_feature_np;
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":197
 *         self.neighbourcells_disp = self.__get_neighbour_cells_displacement(self.pbc, self.cell)
 * 
 *     def get_feature(self, atoms):             # <<<<<<<<<<<<<<
 *         """
 *         """
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.get_feature", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_use_angular);
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XDECREF(__pyx_v_cell);
  __Pyx_XDECREF(__pyx_v_pos_np);
  __Pyx_XDECREF(__pyx_v_cell_displacements_old);
  __Pyx_XDECREF(__pyx_v_bondtypes_2body_old);
  __Pyx_XDECREF(__pyx_v_num_converted_old);
  __Pyx_XDECREF(__pyx_v_feature1_np);
  __Pyx_XDECREF(__pyx_v_bondtypes_3body_old);
  __Pyx_XDECREF(__pyx_v_feature2_np);
  __Pyx_XDECREF(__pyx_v_feature_np);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":433
 *         return feature_np
 * 
 *     def get_featureMat(self, atoms_list):             # <<<<<<<<<<<<<<
 *         featureMat = np.array([self.get_feature(atoms) for atoms in atoms_list])
 *         featureMat = np.array(featureMat)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_5get_featureMat(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_5get_featureMat = {"get_featureMat", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_5get_featureMat, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_5get_featureMat(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_atoms_list = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_featureMat (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_atoms_list,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_atoms_list)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_featureMat", 1, 2, 2, 1); __PYX_ERR(0, 433, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_featureMat") < 0)) __PYX_ERR(0, 433, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_atoms_list = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_featureMat", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 433, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.get_featureMat", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_4get_featureMat(__pyx_self, __pyx_v_self, __pyx_v_atoms_list);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_4get_featureMat(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_atoms_list) {
  PyObject *__pyx_v_featureMat = NULL;
  PyObject *__pyx_v_atoms = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_featureMat", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":434
 * 
 *     def get_featureMat(self, atoms_list):
 *         featureMat = np.array([self.get_feature(atoms) for atoms in atoms_list])             # <<<<<<<<<<<<<<
 *         featureMat = np.array(featureMat)
 *         return featureMat
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_array); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (likely(PyList_CheckExact(__pyx_v_atoms_list)) || PyTuple_CheckExact(__pyx_v_atoms_list)) {
    __pyx_t_4 = __pyx_v_atoms_list; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_atoms_list); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 434, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 434, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 434, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 434, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 434, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      }
    } else {
      __pyx_t_7 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_7)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 434, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_XDECREF_SET(__pyx_v_atoms, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_get_feature); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_7 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_v_atoms) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_v_atoms);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_featureMat = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":435
 *     def get_featureMat(self, atoms_list):
 *         featureMat = np.array([self.get_feature(atoms) for atoms in atoms_list])
 *         featureMat = np.array(featureMat)             # <<<<<<<<<<<<<<
 *         return featureMat
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_array); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_featureMat) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_featureMat);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_featureMat, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":436
 *         featureMat = np.array([self.get_feature(atoms) for atoms in atoms_list])
 *         featureMat = np.array(featureMat)
 *         return featureMat             # <<<<<<<<<<<<<<
 * 
 *     def get_featureGradient(self, atoms):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_featureMat);
  __pyx_r = __pyx_v_featureMat;
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":433
 *         return feature_np
 * 
 *     def get_featureMat(self, atoms_list):             # <<<<<<<<<<<<<<
 *         featureMat = np.array([self.get_feature(atoms) for atoms in atoms_list])
 *         featureMat = np.array(featureMat)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.get_featureMat", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_featureMat);
  __Pyx_XDECREF(__pyx_v_atoms);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":438
 *         return featureMat
 * 
 *     def get_featureGradient(self, atoms):             # <<<<<<<<<<<<<<
 * 
 *         cdef double Rc1 = self.Rc1
 */

/* Python wrapper */
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_7get_featureGradient(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_7get_featureGradient = {"get_featureGradient", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_7get_featureGradient, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_7get_featureGradient(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_atoms = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_featureGradient (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_atoms,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_atoms)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_featureGradient", 1, 2, 2, 1); __PYX_ERR(0, 438, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_featureGradient") < 0)) __PYX_ERR(0, 438, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_atoms = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_featureGradient", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 438, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.get_featureGradient", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_6get_featureGradient(__pyx_self, __pyx_v_self, __pyx_v_atoms);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_6get_featureGradient(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_atoms) {
  double __pyx_v_Rc1;
  double __pyx_v_Rc2;
  double __pyx_v_binwidth1;
  double __pyx_v_binwidth2;
  int __pyx_v_Nbins1;
  int __pyx_v_Nbins2;
  double __pyx_v_sigma1;
  double __pyx_v_sigma2;
  int __pyx_v_nsigma;
  double __pyx_v_eta;
  double __pyx_v_gamma;
  PyObject *__pyx_v_use_angular = 0;
  double __pyx_v_volume;
  int __pyx_v_dim;
  double __pyx_v_m1;
  double __pyx_v_m2;
  double __pyx_v_smearing_norm1;
  double __pyx_v_smearing_norm2;
  int __pyx_v_Nelements_2body;
  int __pyx_v_Nelements_3body;
  int __pyx_v_Nelements;
  struct __pyx_obj_5cymem_5cymem_Pool *__pyx_v_mem = 0;
  CYTHON_UNUSED PyObject *__pyx_v_cell = NULL;
  int __pyx_v_Natoms;
  PyObject *__pyx_v_pos_np = 0;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point *__pyx_v_pos;
  int __pyx_v_m;
  int __pyx_v_Ncells;
  PyObject *__pyx_v_cell_displacements_old = 0;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point *__pyx_v_cell_displacements;
  int __pyx_v_Ntypes;
  CYTHON_UNUSED int __pyx_v_Nbondtypes_2body;
  PyObject *__pyx_v_bondtypes_2body_old = 0;
  int *__pyx_v_bondtypes_2body;
  PyObject *__pyx_v_num_converted_old = 0;
  int *__pyx_v_num_converted;
  double *__pyx_v_feature_grad1;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_RijVec;
  int __pyx_v_num_pairs;
  int __pyx_v_center_bin;
  int __pyx_v_minbin_lim;
  int __pyx_v_maxbin_lim;
  int __pyx_v_newbin;
  int __pyx_v_bondtype_index;
  int __pyx_v_type1;
  int __pyx_v_type2;
  double __pyx_v_Rij;
  double __pyx_v_normalization;
  double __pyx_v_binpos;
  double __pyx_v_c;
  double __pyx_v_arg_low;
  double __pyx_v_arg_up;
  double __pyx_v_value1;
  double __pyx_v_value2;
  double __pyx_v_value;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_n;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_pos_i;
  int __pyx_v_cell_index;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_displacement;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_pos_j;
  int __pyx_v_grad_index;
  PyObject *__pyx_v_feature_grad1_np = NULL;
  CYTHON_UNUSED int __pyx_v_Nbondtypes_3body;
  PyObject *__pyx_v_bondtypes_3body_old = 0;
  int *__pyx_v_bondtypes_3body;
  double *__pyx_v_feature_grad2;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_RikVec;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_angle_grad_i;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_angle_grad_j;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_angle_grad_k;
  double __pyx_v_angle;
  double __pyx_v_cos_angle;
  double __pyx_v_a;
  int __pyx_v_k;
  int __pyx_v_bin_index;
  int __pyx_v_type3;
  int __pyx_v_cell_index1;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_displacement1;
  int __pyx_v_cell_index2;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_displacement2;
  long __pyx_v_k_start;
  __pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point __pyx_v_pos_k;
  double __pyx_v_Rik;
  double __pyx_v_fc_ij;
  double __pyx_v_fc_ik;
  double __pyx_v_fc_grad_ij;
  double __pyx_v_fc_grad_ik;
  PyObject *__pyx_v_feature_grad2_np = NULL;
  PyObject *__pyx_v_feature_grad_np = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  void *__pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  int __pyx_t_18;
  int __pyx_t_19;
  double __pyx_t_20;
  long __pyx_t_21;
  long __pyx_t_22;
  int __pyx_t_23;
  double __pyx_t_24;
  int __pyx_t_25;
  int __pyx_t_26;
  int __pyx_t_27;
  int __pyx_t_28;
  int __pyx_t_29;
  int __pyx_t_30;
  int __pyx_t_31;
  int __pyx_t_32;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_featureGradient", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":440
 *     def get_featureGradient(self, atoms):
 * 
 *         cdef double Rc1 = self.Rc1             # <<<<<<<<<<<<<<
 *         cdef double Rc2 = self.Rc2
 *         cdef double binwidth1 = self.binwidth1
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Rc1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 440, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 440, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Rc1 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":441
 * 
 *         cdef double Rc1 = self.Rc1
 *         cdef double Rc2 = self.Rc2             # <<<<<<<<<<<<<<
 *         cdef double binwidth1 = self.binwidth1
 *         cdef double binwidth2 = self.binwidth2
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Rc2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Rc2 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":442
 *         cdef double Rc1 = self.Rc1
 *         cdef double Rc2 = self.Rc2
 *         cdef double binwidth1 = self.binwidth1             # <<<<<<<<<<<<<<
 *         cdef double binwidth2 = self.binwidth2
 *         cdef int Nbins1 = self.Nbins1
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_binwidth1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_binwidth1 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":443
 *         cdef double Rc2 = self.Rc2
 *         cdef double binwidth1 = self.binwidth1
 *         cdef double binwidth2 = self.binwidth2             # <<<<<<<<<<<<<<
 *         cdef int Nbins1 = self.Nbins1
 *         cdef int Nbins2 = self.Nbins2
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_binwidth2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_binwidth2 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":444
 *         cdef double binwidth1 = self.binwidth1
 *         cdef double binwidth2 = self.binwidth2
 *         cdef int Nbins1 = self.Nbins1             # <<<<<<<<<<<<<<
 *         cdef int Nbins2 = self.Nbins2
 *         cdef double sigma1 = self.sigma1
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbins1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nbins1 = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":445
 *         cdef double binwidth2 = self.binwidth2
 *         cdef int Nbins1 = self.Nbins1
 *         cdef int Nbins2 = self.Nbins2             # <<<<<<<<<<<<<<
 *         cdef double sigma1 = self.sigma1
 *         cdef double sigma2 = self.sigma2
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbins2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nbins2 = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":446
 *         cdef int Nbins1 = self.Nbins1
 *         cdef int Nbins2 = self.Nbins2
 *         cdef double sigma1 = self.sigma1             # <<<<<<<<<<<<<<
 *         cdef double sigma2 = self.sigma2
 *         cdef int nsigma = self.nsigma
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sigma1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_sigma1 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":447
 *         cdef int Nbins2 = self.Nbins2
 *         cdef double sigma1 = self.sigma1
 *         cdef double sigma2 = self.sigma2             # <<<<<<<<<<<<<<
 *         cdef int nsigma = self.nsigma
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sigma2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_sigma2 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":448
 *         cdef double sigma1 = self.sigma1
 *         cdef double sigma2 = self.sigma2
 *         cdef int nsigma = self.nsigma             # <<<<<<<<<<<<<<
 * 
 *         cdef double eta = self.eta
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_nsigma); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_nsigma = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":450
 *         cdef int nsigma = self.nsigma
 * 
 *         cdef double eta = self.eta             # <<<<<<<<<<<<<<
 *         cdef double gamma = self.gamma
 *         cdef use_angular = self.use_angular
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_eta); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_eta = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":451
 * 
 *         cdef double eta = self.eta
 *         cdef double gamma = self.gamma             # <<<<<<<<<<<<<<
 *         cdef use_angular = self.use_angular
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_gamma); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 451, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_gamma = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":452
 *         cdef double eta = self.eta
 *         cdef double gamma = self.gamma
 *         cdef use_angular = self.use_angular             # <<<<<<<<<<<<<<
 * 
 *         cdef double volume = self.volume
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_use_angular); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_use_angular = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":454
 *         cdef use_angular = self.use_angular
 * 
 *         cdef double volume = self.volume             # <<<<<<<<<<<<<<
 *         cdef int dim = self.dim
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_volume); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_volume = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":455
 * 
 *         cdef double volume = self.volume
 *         cdef int dim = self.dim             # <<<<<<<<<<<<<<
 * 
 *         cdef double m1 = self.m1
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_dim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dim = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":457
 *         cdef int dim = self.dim
 * 
 *         cdef double m1 = self.m1             # <<<<<<<<<<<<<<
 *         cdef double m2 = self.m2
 *         cdef double smearing_norm1 = self.smearing_norm1
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_m1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_m1 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":458
 * 
 *         cdef double m1 = self.m1
 *         cdef double m2 = self.m2             # <<<<<<<<<<<<<<
 *         cdef double smearing_norm1 = self.smearing_norm1
 *         cdef double smearing_norm2 = self.smearing_norm2
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_m2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_m2 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":459
 *         cdef double m1 = self.m1
 *         cdef double m2 = self.m2
 *         cdef double smearing_norm1 = self.smearing_norm1             # <<<<<<<<<<<<<<
 *         cdef double smearing_norm2 = self.smearing_norm2
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_smearing_norm1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_smearing_norm1 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":460
 *         cdef double m2 = self.m2
 *         cdef double smearing_norm1 = self.smearing_norm1
 *         cdef double smearing_norm2 = self.smearing_norm2             # <<<<<<<<<<<<<<
 * 
 *         cdef int Nelements_2body = self.Nelements_2body
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_smearing_norm2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_smearing_norm2 = __pyx_t_2;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":462
 *         cdef double smearing_norm2 = self.smearing_norm2
 * 
 *         cdef int Nelements_2body = self.Nelements_2body             # <<<<<<<<<<<<<<
 *         cdef int Nelements_3body = self.Nelements_3body
 *         cdef int Nelements = self.Nelements
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nelements_2body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nelements_2body = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":463
 * 
 *         cdef int Nelements_2body = self.Nelements_2body
 *         cdef int Nelements_3body = self.Nelements_3body             # <<<<<<<<<<<<<<
 *         cdef int Nelements = self.Nelements
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nelements_3body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nelements_3body = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":464
 *         cdef int Nelements_2body = self.Nelements_2body
 *         cdef int Nelements_3body = self.Nelements_3body
 *         cdef int Nelements = self.Nelements             # <<<<<<<<<<<<<<
 * 
 *         # Memory allocation pool
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nelements); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nelements = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":468
 *         # Memory allocation pool
 *         cdef Pool mem
 *         mem = Pool()             # <<<<<<<<<<<<<<
 * 
 *         cell = atoms.get_cell()
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5cymem_5cymem_Pool)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_mem = ((struct __pyx_obj_5cymem_5cymem_Pool *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":470
 *         mem = Pool()
 * 
 *         cell = atoms.get_cell()             # <<<<<<<<<<<<<<
 *         cdef int Natoms = self.Natoms
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_cell); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_cell = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":471
 * 
 *         cell = atoms.get_cell()
 *         cdef int Natoms = self.Natoms             # <<<<<<<<<<<<<<
 * 
 *         # Get positions and convert to Point-struct
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Natoms); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 471, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Natoms = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":474
 * 
 *         # Get positions and convert to Point-struct
 *         cdef list pos_np = atoms.get_positions().tolist()             # <<<<<<<<<<<<<<
 *         cdef Point *pos
 *         pos = <Point*>mem.alloc(Natoms, sizeof(Point))
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_positions); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_tolist); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 474, __pyx_L1_error)
  __pyx_v_pos_np = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":476
 *         cdef list pos_np = atoms.get_positions().tolist()
 *         cdef Point *pos
 *         pos = <Point*>mem.alloc(Natoms, sizeof(Point))             # <<<<<<<<<<<<<<
 *         cdef int m
 *         for m in range(Natoms):
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, __pyx_v_Natoms, (sizeof(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 476, __pyx_L1_error)
  __pyx_v_pos = ((__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":478
 *         pos = <Point*>mem.alloc(Natoms, sizeof(Point))
 *         cdef int m
 *         for m in range(Natoms):             # <<<<<<<<<<<<<<
 *             pos[m].coord[0] = pos_np[m][0]
 *             pos[m].coord[1] = pos_np[m][1]
 */
  __pyx_t_3 = __pyx_v_Natoms;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":479
 *         cdef int m
 *         for m in range(Natoms):
 *             pos[m].coord[0] = pos_np[m][0]             # <<<<<<<<<<<<<<
 *             pos[m].coord[1] = pos_np[m][1]
 *             pos[m].coord[2] = pos_np[m][2]
 */
    if (unlikely(__pyx_v_pos_np == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 479, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_pos_np, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 479, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    ((__pyx_v_pos[__pyx_v_m]).coord[0]) = __pyx_t_2;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":480
 *         for m in range(Natoms):
 *             pos[m].coord[0] = pos_np[m][0]
 *             pos[m].coord[1] = pos_np[m][1]             # <<<<<<<<<<<<<<
 *             pos[m].coord[2] = pos_np[m][2]
 * 
 */
    if (unlikely(__pyx_v_pos_np == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 480, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_pos_np, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 480, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_5, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 480, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 480, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    ((__pyx_v_pos[__pyx_v_m]).coord[1]) = __pyx_t_2;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":481
 *             pos[m].coord[0] = pos_np[m][0]
 *             pos[m].coord[1] = pos_np[m][1]
 *             pos[m].coord[2] = pos_np[m][2]             # <<<<<<<<<<<<<<
 * 
 *         # Get neighbourcells and convert to Point-struct
 */
    if (unlikely(__pyx_v_pos_np == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 481, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_pos_np, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 481, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_1, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 481, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 481, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    ((__pyx_v_pos[__pyx_v_m]).coord[2]) = __pyx_t_2;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":484
 * 
 *         # Get neighbourcells and convert to Point-struct
 *         cdef int Ncells = len(self.neighbourcells_disp)             # <<<<<<<<<<<<<<
 *         cdef list cell_displacements_old = self.neighbourcells_disp
 *         cdef Point *cell_displacements
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_neighbourcells_disp); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = PyObject_Length(__pyx_t_5); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_Ncells = __pyx_t_10;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":485
 *         # Get neighbourcells and convert to Point-struct
 *         cdef int Ncells = len(self.neighbourcells_disp)
 *         cdef list cell_displacements_old = self.neighbourcells_disp             # <<<<<<<<<<<<<<
 *         cdef Point *cell_displacements
 *         cell_displacements = <Point*>mem.alloc(Ncells, sizeof(Point))
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_neighbourcells_disp); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (!(likely(PyList_CheckExact(__pyx_t_5))||((__pyx_t_5) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_5)->tp_name), 0))) __PYX_ERR(0, 485, __pyx_L1_error)
  __pyx_v_cell_displacements_old = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":487
 *         cdef list cell_displacements_old = self.neighbourcells_disp
 *         cdef Point *cell_displacements
 *         cell_displacements = <Point*>mem.alloc(Ncells, sizeof(Point))             # <<<<<<<<<<<<<<
 *         for m in range(Ncells):
 *             cell_displacements[m].coord[0] = cell_displacements_old[m][0]
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, __pyx_v_Ncells, (sizeof(__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 487, __pyx_L1_error)
  __pyx_v_cell_displacements = ((__pyx_t_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_Point *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":488
 *         cdef Point *cell_displacements
 *         cell_displacements = <Point*>mem.alloc(Ncells, sizeof(Point))
 *         for m in range(Ncells):             # <<<<<<<<<<<<<<
 *             cell_displacements[m].coord[0] = cell_displacements_old[m][0]
 *             cell_displacements[m].coord[1] = cell_displacements_old[m][1]
 */
  __pyx_t_3 = __pyx_v_Ncells;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":489
 *         cell_displacements = <Point*>mem.alloc(Ncells, sizeof(Point))
 *         for m in range(Ncells):
 *             cell_displacements[m].coord[0] = cell_displacements_old[m][0]             # <<<<<<<<<<<<<<
 *             cell_displacements[m].coord[1] = cell_displacements_old[m][1]
 *             cell_displacements[m].coord[2] = cell_displacements_old[m][2]
 */
    if (unlikely(__pyx_v_cell_displacements_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 489, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_cell_displacements_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_5, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    ((__pyx_v_cell_displacements[__pyx_v_m]).coord[0]) = __pyx_t_2;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":490
 *         for m in range(Ncells):
 *             cell_displacements[m].coord[0] = cell_displacements_old[m][0]
 *             cell_displacements[m].coord[1] = cell_displacements_old[m][1]             # <<<<<<<<<<<<<<
 *             cell_displacements[m].coord[2] = cell_displacements_old[m][2]
 * 
 */
    if (unlikely(__pyx_v_cell_displacements_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 490, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_cell_displacements_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 490, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 490, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 490, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    ((__pyx_v_cell_displacements[__pyx_v_m]).coord[1]) = __pyx_t_2;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":491
 *             cell_displacements[m].coord[0] = cell_displacements_old[m][0]
 *             cell_displacements[m].coord[1] = cell_displacements_old[m][1]
 *             cell_displacements[m].coord[2] = cell_displacements_old[m][2]             # <<<<<<<<<<<<<<
 * 
 *         # Convert bondtype list into c-array
 */
    if (unlikely(__pyx_v_cell_displacements_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 491, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_cell_displacements_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 491, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_5, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 491, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 491, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    ((__pyx_v_cell_displacements[__pyx_v_m]).coord[2]) = __pyx_t_2;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":494
 * 
 *         # Convert bondtype list into c-array
 *         cdef int Ntypes = self.Ntypes             # <<<<<<<<<<<<<<
 *         cdef int Nbondtypes_2body = self.Nbondtypes_2body
 *         cdef list bondtypes_2body_old = self.bondtypes_2body
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Ntypes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Ntypes = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":495
 *         # Convert bondtype list into c-array
 *         cdef int Ntypes = self.Ntypes
 *         cdef int Nbondtypes_2body = self.Nbondtypes_2body             # <<<<<<<<<<<<<<
 *         cdef list bondtypes_2body_old = self.bondtypes_2body
 *         cdef int *bondtypes_2body
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_2body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nbondtypes_2body = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":496
 *         cdef int Ntypes = self.Ntypes
 *         cdef int Nbondtypes_2body = self.Nbondtypes_2body
 *         cdef list bondtypes_2body_old = self.bondtypes_2body             # <<<<<<<<<<<<<<
 *         cdef int *bondtypes_2body
 *         bondtypes_2body = <int*>mem.alloc(Ntypes*Ntypes, sizeof(int))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_2body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 496, __pyx_L1_error)
  __pyx_v_bondtypes_2body_old = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":498
 *         cdef list bondtypes_2body_old = self.bondtypes_2body
 *         cdef int *bondtypes_2body
 *         bondtypes_2body = <int*>mem.alloc(Ntypes*Ntypes, sizeof(int))             # <<<<<<<<<<<<<<
 *         for m in range(Ntypes*Ntypes):
 *             bondtypes_2body[m] = bondtypes_2body_old[m]
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, (__pyx_v_Ntypes * __pyx_v_Ntypes), (sizeof(int))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 498, __pyx_L1_error)
  __pyx_v_bondtypes_2body = ((int *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":499
 *         cdef int *bondtypes_2body
 *         bondtypes_2body = <int*>mem.alloc(Ntypes*Ntypes, sizeof(int))
 *         for m in range(Ntypes*Ntypes):             # <<<<<<<<<<<<<<
 *             bondtypes_2body[m] = bondtypes_2body_old[m]
 * 
 */
  __pyx_t_3 = (__pyx_v_Ntypes * __pyx_v_Ntypes);
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":500
 *         bondtypes_2body = <int*>mem.alloc(Ntypes*Ntypes, sizeof(int))
 *         for m in range(Ntypes*Ntypes):
 *             bondtypes_2body[m] = bondtypes_2body_old[m]             # <<<<<<<<<<<<<<
 * 
 *         # Get converted atom Ntypes
 */
    if (unlikely(__pyx_v_bondtypes_2body_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 500, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_bondtypes_2body_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 500, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 500, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    (__pyx_v_bondtypes_2body[__pyx_v_m]) = __pyx_t_11;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":503
 * 
 *         # Get converted atom Ntypes
 *         cdef list num_converted_old = convert_atom_types(atoms.get_atomic_numbers())             # <<<<<<<<<<<<<<
 *         cdef int *num_converted
 *         num_converted = <int*>mem.alloc(Natoms, sizeof(int))
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_convert_atom_types); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_atoms, __pyx_n_s_get_atomic_numbers); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_12)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  __pyx_t_4 = (__pyx_t_12) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 503, __pyx_L1_error)
  __pyx_v_num_converted_old = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":505
 *         cdef list num_converted_old = convert_atom_types(atoms.get_atomic_numbers())
 *         cdef int *num_converted
 *         num_converted = <int*>mem.alloc(Natoms, sizeof(int))             # <<<<<<<<<<<<<<
 *         for m in range(Natoms):
 *             num_converted[m] = num_converted_old[m]
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, __pyx_v_Natoms, (sizeof(int))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 505, __pyx_L1_error)
  __pyx_v_num_converted = ((int *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":506
 *         cdef int *num_converted
 *         num_converted = <int*>mem.alloc(Natoms, sizeof(int))
 *         for m in range(Natoms):             # <<<<<<<<<<<<<<
 *             num_converted[m] = num_converted_old[m]
 * 
 */
  __pyx_t_3 = __pyx_v_Natoms;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":507
 *         num_converted = <int*>mem.alloc(Natoms, sizeof(int))
 *         for m in range(Natoms):
 *             num_converted[m] = num_converted_old[m]             # <<<<<<<<<<<<<<
 * 
 *         # RADIAL FEATURE GRADIENT
 */
    if (unlikely(__pyx_v_num_converted_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 507, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_num_converted_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 507, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 507, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    (__pyx_v_num_converted[__pyx_v_m]) = __pyx_t_11;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":513
 *         # Initialize radial feature-gradient
 *         cdef double *feature_grad1
 *         feature_grad1 = <double*>mem.alloc(Nelements_2body * Natoms * dim, sizeof(double))             # <<<<<<<<<<<<<<
 * 
 *         cdef Point RijVec
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, ((__pyx_v_Nelements_2body * __pyx_v_Natoms) * __pyx_v_dim), (sizeof(double))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 513, __pyx_L1_error)
  __pyx_v_feature_grad1 = ((double *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":519
 *         cdef double Rij, normalization, binpos, c, arg_low, arg_up, value1, value2, value
 *         cdef int i, j, n
 *         for i in range(Natoms):             # <<<<<<<<<<<<<<
 *             pos_i = pos[i]
 *             for cell_index in range(Ncells):
 */
  __pyx_t_3 = __pyx_v_Natoms;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":520
 *         cdef int i, j, n
 *         for i in range(Natoms):
 *             pos_i = pos[i]             # <<<<<<<<<<<<<<
 *             for cell_index in range(Ncells):
 *                 displacement = cell_displacements[cell_index]
 */
    __pyx_v_pos_i = (__pyx_v_pos[__pyx_v_i]);

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":521
 *         for i in range(Natoms):
 *             pos_i = pos[i]
 *             for cell_index in range(Ncells):             # <<<<<<<<<<<<<<
 *                 displacement = cell_displacements[cell_index]
 *                 for j in range(Natoms):
 */
    __pyx_t_11 = __pyx_v_Ncells;
    __pyx_t_13 = __pyx_t_11;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_cell_index = __pyx_t_14;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":522
 *             pos_i = pos[i]
 *             for cell_index in range(Ncells):
 *                 displacement = cell_displacements[cell_index]             # <<<<<<<<<<<<<<
 *                 for j in range(Natoms):
 *                     pos_j = add(pos[j], displacement)
 */
      __pyx_v_displacement = (__pyx_v_cell_displacements[__pyx_v_cell_index]);

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":523
 *             for cell_index in range(Ncells):
 *                 displacement = cell_displacements[cell_index]
 *                 for j in range(Natoms):             # <<<<<<<<<<<<<<
 *                     pos_j = add(pos[j], displacement)
 *                     Rij = euclidean(pos_i, pos_j)
 */
      __pyx_t_15 = __pyx_v_Natoms;
      __pyx_t_16 = __pyx_t_15;
      for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
        __pyx_v_j = __pyx_t_17;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":524
 *                 displacement = cell_displacements[cell_index]
 *                 for j in range(Natoms):
 *                     pos_j = add(pos[j], displacement)             # <<<<<<<<<<<<<<
 *                     Rij = euclidean(pos_i, pos_j)
 * 
 */
        __pyx_v_pos_j = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_add((__pyx_v_pos[__pyx_v_j]), __pyx_v_displacement);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":525
 *                 for j in range(Natoms):
 *                     pos_j = add(pos[j], displacement)
 *                     Rij = euclidean(pos_i, pos_j)             # <<<<<<<<<<<<<<
 * 
 *                     # Stop if distance too long or atoms are the same one.
 */
        __pyx_v_Rij = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_euclidean(__pyx_v_pos_i, __pyx_v_pos_j);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":528
 * 
 *                     # Stop if distance too long or atoms are the same one.
 *                     if Rij > Rc1+nsigma*sigma1 or Rij < 1e-6:             # <<<<<<<<<<<<<<
 *                         continue
 *                     RijVec = subtract(pos_j,pos_i)
 */
        __pyx_t_19 = ((__pyx_v_Rij > (__pyx_v_Rc1 + (__pyx_v_nsigma * __pyx_v_sigma1))) != 0);
        if (!__pyx_t_19) {
        } else {
          __pyx_t_18 = __pyx_t_19;
          goto __pyx_L18_bool_binop_done;
        }
        __pyx_t_19 = ((__pyx_v_Rij < 1e-6) != 0);
        __pyx_t_18 = __pyx_t_19;
        __pyx_L18_bool_binop_done:;
        if (__pyx_t_18) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":529
 *                     # Stop if distance too long or atoms are the same one.
 *                     if Rij > Rc1+nsigma*sigma1 or Rij < 1e-6:
 *                         continue             # <<<<<<<<<<<<<<
 *                     RijVec = subtract(pos_j,pos_i)
 * 
 */
          goto __pyx_L15_continue;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":528
 * 
 *                     # Stop if distance too long or atoms are the same one.
 *                     if Rij > Rc1+nsigma*sigma1 or Rij < 1e-6:             # <<<<<<<<<<<<<<
 *                         continue
 *                     RijVec = subtract(pos_j,pos_i)
 */
        }

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":530
 *                     if Rij > Rc1+nsigma*sigma1 or Rij < 1e-6:
 *                         continue
 *                     RijVec = subtract(pos_j,pos_i)             # <<<<<<<<<<<<<<
 * 
 *                     # determine bondtype
 */
        __pyx_v_RijVec = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_subtract(__pyx_v_pos_j, __pyx_v_pos_i);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":533
 * 
 *                     # determine bondtype
 *                     if num_converted[i] <= num_converted[j]:             # <<<<<<<<<<<<<<
 *                         type1 = num_converted[i]
 *                         type2 = num_converted[j]
 */
        __pyx_t_18 = (((__pyx_v_num_converted[__pyx_v_i]) <= (__pyx_v_num_converted[__pyx_v_j])) != 0);
        if (__pyx_t_18) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":534
 *                     # determine bondtype
 *                     if num_converted[i] <= num_converted[j]:
 *                         type1 = num_converted[i]             # <<<<<<<<<<<<<<
 *                         type2 = num_converted[j]
 *                     else:
 */
          __pyx_v_type1 = (__pyx_v_num_converted[__pyx_v_i]);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":535
 *                     if num_converted[i] <= num_converted[j]:
 *                         type1 = num_converted[i]
 *                         type2 = num_converted[j]             # <<<<<<<<<<<<<<
 *                     else:
 *                         type1 = num_converted[j]
 */
          __pyx_v_type2 = (__pyx_v_num_converted[__pyx_v_j]);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":533
 * 
 *                     # determine bondtype
 *                     if num_converted[i] <= num_converted[j]:             # <<<<<<<<<<<<<<
 *                         type1 = num_converted[i]
 *                         type2 = num_converted[j]
 */
          goto __pyx_L20;
        }

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":537
 *                         type2 = num_converted[j]
 *                     else:
 *                         type1 = num_converted[j]             # <<<<<<<<<<<<<<
 *                         type2 = num_converted[i]
 *                     bondtype_index = Nbins1*bondtypes_2body[Ntypes*type1+type2]
 */
        /*else*/ {
          __pyx_v_type1 = (__pyx_v_num_converted[__pyx_v_j]);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":538
 *                     else:
 *                         type1 = num_converted[j]
 *                         type2 = num_converted[i]             # <<<<<<<<<<<<<<
 *                     bondtype_index = Nbins1*bondtypes_2body[Ntypes*type1+type2]
 * 
 */
          __pyx_v_type2 = (__pyx_v_num_converted[__pyx_v_i]);
        }
        __pyx_L20:;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":539
 *                         type1 = num_converted[j]
 *                         type2 = num_converted[i]
 *                     bondtype_index = Nbins1*bondtypes_2body[Ntypes*type1+type2]             # <<<<<<<<<<<<<<
 * 
 *                     # Calculate normalization
 */
        __pyx_v_bondtype_index = (__pyx_v_Nbins1 * (__pyx_v_bondtypes_2body[((__pyx_v_Ntypes * __pyx_v_type1) + __pyx_v_type2)]));

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":542
 * 
 *                     # Calculate normalization
 *                     num_pairs = Natoms*Natoms             # <<<<<<<<<<<<<<
 *                     normalization = 1./smearing_norm1
 *                     normalization /= 4*M_PI*Rij*Rij * binwidth1 * num_pairs/volume
 */
        __pyx_v_num_pairs = (__pyx_v_Natoms * __pyx_v_Natoms);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":543
 *                     # Calculate normalization
 *                     num_pairs = Natoms*Natoms
 *                     normalization = 1./smearing_norm1             # <<<<<<<<<<<<<<
 *                     normalization /= 4*M_PI*Rij*Rij * binwidth1 * num_pairs/volume
 * 
 */
        if (unlikely(__pyx_v_smearing_norm1 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 543, __pyx_L1_error)
        }
        __pyx_v_normalization = (1. / __pyx_v_smearing_norm1);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":544
 *                     num_pairs = Natoms*Natoms
 *                     normalization = 1./smearing_norm1
 *                     normalization /= 4*M_PI*Rij*Rij * binwidth1 * num_pairs/volume             # <<<<<<<<<<<<<<
 * 
 *                     # Identify what bin 'Rij' belongs to + it's position in this bin
 */
        __pyx_t_2 = (((((4.0 * M_PI) * __pyx_v_Rij) * __pyx_v_Rij) * __pyx_v_binwidth1) * __pyx_v_num_pairs);
        if (unlikely(__pyx_v_volume == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 544, __pyx_L1_error)
        }
        __pyx_t_20 = (__pyx_t_2 / __pyx_v_volume);
        if (unlikely(__pyx_t_20 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 544, __pyx_L1_error)
        }
        __pyx_v_normalization = (__pyx_v_normalization / __pyx_t_20);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":547
 * 
 *                     # Identify what bin 'Rij' belongs to + it's position in this bin
 *                     center_bin = <int> floor(Rij/binwidth1)             # <<<<<<<<<<<<<<
 *                     binpos = Rij/binwidth1 - center_bin
 * 
 */
        if (unlikely(__pyx_v_binwidth1 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 547, __pyx_L1_error)
        }
        __pyx_v_center_bin = ((int)floor((__pyx_v_Rij / __pyx_v_binwidth1)));

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":548
 *                     # Identify what bin 'Rij' belongs to + it's position in this bin
 *                     center_bin = <int> floor(Rij/binwidth1)
 *                     binpos = Rij/binwidth1 - center_bin             # <<<<<<<<<<<<<<
 * 
 *                     # Lower and upper range of bins affected by the current atomic distance deltaR.
 */
        if (unlikely(__pyx_v_binwidth1 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 548, __pyx_L1_error)
        }
        __pyx_v_binpos = ((__pyx_v_Rij / __pyx_v_binwidth1) - __pyx_v_center_bin);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":551
 * 
 *                     # Lower and upper range of bins affected by the current atomic distance deltaR.
 *                     minbin_lim = <int> -ceil(m1 - binpos)             # <<<<<<<<<<<<<<
 *                     maxbin_lim = <int> ceil(m1 - (1-binpos))
 * 
 */
        __pyx_v_minbin_lim = ((int)(-ceil((__pyx_v_m1 - __pyx_v_binpos))));

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":552
 *                     # Lower and upper range of bins affected by the current atomic distance deltaR.
 *                     minbin_lim = <int> -ceil(m1 - binpos)
 *                     maxbin_lim = <int> ceil(m1 - (1-binpos))             # <<<<<<<<<<<<<<
 * 
 *                     for n in range(minbin_lim, maxbin_lim + 1):
 */
        __pyx_v_maxbin_lim = ((int)ceil((__pyx_v_m1 - (1.0 - __pyx_v_binpos))));

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":554
 *                     maxbin_lim = <int> ceil(m1 - (1-binpos))
 * 
 *                     for n in range(minbin_lim, maxbin_lim + 1):             # <<<<<<<<<<<<<<
 *                         newbin = center_bin + n
 *                         if newbin < 0 or newbin >= Nbins1:
 */
        __pyx_t_21 = (__pyx_v_maxbin_lim + 1);
        __pyx_t_22 = __pyx_t_21;
        for (__pyx_t_23 = __pyx_v_minbin_lim; __pyx_t_23 < __pyx_t_22; __pyx_t_23+=1) {
          __pyx_v_n = __pyx_t_23;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":555
 * 
 *                     for n in range(minbin_lim, maxbin_lim + 1):
 *                         newbin = center_bin + n             # <<<<<<<<<<<<<<
 *                         if newbin < 0 or newbin >= Nbins1:
 *                             continue
 */
          __pyx_v_newbin = (__pyx_v_center_bin + __pyx_v_n);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":556
 *                     for n in range(minbin_lim, maxbin_lim + 1):
 *                         newbin = center_bin + n
 *                         if newbin < 0 or newbin >= Nbins1:             # <<<<<<<<<<<<<<
 *                             continue
 * 
 */
          __pyx_t_19 = ((__pyx_v_newbin < 0) != 0);
          if (!__pyx_t_19) {
          } else {
            __pyx_t_18 = __pyx_t_19;
            goto __pyx_L24_bool_binop_done;
          }
          __pyx_t_19 = ((__pyx_v_newbin >= __pyx_v_Nbins1) != 0);
          __pyx_t_18 = __pyx_t_19;
          __pyx_L24_bool_binop_done:;
          if (__pyx_t_18) {

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":557
 *                         newbin = center_bin + n
 *                         if newbin < 0 or newbin >= Nbins1:
 *                             continue             # <<<<<<<<<<<<<<
 * 
 *                         # Calculate gauss contribution to current bin
 */
            goto __pyx_L21_continue;

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":556
 *                     for n in range(minbin_lim, maxbin_lim + 1):
 *                         newbin = center_bin + n
 *                         if newbin < 0 or newbin >= Nbins1:             # <<<<<<<<<<<<<<
 *                             continue
 * 
 */
          }

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":560
 * 
 *                         # Calculate gauss contribution to current bin
 *                         c = 1./sqrt(2)*binwidth1/sigma1             # <<<<<<<<<<<<<<
 *                         arg_low = max(-m1, n-binpos)
 *                         arg_up = min(m1, n+(1-binpos))
 */
          __pyx_t_20 = sqrt(2.0);
          if (unlikely(__pyx_t_20 == 0)) {
            PyErr_SetString(PyExc_ZeroDivisionError, "float division");
            __PYX_ERR(0, 560, __pyx_L1_error)
          }
          __pyx_t_2 = ((1. / __pyx_t_20) * __pyx_v_binwidth1);
          if (unlikely(__pyx_v_sigma1 == 0)) {
            PyErr_SetString(PyExc_ZeroDivisionError, "float division");
            __PYX_ERR(0, 560, __pyx_L1_error)
          }
          __pyx_v_c = (__pyx_t_2 / __pyx_v_sigma1);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":561
 *                         # Calculate gauss contribution to current bin
 *                         c = 1./sqrt(2)*binwidth1/sigma1
 *                         arg_low = max(-m1, n-binpos)             # <<<<<<<<<<<<<<
 *                         arg_up = min(m1, n+(1-binpos))
 *                         value1 = -1./Rij * (erf(c*arg_up) - erf(c*arg_low))
 */
          __pyx_t_2 = (__pyx_v_n - __pyx_v_binpos);
          __pyx_t_20 = (-__pyx_v_m1);
          if (((__pyx_t_2 > __pyx_t_20) != 0)) {
            __pyx_t_24 = __pyx_t_2;
          } else {
            __pyx_t_24 = __pyx_t_20;
          }
          __pyx_v_arg_low = __pyx_t_24;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":562
 *                         c = 1./sqrt(2)*binwidth1/sigma1
 *                         arg_low = max(-m1, n-binpos)
 *                         arg_up = min(m1, n+(1-binpos))             # <<<<<<<<<<<<<<
 *                         value1 = -1./Rij * (erf(c*arg_up) - erf(c*arg_low))
 *                         value2 = -1./(sigma1*sqrt(2*M_PI)) * (exp(-pow(c*arg_up,2)) - exp(-pow(c*arg_low,2)))
 */
          __pyx_t_24 = (__pyx_v_n + (1.0 - __pyx_v_binpos));
          __pyx_t_2 = __pyx_v_m1;
          if (((__pyx_t_24 < __pyx_t_2) != 0)) {
            __pyx_t_20 = __pyx_t_24;
          } else {
            __pyx_t_20 = __pyx_t_2;
          }
          __pyx_v_arg_up = __pyx_t_20;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":563
 *                         arg_low = max(-m1, n-binpos)
 *                         arg_up = min(m1, n+(1-binpos))
 *                         value1 = -1./Rij * (erf(c*arg_up) - erf(c*arg_low))             # <<<<<<<<<<<<<<
 *                         value2 = -1./(sigma1*sqrt(2*M_PI)) * (exp(-pow(c*arg_up,2)) - exp(-pow(c*arg_low,2)))
 *                         value = value1 + value2
 */
          if (unlikely(__pyx_v_Rij == 0)) {
            PyErr_SetString(PyExc_ZeroDivisionError, "float division");
            __PYX_ERR(0, 563, __pyx_L1_error)
          }
          __pyx_v_value1 = ((-1. / __pyx_v_Rij) * (erf((__pyx_v_c * __pyx_v_arg_up)) - erf((__pyx_v_c * __pyx_v_arg_low))));

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":564
 *                         arg_up = min(m1, n+(1-binpos))
 *                         value1 = -1./Rij * (erf(c*arg_up) - erf(c*arg_low))
 *                         value2 = -1./(sigma1*sqrt(2*M_PI)) * (exp(-pow(c*arg_up,2)) - exp(-pow(c*arg_low,2)))             # <<<<<<<<<<<<<<
 *                         value = value1 + value2
 * 
 */
          __pyx_t_20 = (__pyx_v_sigma1 * sqrt((2.0 * M_PI)));
          if (unlikely(__pyx_t_20 == 0)) {
            PyErr_SetString(PyExc_ZeroDivisionError, "float division");
            __PYX_ERR(0, 564, __pyx_L1_error)
          }
          __pyx_v_value2 = ((-1. / __pyx_t_20) * (exp((-pow((__pyx_v_c * __pyx_v_arg_up), 2.0))) - exp((-pow((__pyx_v_c * __pyx_v_arg_low), 2.0)))));

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":565
 *                         value1 = -1./Rij * (erf(c*arg_up) - erf(c*arg_low))
 *                         value2 = -1./(sigma1*sqrt(2*M_PI)) * (exp(-pow(c*arg_up,2)) - exp(-pow(c*arg_low,2)))
 *                         value = value1 + value2             # <<<<<<<<<<<<<<
 * 
 *                         # Apply normalization
 */
          __pyx_v_value = (__pyx_v_value1 + __pyx_v_value2);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":568
 * 
 *                         # Apply normalization
 *                         value *= normalization             # <<<<<<<<<<<<<<
 * 
 *                         # Add to the the gradient matrix
 */
          __pyx_v_value = (__pyx_v_value * __pyx_v_normalization);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":571
 * 
 *                         # Add to the the gradient matrix
 *                         for m in range(3):             # <<<<<<<<<<<<<<
 *                             feature_grad1[(bondtype_index + newbin) * Natoms*dim + dim*i+m] += -value/Rij * RijVec.coord[m]
 *                             feature_grad1[(bondtype_index + newbin) * Natoms*dim + dim*j+m] += value/Rij * RijVec.coord[m]
 */
          for (__pyx_t_25 = 0; __pyx_t_25 < 3; __pyx_t_25+=1) {
            __pyx_v_m = __pyx_t_25;

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":572
 *                         # Add to the the gradient matrix
 *                         for m in range(3):
 *                             feature_grad1[(bondtype_index + newbin) * Natoms*dim + dim*i+m] += -value/Rij * RijVec.coord[m]             # <<<<<<<<<<<<<<
 *                             feature_grad1[(bondtype_index + newbin) * Natoms*dim + dim*j+m] += value/Rij * RijVec.coord[m]
 * 
 */
            __pyx_t_26 = (((((__pyx_v_bondtype_index + __pyx_v_newbin) * __pyx_v_Natoms) * __pyx_v_dim) + (__pyx_v_dim * __pyx_v_i)) + __pyx_v_m);
            __pyx_t_20 = (-__pyx_v_value);
            if (unlikely(__pyx_v_Rij == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 572, __pyx_L1_error)
            }
            (__pyx_v_feature_grad1[__pyx_t_26]) = ((__pyx_v_feature_grad1[__pyx_t_26]) + ((__pyx_t_20 / __pyx_v_Rij) * (__pyx_v_RijVec.coord[__pyx_v_m])));

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":573
 *                         for m in range(3):
 *                             feature_grad1[(bondtype_index + newbin) * Natoms*dim + dim*i+m] += -value/Rij * RijVec.coord[m]
 *                             feature_grad1[(bondtype_index + newbin) * Natoms*dim + dim*j+m] += value/Rij * RijVec.coord[m]             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_26 = (((((__pyx_v_bondtype_index + __pyx_v_newbin) * __pyx_v_Natoms) * __pyx_v_dim) + (__pyx_v_dim * __pyx_v_j)) + __pyx_v_m);
            if (unlikely(__pyx_v_Rij == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 573, __pyx_L1_error)
            }
            (__pyx_v_feature_grad1[__pyx_t_26]) = ((__pyx_v_feature_grad1[__pyx_t_26]) + ((__pyx_v_value / __pyx_v_Rij) * (__pyx_v_RijVec.coord[__pyx_v_m])));
          }
          __pyx_L21_continue:;
        }
        __pyx_L15_continue:;
      }
    }
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":578
 *         # Convert radial feature to numpy array
 *         cdef int grad_index
 *         feature_grad1_np = np.zeros((Natoms*dim, Nelements_2body))             # <<<<<<<<<<<<<<
 *         for m in range(Nelements_2body):
 *             for grad_index in range(Natoms*dim):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_int((__pyx_v_Natoms * __pyx_v_dim)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_Nelements_2body); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_6);
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_12);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_feature_grad1_np = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":579
 *         cdef int grad_index
 *         feature_grad1_np = np.zeros((Natoms*dim, Nelements_2body))
 *         for m in range(Nelements_2body):             # <<<<<<<<<<<<<<
 *             for grad_index in range(Natoms*dim):
 *                 feature_grad1_np[grad_index][m] = feature_grad1[m * Natoms*dim + grad_index]
 */
  __pyx_t_3 = __pyx_v_Nelements_2body;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":580
 *         feature_grad1_np = np.zeros((Natoms*dim, Nelements_2body))
 *         for m in range(Nelements_2body):
 *             for grad_index in range(Natoms*dim):             # <<<<<<<<<<<<<<
 *                 feature_grad1_np[grad_index][m] = feature_grad1[m * Natoms*dim + grad_index]
 * 
 */
    __pyx_t_11 = (__pyx_v_Natoms * __pyx_v_dim);
    __pyx_t_13 = __pyx_t_11;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_grad_index = __pyx_t_14;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":581
 *         for m in range(Nelements_2body):
 *             for grad_index in range(Natoms*dim):
 *                 feature_grad1_np[grad_index][m] = feature_grad1[m * Natoms*dim + grad_index]             # <<<<<<<<<<<<<<
 * 
 *         # Return feature if only radial part is desired
 */
      __pyx_t_1 = PyFloat_FromDouble((__pyx_v_feature_grad1[(((__pyx_v_m * __pyx_v_Natoms) * __pyx_v_dim) + __pyx_v_grad_index)])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 581, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_feature_grad1_np, __pyx_v_grad_index, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 581, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(__Pyx_SetItemInt(__pyx_t_4, __pyx_v_m, __pyx_t_1, int, 1, __Pyx_PyInt_From_int, 0, 1, 1) < 0)) __PYX_ERR(0, 581, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":584
 * 
 *         # Return feature if only radial part is desired
 *         if not use_angular:             # <<<<<<<<<<<<<<
 *             return feature_grad1_np
 * 
 */
  __pyx_t_18 = __Pyx_PyObject_IsTrue(__pyx_v_use_angular); if (unlikely(__pyx_t_18 < 0)) __PYX_ERR(0, 584, __pyx_L1_error)
  __pyx_t_19 = ((!__pyx_t_18) != 0);
  if (__pyx_t_19) {

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":585
 *         # Return feature if only radial part is desired
 *         if not use_angular:
 *             return feature_grad1_np             # <<<<<<<<<<<<<<
 * 
 *         # ANGULAR FEATURE-GRADIENT
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_feature_grad1_np);
    __pyx_r = __pyx_v_feature_grad1_np;
    goto __pyx_L0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":584
 * 
 *         # Return feature if only radial part is desired
 *         if not use_angular:             # <<<<<<<<<<<<<<
 *             return feature_grad1_np
 * 
 */
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":590
 * 
 *         # Convert 3body bondtype list into c-array
 *         cdef int Nbondtypes_3body = self.Nbondtypes_3body             # <<<<<<<<<<<<<<
 *         cdef list bondtypes_3body_old = self.bondtypes_3body
 *         cdef int *bondtypes_3body
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Nbondtypes_3body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Nbondtypes_3body = __pyx_t_3;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":591
 *         # Convert 3body bondtype list into c-array
 *         cdef int Nbondtypes_3body = self.Nbondtypes_3body
 *         cdef list bondtypes_3body_old = self.bondtypes_3body             # <<<<<<<<<<<<<<
 *         cdef int *bondtypes_3body
 *         bondtypes_3body = <int*>mem.alloc(Ntypes*Ntypes*Ntypes, sizeof(int))
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_bondtypes_3body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 591, __pyx_L1_error)
  __pyx_v_bondtypes_3body_old = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":593
 *         cdef list bondtypes_3body_old = self.bondtypes_3body
 *         cdef int *bondtypes_3body
 *         bondtypes_3body = <int*>mem.alloc(Ntypes*Ntypes*Ntypes, sizeof(int))             # <<<<<<<<<<<<<<
 *         for m in range(Ntypes*Ntypes*Ntypes):
 *             bondtypes_3body[m] = bondtypes_3body_old[m]
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, ((__pyx_v_Ntypes * __pyx_v_Ntypes) * __pyx_v_Ntypes), (sizeof(int))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 593, __pyx_L1_error)
  __pyx_v_bondtypes_3body = ((int *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":594
 *         cdef int *bondtypes_3body
 *         bondtypes_3body = <int*>mem.alloc(Ntypes*Ntypes*Ntypes, sizeof(int))
 *         for m in range(Ntypes*Ntypes*Ntypes):             # <<<<<<<<<<<<<<
 *             bondtypes_3body[m] = bondtypes_3body_old[m]
 * 
 */
  __pyx_t_3 = ((__pyx_v_Ntypes * __pyx_v_Ntypes) * __pyx_v_Ntypes);
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":595
 *         bondtypes_3body = <int*>mem.alloc(Ntypes*Ntypes*Ntypes, sizeof(int))
 *         for m in range(Ntypes*Ntypes*Ntypes):
 *             bondtypes_3body[m] = bondtypes_3body_old[m]             # <<<<<<<<<<<<<<
 * 
 *         # Initialize angular feature-gradient
 */
    if (unlikely(__pyx_v_bondtypes_3body_old == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 595, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_bondtypes_3body_old, __pyx_v_m, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    (__pyx_v_bondtypes_3body[__pyx_v_m]) = __pyx_t_11;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":599
 *         # Initialize angular feature-gradient
 *         cdef double *feature_grad2
 *         feature_grad2 = <double*>mem.alloc(Nelements_3body * Natoms * dim, sizeof(double))             # <<<<<<<<<<<<<<
 * 
 *         cdef Point RikVec, angle_grad_i, angle_grad_j, angle_grad_k
 */
  __pyx_t_7 = ((struct __pyx_vtabstruct_5cymem_5cymem_Pool *)__pyx_v_mem->__pyx_vtab)->alloc(__pyx_v_mem, ((__pyx_v_Nelements_3body * __pyx_v_Natoms) * __pyx_v_dim), (sizeof(double))); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 599, __pyx_L1_error)
  __pyx_v_feature_grad2 = ((double *)__pyx_t_7);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":604
 *         cdef double angle, cos_angle, a
 *         cdef int k, cond_ij, cond_ik, bin_index, type3
 *         for i in range(Natoms):             # <<<<<<<<<<<<<<
 *             pos_i = pos[i]
 *             for cell_index1 in range(Ncells):
 */
  __pyx_t_3 = __pyx_v_Natoms;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":605
 *         cdef int k, cond_ij, cond_ik, bin_index, type3
 *         for i in range(Natoms):
 *             pos_i = pos[i]             # <<<<<<<<<<<<<<
 *             for cell_index1 in range(Ncells):
 *                 displacement1 = cell_displacements[cell_index1]
 */
    __pyx_v_pos_i = (__pyx_v_pos[__pyx_v_i]);

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":606
 *         for i in range(Natoms):
 *             pos_i = pos[i]
 *             for cell_index1 in range(Ncells):             # <<<<<<<<<<<<<<
 *                 displacement1 = cell_displacements[cell_index1]
 *                 for j in range(Natoms):
 */
    __pyx_t_11 = __pyx_v_Ncells;
    __pyx_t_13 = __pyx_t_11;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_cell_index1 = __pyx_t_14;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":607
 *             pos_i = pos[i]
 *             for cell_index1 in range(Ncells):
 *                 displacement1 = cell_displacements[cell_index1]             # <<<<<<<<<<<<<<
 *                 for j in range(Natoms):
 *                     pos_j = add(pos[j], displacement1)
 */
      __pyx_v_displacement1 = (__pyx_v_cell_displacements[__pyx_v_cell_index1]);

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":608
 *             for cell_index1 in range(Ncells):
 *                 displacement1 = cell_displacements[cell_index1]
 *                 for j in range(Natoms):             # <<<<<<<<<<<<<<
 *                     pos_j = add(pos[j], displacement1)
 *                     Rij = euclidean(pos[i], pos_j)
 */
      __pyx_t_15 = __pyx_v_Natoms;
      __pyx_t_16 = __pyx_t_15;
      for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
        __pyx_v_j = __pyx_t_17;

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":609
 *                 displacement1 = cell_displacements[cell_index1]
 *                 for j in range(Natoms):
 *                     pos_j = add(pos[j], displacement1)             # <<<<<<<<<<<<<<
 *                     Rij = euclidean(pos[i], pos_j)
 *                     if Rij > Rc2 or Rij < 1e-6:
 */
        __pyx_v_pos_j = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_add((__pyx_v_pos[__pyx_v_j]), __pyx_v_displacement1);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":610
 *                 for j in range(Natoms):
 *                     pos_j = add(pos[j], displacement1)
 *                     Rij = euclidean(pos[i], pos_j)             # <<<<<<<<<<<<<<
 *                     if Rij > Rc2 or Rij < 1e-6:
 *                         continue
 */
        __pyx_v_Rij = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_euclidean((__pyx_v_pos[__pyx_v_i]), __pyx_v_pos_j);

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":611
 *                     pos_j = add(pos[j], displacement1)
 *                     Rij = euclidean(pos[i], pos_j)
 *                     if Rij > Rc2 or Rij < 1e-6:             # <<<<<<<<<<<<<<
 *                         continue
 *                     for cell_index2 in range(cell_index1, Ncells):
 */
        __pyx_t_18 = ((__pyx_v_Rij > __pyx_v_Rc2) != 0);
        if (!__pyx_t_18) {
        } else {
          __pyx_t_19 = __pyx_t_18;
          goto __pyx_L42_bool_binop_done;
        }
        __pyx_t_18 = ((__pyx_v_Rij < 1e-6) != 0);
        __pyx_t_19 = __pyx_t_18;
        __pyx_L42_bool_binop_done:;
        if (__pyx_t_19) {

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":612
 *                     Rij = euclidean(pos[i], pos_j)
 *                     if Rij > Rc2 or Rij < 1e-6:
 *                         continue             # <<<<<<<<<<<<<<
 *                     for cell_index2 in range(cell_index1, Ncells):
 *                         displacement2 = cell_displacements[cell_index2]
 */
          goto __pyx_L39_continue;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":611
 *                     pos_j = add(pos[j], displacement1)
 *                     Rij = euclidean(pos[i], pos_j)
 *                     if Rij > Rc2 or Rij < 1e-6:             # <<<<<<<<<<<<<<
 *                         continue
 *                     for cell_index2 in range(cell_index1, Ncells):
 */
        }

        /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":613
 *                     if Rij > Rc2 or Rij < 1e-6:
 *                         continue
 *                     for cell_index2 in range(cell_index1, Ncells):             # <<<<<<<<<<<<<<
 *                         displacement2 = cell_displacements[cell_index2]
 *                         if cell_index1 == cell_index2:
 */
        __pyx_t_23 = __pyx_v_Ncells;
        __pyx_t_25 = __pyx_t_23;
        for (__pyx_t_26 = __pyx_v_cell_index1; __pyx_t_26 < __pyx_t_25; __pyx_t_26+=1) {
          __pyx_v_cell_index2 = __pyx_t_26;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":614
 *                         continue
 *                     for cell_index2 in range(cell_index1, Ncells):
 *                         displacement2 = cell_displacements[cell_index2]             # <<<<<<<<<<<<<<
 *                         if cell_index1 == cell_index2:
 *                             k_start = j+1
 */
          __pyx_v_displacement2 = (__pyx_v_cell_displacements[__pyx_v_cell_index2]);

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":615
 *                     for cell_index2 in range(cell_index1, Ncells):
 *                         displacement2 = cell_displacements[cell_index2]
 *                         if cell_index1 == cell_index2:             # <<<<<<<<<<<<<<
 *                             k_start = j+1
 *                         else:
 */
          __pyx_t_19 = ((__pyx_v_cell_index1 == __pyx_v_cell_index2) != 0);
          if (__pyx_t_19) {

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":616
 *                         displacement2 = cell_displacements[cell_index2]
 *                         if cell_index1 == cell_index2:
 *                             k_start = j+1             # <<<<<<<<<<<<<<
 *                         else:
 *                             k_start = 0
 */
            __pyx_v_k_start = (__pyx_v_j + 1);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":615
 *                     for cell_index2 in range(cell_index1, Ncells):
 *                         displacement2 = cell_displacements[cell_index2]
 *                         if cell_index1 == cell_index2:             # <<<<<<<<<<<<<<
 *                             k_start = j+1
 *                         else:
 */
            goto __pyx_L46;
          }

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":618
 *                             k_start = j+1
 *                         else:
 *                             k_start = 0             # <<<<<<<<<<<<<<
 *                         for k in range(k_start, Natoms):
 *                             pos_k = add(pos[k], displacement2)
 */
          /*else*/ {
            __pyx_v_k_start = 0;
          }
          __pyx_L46:;

          /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":619
 *                         else:
 *                             k_start = 0
 *                         for k in range(k_start, Natoms):             # <<<<<<<<<<<<<<
 *                             pos_k = add(pos[k], displacement2)
 *                             Rik = euclidean(pos_i, pos_k)
 */
          __pyx_t_27 = __pyx_v_Natoms;
          __pyx_t_28 = __pyx_t_27;
          for (__pyx_t_29 = __pyx_v_k_start; __pyx_t_29 < __pyx_t_28; __pyx_t_29+=1) {
            __pyx_v_k = __pyx_t_29;

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":620
 *                             k_start = 0
 *                         for k in range(k_start, Natoms):
 *                             pos_k = add(pos[k], displacement2)             # <<<<<<<<<<<<<<
 *                             Rik = euclidean(pos_i, pos_k)
 *                             if Rik > Rc2 or Rik < 1e-6:
 */
            __pyx_v_pos_k = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_add((__pyx_v_pos[__pyx_v_k]), __pyx_v_displacement2);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":621
 *                         for k in range(k_start, Natoms):
 *                             pos_k = add(pos[k], displacement2)
 *                             Rik = euclidean(pos_i, pos_k)             # <<<<<<<<<<<<<<
 *                             if Rik > Rc2 or Rik < 1e-6:
 *                                 continue
 */
            __pyx_v_Rik = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_euclidean(__pyx_v_pos_i, __pyx_v_pos_k);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":622
 *                             pos_k = add(pos[k], displacement2)
 *                             Rik = euclidean(pos_i, pos_k)
 *                             if Rik > Rc2 or Rik < 1e-6:             # <<<<<<<<<<<<<<
 *                                 continue
 * 
 */
            __pyx_t_18 = ((__pyx_v_Rik > __pyx_v_Rc2) != 0);
            if (!__pyx_t_18) {
            } else {
              __pyx_t_19 = __pyx_t_18;
              goto __pyx_L50_bool_binop_done;
            }
            __pyx_t_18 = ((__pyx_v_Rik < 1e-6) != 0);
            __pyx_t_19 = __pyx_t_18;
            __pyx_L50_bool_binop_done:;
            if (__pyx_t_19) {

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":623
 *                             Rik = euclidean(pos_i, pos_k)
 *                             if Rik > Rc2 or Rik < 1e-6:
 *                                 continue             # <<<<<<<<<<<<<<
 * 
 *                             # determine bondtype
 */
              goto __pyx_L47_continue;

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":622
 *                             pos_k = add(pos[k], displacement2)
 *                             Rik = euclidean(pos_i, pos_k)
 *                             if Rik > Rc2 or Rik < 1e-6:             # <<<<<<<<<<<<<<
 *                                 continue
 * 
 */
            }

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":626
 * 
 *                             # determine bondtype
 *                             type1 = num_converted[i]             # <<<<<<<<<<<<<<
 *                             if num_converted[j] <= num_converted[k]:
 *                                 type2 = num_converted[j]
 */
            __pyx_v_type1 = (__pyx_v_num_converted[__pyx_v_i]);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":627
 *                             # determine bondtype
 *                             type1 = num_converted[i]
 *                             if num_converted[j] <= num_converted[k]:             # <<<<<<<<<<<<<<
 *                                 type2 = num_converted[j]
 *                                 type3 = num_converted[k]
 */
            __pyx_t_19 = (((__pyx_v_num_converted[__pyx_v_j]) <= (__pyx_v_num_converted[__pyx_v_k])) != 0);
            if (__pyx_t_19) {

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":628
 *                             type1 = num_converted[i]
 *                             if num_converted[j] <= num_converted[k]:
 *                                 type2 = num_converted[j]             # <<<<<<<<<<<<<<
 *                                 type3 = num_converted[k]
 *                             else:
 */
              __pyx_v_type2 = (__pyx_v_num_converted[__pyx_v_j]);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":629
 *                             if num_converted[j] <= num_converted[k]:
 *                                 type2 = num_converted[j]
 *                                 type3 = num_converted[k]             # <<<<<<<<<<<<<<
 *                             else:
 *                                 type2 = num_converted[k]
 */
              __pyx_v_type3 = (__pyx_v_num_converted[__pyx_v_k]);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":627
 *                             # determine bondtype
 *                             type1 = num_converted[i]
 *                             if num_converted[j] <= num_converted[k]:             # <<<<<<<<<<<<<<
 *                                 type2 = num_converted[j]
 *                                 type3 = num_converted[k]
 */
              goto __pyx_L52;
            }

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":631
 *                                 type3 = num_converted[k]
 *                             else:
 *                                 type2 = num_converted[k]             # <<<<<<<<<<<<<<
 *                                 type3 = num_converted[j]
 *                             bondtype_index = Nbins2*bondtypes_3body[Ntypes*Ntypes*type1 + Ntypes*type2 + type3]
 */
            /*else*/ {
              __pyx_v_type2 = (__pyx_v_num_converted[__pyx_v_k]);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":632
 *                             else:
 *                                 type2 = num_converted[k]
 *                                 type3 = num_converted[j]             # <<<<<<<<<<<<<<
 *                             bondtype_index = Nbins2*bondtypes_3body[Ntypes*Ntypes*type1 + Ntypes*type2 + type3]
 * 
 */
              __pyx_v_type3 = (__pyx_v_num_converted[__pyx_v_j]);
            }
            __pyx_L52:;

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":633
 *                                 type2 = num_converted[k]
 *                                 type3 = num_converted[j]
 *                             bondtype_index = Nbins2*bondtypes_3body[Ntypes*Ntypes*type1 + Ntypes*type2 + type3]             # <<<<<<<<<<<<<<
 * 
 *                             # Calculate angle
 */
            __pyx_v_bondtype_index = (__pyx_v_Nbins2 * (__pyx_v_bondtypes_3body[((((__pyx_v_Ntypes * __pyx_v_Ntypes) * __pyx_v_type1) + (__pyx_v_Ntypes * __pyx_v_type2)) + __pyx_v_type3)]));

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":636
 * 
 *                             # Calculate angle
 *                             RijVec = subtract(pos_j,pos_i)             # <<<<<<<<<<<<<<
 *                             RikVec = subtract(pos_k, pos_i)
 * 
 */
            __pyx_v_RijVec = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_subtract(__pyx_v_pos_j, __pyx_v_pos_i);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":637
 *                             # Calculate angle
 *                             RijVec = subtract(pos_j,pos_i)
 *                             RikVec = subtract(pos_k, pos_i)             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_v_RikVec = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_subtract(__pyx_v_pos_k, __pyx_v_pos_i);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":640
 * 
 * 
 *                             angle = get_angle(RijVec, RikVec)             # <<<<<<<<<<<<<<
 *                             cos_angle = cos(angle)
 * 
 */
            __pyx_v_angle = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_get_angle(__pyx_v_RijVec, __pyx_v_RikVec);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":641
 * 
 *                             angle = get_angle(RijVec, RikVec)
 *                             cos_angle = cos(angle)             # <<<<<<<<<<<<<<
 * 
 *                             for m in range(3):
 */
            __pyx_v_cos_angle = cos(__pyx_v_angle);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":643
 *                             cos_angle = cos(angle)
 * 
 *                             for m in range(3):             # <<<<<<<<<<<<<<
 *                                 if not (angle == 0 or angle == M_PI):
 *                                     a = -1/sqrt(1 - cos_angle*cos_angle)
 */
            for (__pyx_t_30 = 0; __pyx_t_30 < 3; __pyx_t_30+=1) {
              __pyx_v_m = __pyx_t_30;

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":644
 * 
 *                             for m in range(3):
 *                                 if not (angle == 0 or angle == M_PI):             # <<<<<<<<<<<<<<
 *                                     a = -1/sqrt(1 - cos_angle*cos_angle)
 *                                     angle_grad_j.coord[m] = a * (RikVec.coord[m]/(Rij*Rik) - cos_angle*RijVec.coord[m]/(Rij*Rij))
 */
              __pyx_t_18 = ((__pyx_v_angle == 0.0) != 0);
              if (!__pyx_t_18) {
              } else {
                __pyx_t_19 = __pyx_t_18;
                goto __pyx_L56_bool_binop_done;
              }
              __pyx_t_18 = ((__pyx_v_angle == M_PI) != 0);
              __pyx_t_19 = __pyx_t_18;
              __pyx_L56_bool_binop_done:;
              __pyx_t_18 = ((!__pyx_t_19) != 0);
              if (__pyx_t_18) {

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":645
 *                             for m in range(3):
 *                                 if not (angle == 0 or angle == M_PI):
 *                                     a = -1/sqrt(1 - cos_angle*cos_angle)             # <<<<<<<<<<<<<<
 *                                     angle_grad_j.coord[m] = a * (RikVec.coord[m]/(Rij*Rik) - cos_angle*RijVec.coord[m]/(Rij*Rij))
 *                                     angle_grad_k.coord[m] = a * (RijVec.coord[m]/(Rij*Rik) - cos_angle*RikVec.coord[m]/(Rik*Rik))
 */
                __pyx_t_20 = sqrt((1.0 - (__pyx_v_cos_angle * __pyx_v_cos_angle)));
                if (unlikely(__pyx_t_20 == 0)) {
                  PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                  __PYX_ERR(0, 645, __pyx_L1_error)
                }
                __pyx_v_a = (-1.0 / __pyx_t_20);

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":646
 *                                 if not (angle == 0 or angle == M_PI):
 *                                     a = -1/sqrt(1 - cos_angle*cos_angle)
 *                                     angle_grad_j.coord[m] = a * (RikVec.coord[m]/(Rij*Rik) - cos_angle*RijVec.coord[m]/(Rij*Rij))             # <<<<<<<<<<<<<<
 *                                     angle_grad_k.coord[m] = a * (RijVec.coord[m]/(Rij*Rik) - cos_angle*RikVec.coord[m]/(Rik*Rik))
 *                                     angle_grad_i.coord[m] = -(angle_grad_j.coord[m] + angle_grad_k.coord[m])
 */
                __pyx_t_20 = (__pyx_v_Rij * __pyx_v_Rik);
                if (unlikely(__pyx_t_20 == 0)) {
                  PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                  __PYX_ERR(0, 646, __pyx_L1_error)
                }
                __pyx_t_24 = (__pyx_v_cos_angle * (__pyx_v_RijVec.coord[__pyx_v_m]));
                __pyx_t_2 = (__pyx_v_Rij * __pyx_v_Rij);
                if (unlikely(__pyx_t_2 == 0)) {
                  PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                  __PYX_ERR(0, 646, __pyx_L1_error)
                }
                (__pyx_v_angle_grad_j.coord[__pyx_v_m]) = (__pyx_v_a * (((__pyx_v_RikVec.coord[__pyx_v_m]) / __pyx_t_20) - (__pyx_t_24 / __pyx_t_2)));

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":647
 *                                     a = -1/sqrt(1 - cos_angle*cos_angle)
 *                                     angle_grad_j.coord[m] = a * (RikVec.coord[m]/(Rij*Rik) - cos_angle*RijVec.coord[m]/(Rij*Rij))
 *                                     angle_grad_k.coord[m] = a * (RijVec.coord[m]/(Rij*Rik) - cos_angle*RikVec.coord[m]/(Rik*Rik))             # <<<<<<<<<<<<<<
 *                                     angle_grad_i.coord[m] = -(angle_grad_j.coord[m] + angle_grad_k.coord[m])
 *                                 else:
 */
                __pyx_t_2 = (__pyx_v_Rij * __pyx_v_Rik);
                if (unlikely(__pyx_t_2 == 0)) {
                  PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                  __PYX_ERR(0, 647, __pyx_L1_error)
                }
                __pyx_t_24 = (__pyx_v_cos_angle * (__pyx_v_RikVec.coord[__pyx_v_m]));
                __pyx_t_20 = (__pyx_v_Rik * __pyx_v_Rik);
                if (unlikely(__pyx_t_20 == 0)) {
                  PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                  __PYX_ERR(0, 647, __pyx_L1_error)
                }
                (__pyx_v_angle_grad_k.coord[__pyx_v_m]) = (__pyx_v_a * (((__pyx_v_RijVec.coord[__pyx_v_m]) / __pyx_t_2) - (__pyx_t_24 / __pyx_t_20)));

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":648
 *                                     angle_grad_j.coord[m] = a * (RikVec.coord[m]/(Rij*Rik) - cos_angle*RijVec.coord[m]/(Rij*Rij))
 *                                     angle_grad_k.coord[m] = a * (RijVec.coord[m]/(Rij*Rik) - cos_angle*RikVec.coord[m]/(Rik*Rik))
 *                                     angle_grad_i.coord[m] = -(angle_grad_j.coord[m] + angle_grad_k.coord[m])             # <<<<<<<<<<<<<<
 *                                 else:
 *                                     angle_grad_j.coord[m] = 0
 */
                (__pyx_v_angle_grad_i.coord[__pyx_v_m]) = (-((__pyx_v_angle_grad_j.coord[__pyx_v_m]) + (__pyx_v_angle_grad_k.coord[__pyx_v_m])));

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":644
 * 
 *                             for m in range(3):
 *                                 if not (angle == 0 or angle == M_PI):             # <<<<<<<<<<<<<<
 *                                     a = -1/sqrt(1 - cos_angle*cos_angle)
 *                                     angle_grad_j.coord[m] = a * (RikVec.coord[m]/(Rij*Rik) - cos_angle*RijVec.coord[m]/(Rij*Rij))
 */
                goto __pyx_L55;
              }

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":650
 *                                     angle_grad_i.coord[m] = -(angle_grad_j.coord[m] + angle_grad_k.coord[m])
 *                                 else:
 *                                     angle_grad_j.coord[m] = 0             # <<<<<<<<<<<<<<
 *                                     angle_grad_k.coord[m] = 0
 *                                     angle_grad_i.coord[m] = 0
 */
              /*else*/ {
                (__pyx_v_angle_grad_j.coord[__pyx_v_m]) = 0.0;

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":651
 *                                 else:
 *                                     angle_grad_j.coord[m] = 0
 *                                     angle_grad_k.coord[m] = 0             # <<<<<<<<<<<<<<
 *                                     angle_grad_i.coord[m] = 0
 * 
 */
                (__pyx_v_angle_grad_k.coord[__pyx_v_m]) = 0.0;

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":652
 *                                     angle_grad_j.coord[m] = 0
 *                                     angle_grad_k.coord[m] = 0
 *                                     angle_grad_i.coord[m] = 0             # <<<<<<<<<<<<<<
 * 
 *                             fc_ij = f_cutoff(Rij, gamma, Rc2)
 */
                (__pyx_v_angle_grad_i.coord[__pyx_v_m]) = 0.0;
              }
              __pyx_L55:;
            }

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":654
 *                                     angle_grad_i.coord[m] = 0
 * 
 *                             fc_ij = f_cutoff(Rij, gamma, Rc2)             # <<<<<<<<<<<<<<
 *                             fc_ik = f_cutoff(Rik, gamma, Rc2)
 *                             fc_grad_ij = f_cutoff_grad(Rij, gamma, Rc2)
 */
            __pyx_v_fc_ij = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_f_cutoff(__pyx_v_Rij, __pyx_v_gamma, __pyx_v_Rc2);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":655
 * 
 *                             fc_ij = f_cutoff(Rij, gamma, Rc2)
 *                             fc_ik = f_cutoff(Rik, gamma, Rc2)             # <<<<<<<<<<<<<<
 *                             fc_grad_ij = f_cutoff_grad(Rij, gamma, Rc2)
 *                             fc_grad_ik = f_cutoff_grad(Rik, gamma, Rc2)
 */
            __pyx_v_fc_ik = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_f_cutoff(__pyx_v_Rik, __pyx_v_gamma, __pyx_v_Rc2);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":656
 *                             fc_ij = f_cutoff(Rij, gamma, Rc2)
 *                             fc_ik = f_cutoff(Rik, gamma, Rc2)
 *                             fc_grad_ij = f_cutoff_grad(Rij, gamma, Rc2)             # <<<<<<<<<<<<<<
 *                             fc_grad_ik = f_cutoff_grad(Rik, gamma, Rc2)
 * 
 */
            __pyx_v_fc_grad_ij = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_f_cutoff_grad(__pyx_v_Rij, __pyx_v_gamma, __pyx_v_Rc2);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":657
 *                             fc_ik = f_cutoff(Rik, gamma, Rc2)
 *                             fc_grad_ij = f_cutoff_grad(Rij, gamma, Rc2)
 *                             fc_grad_ik = f_cutoff_grad(Rik, gamma, Rc2)             # <<<<<<<<<<<<<<
 * 
 *                             # Calculate normalization
 */
            __pyx_v_fc_grad_ik = __pyx_f_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_f_cutoff_grad(__pyx_v_Rik, __pyx_v_gamma, __pyx_v_Rc2);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":660
 * 
 *                             # Calculate normalization
 *                             num_pairs = Natoms*Natoms*Natoms             # <<<<<<<<<<<<<<
 *                             normalization = 1./smearing_norm2
 *                             normalization /= num_pairs/volume
 */
            __pyx_v_num_pairs = ((__pyx_v_Natoms * __pyx_v_Natoms) * __pyx_v_Natoms);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":661
 *                             # Calculate normalization
 *                             num_pairs = Natoms*Natoms*Natoms
 *                             normalization = 1./smearing_norm2             # <<<<<<<<<<<<<<
 *                             normalization /= num_pairs/volume
 * 
 */
            if (unlikely(__pyx_v_smearing_norm2 == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 661, __pyx_L1_error)
            }
            __pyx_v_normalization = (1. / __pyx_v_smearing_norm2);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":662
 *                             num_pairs = Natoms*Natoms*Natoms
 *                             normalization = 1./smearing_norm2
 *                             normalization /= num_pairs/volume             # <<<<<<<<<<<<<<
 * 
 *                             # Identify what bin 'Rij' belongs to + it's position in this bin
 */
            if (unlikely(__pyx_v_volume == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 662, __pyx_L1_error)
            }
            __pyx_t_20 = (__pyx_v_num_pairs / __pyx_v_volume);
            if (unlikely(__pyx_t_20 == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 662, __pyx_L1_error)
            }
            __pyx_v_normalization = (__pyx_v_normalization / __pyx_t_20);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":665
 * 
 *                             # Identify what bin 'Rij' belongs to + it's position in this bin
 *                             center_bin = <int> floor(angle/binwidth1)             # <<<<<<<<<<<<<<
 *                             binpos = angle/binwidth2 - center_bin
 * 
 */
            if (unlikely(__pyx_v_binwidth1 == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 665, __pyx_L1_error)
            }
            __pyx_v_center_bin = ((int)floor((__pyx_v_angle / __pyx_v_binwidth1)));

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":666
 *                             # Identify what bin 'Rij' belongs to + it's position in this bin
 *                             center_bin = <int> floor(angle/binwidth1)
 *                             binpos = angle/binwidth2 - center_bin             # <<<<<<<<<<<<<<
 * 
 *                             # Lower and upper range of bins affected by the current atomic distance deltaR.
 */
            if (unlikely(__pyx_v_binwidth2 == 0)) {
              PyErr_SetString(PyExc_ZeroDivisionError, "float division");
              __PYX_ERR(0, 666, __pyx_L1_error)
            }
            __pyx_v_binpos = ((__pyx_v_angle / __pyx_v_binwidth2) - __pyx_v_center_bin);

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":669
 * 
 *                             # Lower and upper range of bins affected by the current atomic distance deltaR.
 *                             minbin_lim = <int> -ceil(m2 - binpos)             # <<<<<<<<<<<<<<
 *                             maxbin_lim = <int> ceil(m2 - (1-binpos))
 * 
 */
            __pyx_v_minbin_lim = ((int)(-ceil((__pyx_v_m2 - __pyx_v_binpos))));

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":670
 *                             # Lower and upper range of bins affected by the current atomic distance deltaR.
 *                             minbin_lim = <int> -ceil(m2 - binpos)
 *                             maxbin_lim = <int> ceil(m2 - (1-binpos))             # <<<<<<<<<<<<<<
 * 
 *                             for n in range(minbin_lim, maxbin_lim + 1):
 */
            __pyx_v_maxbin_lim = ((int)ceil((__pyx_v_m2 - (1.0 - __pyx_v_binpos))));

            /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":672
 *                             maxbin_lim = <int> ceil(m2 - (1-binpos))
 * 
 *                             for n in range(minbin_lim, maxbin_lim + 1):             # <<<<<<<<<<<<<<
 *                                 newbin = center_bin + n
 * 
 */
            __pyx_t_21 = (__pyx_v_maxbin_lim + 1);
            __pyx_t_22 = __pyx_t_21;
            for (__pyx_t_30 = __pyx_v_minbin_lim; __pyx_t_30 < __pyx_t_22; __pyx_t_30+=1) {
              __pyx_v_n = __pyx_t_30;

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":673
 * 
 *                             for n in range(minbin_lim, maxbin_lim + 1):
 *                                 newbin = center_bin + n             # <<<<<<<<<<<<<<
 * 
 *                                 # Wrap current bin into correct bin-range
 */
              __pyx_v_newbin = (__pyx_v_center_bin + __pyx_v_n);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":676
 * 
 *                                 # Wrap current bin into correct bin-range
 *                                 if newbin < 0:             # <<<<<<<<<<<<<<
 *                                     newbin = abs(newbin)
 *                                 if newbin > Nbins2-1:
 */
              __pyx_t_18 = ((__pyx_v_newbin < 0) != 0);
              if (__pyx_t_18) {

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":677
 *                                 # Wrap current bin into correct bin-range
 *                                 if newbin < 0:
 *                                     newbin = abs(newbin)             # <<<<<<<<<<<<<<
 *                                 if newbin > Nbins2-1:
 *                                     newbin = 2*Nbins2 - newbin - 1
 */
                __pyx_t_31 = abs(__pyx_v_newbin); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 677, __pyx_L1_error)
                __pyx_v_newbin = __pyx_t_31;

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":676
 * 
 *                                 # Wrap current bin into correct bin-range
 *                                 if newbin < 0:             # <<<<<<<<<<<<<<
 *                                     newbin = abs(newbin)
 *                                 if newbin > Nbins2-1:
 */
              }

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":678
 *                                 if newbin < 0:
 *                                     newbin = abs(newbin)
 *                                 if newbin > Nbins2-1:             # <<<<<<<<<<<<<<
 *                                     newbin = 2*Nbins2 - newbin - 1
 * 
 */
              __pyx_t_18 = ((__pyx_v_newbin > (__pyx_v_Nbins2 - 1)) != 0);
              if (__pyx_t_18) {

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":679
 *                                     newbin = abs(newbin)
 *                                 if newbin > Nbins2-1:
 *                                     newbin = 2*Nbins2 - newbin - 1             # <<<<<<<<<<<<<<
 * 
 *                                 # Calculate gauss contribution to current bin
 */
                __pyx_v_newbin = (((2 * __pyx_v_Nbins2) - __pyx_v_newbin) - 1);

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":678
 *                                 if newbin < 0:
 *                                     newbin = abs(newbin)
 *                                 if newbin > Nbins2-1:             # <<<<<<<<<<<<<<
 *                                     newbin = 2*Nbins2 - newbin - 1
 * 
 */
              }

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":682
 * 
 *                                 # Calculate gauss contribution to current bin
 *                                 c = 1./sqrt(2)*binwidth2/sigma2             # <<<<<<<<<<<<<<
 *                                 arg_low = max(-m2, n-binpos)
 *                                 arg_up = min(m2, n+(1-binpos))
 */
              __pyx_t_20 = sqrt(2.0);
              if (unlikely(__pyx_t_20 == 0)) {
                PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                __PYX_ERR(0, 682, __pyx_L1_error)
              }
              __pyx_t_24 = ((1. / __pyx_t_20) * __pyx_v_binwidth2);
              if (unlikely(__pyx_v_sigma2 == 0)) {
                PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                __PYX_ERR(0, 682, __pyx_L1_error)
              }
              __pyx_v_c = (__pyx_t_24 / __pyx_v_sigma2);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":683
 *                                 # Calculate gauss contribution to current bin
 *                                 c = 1./sqrt(2)*binwidth2/sigma2
 *                                 arg_low = max(-m2, n-binpos)             # <<<<<<<<<<<<<<
 *                                 arg_up = min(m2, n+(1-binpos))
 *                                 value1 = 0.5*erf(c*arg_up)-0.5*erf(c*arg_low)
 */
              __pyx_t_24 = (__pyx_v_n - __pyx_v_binpos);
              __pyx_t_20 = (-__pyx_v_m2);
              if (((__pyx_t_24 > __pyx_t_20) != 0)) {
                __pyx_t_2 = __pyx_t_24;
              } else {
                __pyx_t_2 = __pyx_t_20;
              }
              __pyx_v_arg_low = __pyx_t_2;

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":684
 *                                 c = 1./sqrt(2)*binwidth2/sigma2
 *                                 arg_low = max(-m2, n-binpos)
 *                                 arg_up = min(m2, n+(1-binpos))             # <<<<<<<<<<<<<<
 *                                 value1 = 0.5*erf(c*arg_up)-0.5*erf(c*arg_low)
 *                                 value2 = -1./(sigma2*sqrt(2*M_PI)) * (exp(-pow(c*arg_up, 2)) - exp(-pow(c*arg_low, 2)))
 */
              __pyx_t_2 = (__pyx_v_n + (1.0 - __pyx_v_binpos));
              __pyx_t_24 = __pyx_v_m2;
              if (((__pyx_t_2 < __pyx_t_24) != 0)) {
                __pyx_t_20 = __pyx_t_2;
              } else {
                __pyx_t_20 = __pyx_t_24;
              }
              __pyx_v_arg_up = __pyx_t_20;

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":685
 *                                 arg_low = max(-m2, n-binpos)
 *                                 arg_up = min(m2, n+(1-binpos))
 *                                 value1 = 0.5*erf(c*arg_up)-0.5*erf(c*arg_low)             # <<<<<<<<<<<<<<
 *                                 value2 = -1./(sigma2*sqrt(2*M_PI)) * (exp(-pow(c*arg_up, 2)) - exp(-pow(c*arg_low, 2)))
 * 
 */
              __pyx_v_value1 = ((0.5 * erf((__pyx_v_c * __pyx_v_arg_up))) - (0.5 * erf((__pyx_v_c * __pyx_v_arg_low))));

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":686
 *                                 arg_up = min(m2, n+(1-binpos))
 *                                 value1 = 0.5*erf(c*arg_up)-0.5*erf(c*arg_low)
 *                                 value2 = -1./(sigma2*sqrt(2*M_PI)) * (exp(-pow(c*arg_up, 2)) - exp(-pow(c*arg_low, 2)))             # <<<<<<<<<<<<<<
 * 
 *                                 # Apply normalization
 */
              __pyx_t_20 = (__pyx_v_sigma2 * sqrt((2.0 * M_PI)));
              if (unlikely(__pyx_t_20 == 0)) {
                PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                __PYX_ERR(0, 686, __pyx_L1_error)
              }
              __pyx_v_value2 = ((-1. / __pyx_t_20) * (exp((-pow((__pyx_v_c * __pyx_v_arg_up), 2.0))) - exp((-pow((__pyx_v_c * __pyx_v_arg_low), 2.0)))));

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":689
 * 
 *                                 # Apply normalization
 *                                 value1 *= normalization             # <<<<<<<<<<<<<<
 *                                 value2 *= normalization
 * 
 */
              __pyx_v_value1 = (__pyx_v_value1 * __pyx_v_normalization);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":690
 *                                 # Apply normalization
 *                                 value1 *= normalization
 *                                 value2 *= normalization             # <<<<<<<<<<<<<<
 * 
 *                                 bin_index = (bondtype_index + newbin) * Natoms*dim
 */
              __pyx_v_value2 = (__pyx_v_value2 * __pyx_v_normalization);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":692
 *                                 value2 *= normalization
 * 
 *                                 bin_index = (bondtype_index + newbin) * Natoms*dim             # <<<<<<<<<<<<<<
 *                                 for m in range(3):
 *                                     feature_grad2[bin_index + dim*i+m] += -value1 * fc_ik*fc_grad_ij * RijVec.coord[m]/Rij
 */
              __pyx_v_bin_index = (((__pyx_v_bondtype_index + __pyx_v_newbin) * __pyx_v_Natoms) * __pyx_v_dim);

              /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":693
 * 
 *                                 bin_index = (bondtype_index + newbin) * Natoms*dim
 *                                 for m in range(3):             # <<<<<<<<<<<<<<
 *                                     feature_grad2[bin_index + dim*i+m] += -value1 * fc_ik*fc_grad_ij * RijVec.coord[m]/Rij
 *                                     feature_grad2[bin_index + dim*j+m] += value1 * fc_ik*fc_grad_ij * RijVec.coord[m]/Rij
 */
              for (__pyx_t_31 = 0; __pyx_t_31 < 3; __pyx_t_31+=1) {
                __pyx_v_m = __pyx_t_31;

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":694
 *                                 bin_index = (bondtype_index + newbin) * Natoms*dim
 *                                 for m in range(3):
 *                                     feature_grad2[bin_index + dim*i+m] += -value1 * fc_ik*fc_grad_ij * RijVec.coord[m]/Rij             # <<<<<<<<<<<<<<
 *                                     feature_grad2[bin_index + dim*j+m] += value1 * fc_ik*fc_grad_ij * RijVec.coord[m]/Rij
 * 
 */
                __pyx_t_32 = ((__pyx_v_bin_index + (__pyx_v_dim * __pyx_v_i)) + __pyx_v_m);
                __pyx_t_20 = ((((-__pyx_v_value1) * __pyx_v_fc_ik) * __pyx_v_fc_grad_ij) * (__pyx_v_RijVec.coord[__pyx_v_m]));
                if (unlikely(__pyx_v_Rij == 0)) {
                  PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                  __PYX_ERR(0, 694, __pyx_L1_error)
                }
                (__pyx_v_feature_grad2[__pyx_t_32]) = ((__pyx_v_feature_grad2[__pyx_t_32]) + (__pyx_t_20 / __pyx_v_Rij));

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":695
 *                                 for m in range(3):
 *                                     feature_grad2[bin_index + dim*i+m] += -value1 * fc_ik*fc_grad_ij * RijVec.coord[m]/Rij
 *                                     feature_grad2[bin_index + dim*j+m] += value1 * fc_ik*fc_grad_ij * RijVec.coord[m]/Rij             # <<<<<<<<<<<<<<
 * 
 *                                     feature_grad2[bin_index + dim*i+m] += -value1 * fc_ij*fc_grad_ik * RikVec.coord[m]/Rik
 */
                __pyx_t_32 = ((__pyx_v_bin_index + (__pyx_v_dim * __pyx_v_j)) + __pyx_v_m);
                __pyx_t_20 = (((__pyx_v_value1 * __pyx_v_fc_ik) * __pyx_v_fc_grad_ij) * (__pyx_v_RijVec.coord[__pyx_v_m]));
                if (unlikely(__pyx_v_Rij == 0)) {
                  PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                  __PYX_ERR(0, 695, __pyx_L1_error)
                }
                (__pyx_v_feature_grad2[__pyx_t_32]) = ((__pyx_v_feature_grad2[__pyx_t_32]) + (__pyx_t_20 / __pyx_v_Rij));

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":697
 *                                     feature_grad2[bin_index + dim*j+m] += value1 * fc_ik*fc_grad_ij * RijVec.coord[m]/Rij
 * 
 *                                     feature_grad2[bin_index + dim*i+m] += -value1 * fc_ij*fc_grad_ik * RikVec.coord[m]/Rik             # <<<<<<<<<<<<<<
 *                                     feature_grad2[bin_index + dim*k+m] += value1 * fc_ij*fc_grad_ik * RikVec.coord[m]/Rik
 * 
 */
                __pyx_t_32 = ((__pyx_v_bin_index + (__pyx_v_dim * __pyx_v_i)) + __pyx_v_m);
                __pyx_t_20 = ((((-__pyx_v_value1) * __pyx_v_fc_ij) * __pyx_v_fc_grad_ik) * (__pyx_v_RikVec.coord[__pyx_v_m]));
                if (unlikely(__pyx_v_Rik == 0)) {
                  PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                  __PYX_ERR(0, 697, __pyx_L1_error)
                }
                (__pyx_v_feature_grad2[__pyx_t_32]) = ((__pyx_v_feature_grad2[__pyx_t_32]) + (__pyx_t_20 / __pyx_v_Rik));

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":698
 * 
 *                                     feature_grad2[bin_index + dim*i+m] += -value1 * fc_ij*fc_grad_ik * RikVec.coord[m]/Rik
 *                                     feature_grad2[bin_index + dim*k+m] += value1 * fc_ij*fc_grad_ik * RikVec.coord[m]/Rik             # <<<<<<<<<<<<<<
 * 
 *                                     feature_grad2[bin_index + dim*i+m] += value2 * fc_ij * fc_ik * angle_grad_i.coord[m]
 */
                __pyx_t_32 = ((__pyx_v_bin_index + (__pyx_v_dim * __pyx_v_k)) + __pyx_v_m);
                __pyx_t_20 = (((__pyx_v_value1 * __pyx_v_fc_ij) * __pyx_v_fc_grad_ik) * (__pyx_v_RikVec.coord[__pyx_v_m]));
                if (unlikely(__pyx_v_Rik == 0)) {
                  PyErr_SetString(PyExc_ZeroDivisionError, "float division");
                  __PYX_ERR(0, 698, __pyx_L1_error)
                }
                (__pyx_v_feature_grad2[__pyx_t_32]) = ((__pyx_v_feature_grad2[__pyx_t_32]) + (__pyx_t_20 / __pyx_v_Rik));

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":700
 *                                     feature_grad2[bin_index + dim*k+m] += value1 * fc_ij*fc_grad_ik * RikVec.coord[m]/Rik
 * 
 *                                     feature_grad2[bin_index + dim*i+m] += value2 * fc_ij * fc_ik * angle_grad_i.coord[m]             # <<<<<<<<<<<<<<
 *                                     feature_grad2[bin_index + dim*j+m] += value2 * fc_ij * fc_ik * angle_grad_j.coord[m]
 *                                     feature_grad2[bin_index + dim*k+m] += value2 * fc_ij * fc_ik * angle_grad_k.coord[m]
 */
                __pyx_t_32 = ((__pyx_v_bin_index + (__pyx_v_dim * __pyx_v_i)) + __pyx_v_m);
                (__pyx_v_feature_grad2[__pyx_t_32]) = ((__pyx_v_feature_grad2[__pyx_t_32]) + (((__pyx_v_value2 * __pyx_v_fc_ij) * __pyx_v_fc_ik) * (__pyx_v_angle_grad_i.coord[__pyx_v_m])));

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":701
 * 
 *                                     feature_grad2[bin_index + dim*i+m] += value2 * fc_ij * fc_ik * angle_grad_i.coord[m]
 *                                     feature_grad2[bin_index + dim*j+m] += value2 * fc_ij * fc_ik * angle_grad_j.coord[m]             # <<<<<<<<<<<<<<
 *                                     feature_grad2[bin_index + dim*k+m] += value2 * fc_ij * fc_ik * angle_grad_k.coord[m]
 * 
 */
                __pyx_t_32 = ((__pyx_v_bin_index + (__pyx_v_dim * __pyx_v_j)) + __pyx_v_m);
                (__pyx_v_feature_grad2[__pyx_t_32]) = ((__pyx_v_feature_grad2[__pyx_t_32]) + (((__pyx_v_value2 * __pyx_v_fc_ij) * __pyx_v_fc_ik) * (__pyx_v_angle_grad_j.coord[__pyx_v_m])));

                /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":702
 *                                     feature_grad2[bin_index + dim*i+m] += value2 * fc_ij * fc_ik * angle_grad_i.coord[m]
 *                                     feature_grad2[bin_index + dim*j+m] += value2 * fc_ij * fc_ik * angle_grad_j.coord[m]
 *                                     feature_grad2[bin_index + dim*k+m] += value2 * fc_ij * fc_ik * angle_grad_k.coord[m]             # <<<<<<<<<<<<<<
 * 
 * 
 */
                __pyx_t_32 = ((__pyx_v_bin_index + (__pyx_v_dim * __pyx_v_k)) + __pyx_v_m);
                (__pyx_v_feature_grad2[__pyx_t_32]) = ((__pyx_v_feature_grad2[__pyx_t_32]) + (((__pyx_v_value2 * __pyx_v_fc_ij) * __pyx_v_fc_ik) * (__pyx_v_angle_grad_k.coord[__pyx_v_m])));
              }
            }
            __pyx_L47_continue:;
          }
        }
        __pyx_L39_continue:;
      }
    }
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":705
 * 
 * 
 *         feature_grad2_np = np.zeros((Natoms*dim, Nelements_3body))             # <<<<<<<<<<<<<<
 *         for m in range(Nelements_3body):
 *             for grad_index in range(Natoms*dim):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_Natoms * __pyx_v_dim)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_Nelements_3body); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_feature_grad2_np = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":706
 * 
 *         feature_grad2_np = np.zeros((Natoms*dim, Nelements_3body))
 *         for m in range(Nelements_3body):             # <<<<<<<<<<<<<<
 *             for grad_index in range(Natoms*dim):
 *                 feature_grad2_np[grad_index][m] = eta * feature_grad2[m * Natoms*dim + grad_index]
 */
  __pyx_t_3 = __pyx_v_Nelements_3body;
  __pyx_t_8 = __pyx_t_3;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_m = __pyx_t_9;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":707
 *         feature_grad2_np = np.zeros((Natoms*dim, Nelements_3body))
 *         for m in range(Nelements_3body):
 *             for grad_index in range(Natoms*dim):             # <<<<<<<<<<<<<<
 *                 feature_grad2_np[grad_index][m] = eta * feature_grad2[m * Natoms*dim + grad_index]
 * 
 */
    __pyx_t_11 = (__pyx_v_Natoms * __pyx_v_dim);
    __pyx_t_13 = __pyx_t_11;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_grad_index = __pyx_t_14;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":708
 *         for m in range(Nelements_3body):
 *             for grad_index in range(Natoms*dim):
 *                 feature_grad2_np[grad_index][m] = eta * feature_grad2[m * Natoms*dim + grad_index]             # <<<<<<<<<<<<<<
 * 
 *         feature_grad_np = np.zeros((Natoms*dim, Nelements))
 */
      __pyx_t_1 = PyFloat_FromDouble((__pyx_v_eta * (__pyx_v_feature_grad2[(((__pyx_v_m * __pyx_v_Natoms) * __pyx_v_dim) + __pyx_v_grad_index)]))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 708, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_12 = __Pyx_GetItemInt(__pyx_v_feature_grad2_np, __pyx_v_grad_index, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 708, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      if (unlikely(__Pyx_SetItemInt(__pyx_t_12, __pyx_v_m, __pyx_t_1, int, 1, __Pyx_PyInt_From_int, 0, 1, 1) < 0)) __PYX_ERR(0, 708, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":710
 *                 feature_grad2_np[grad_index][m] = eta * feature_grad2[m * Natoms*dim + grad_index]
 * 
 *         feature_grad_np = np.zeros((Natoms*dim, Nelements))             # <<<<<<<<<<<<<<
 *         feature_grad_np[:, :Nelements_2body] = feature_grad1_np
 *         feature_grad_np[:, Nelements_2body:] = feature_grad2_np
 */
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyInt_From_int((__pyx_v_Natoms * __pyx_v_dim)); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_Nelements); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
  __pyx_t_12 = 0;
  __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_feature_grad_np = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":711
 * 
 *         feature_grad_np = np.zeros((Natoms*dim, Nelements))
 *         feature_grad_np[:, :Nelements_2body] = feature_grad1_np             # <<<<<<<<<<<<<<
 *         feature_grad_np[:, Nelements_2body:] = feature_grad2_np
 *         return feature_grad_np
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_Nelements_2body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PySlice_New(Py_None, __pyx_t_1, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_slice_);
  __Pyx_GIVEREF(__pyx_slice_);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_slice_);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __pyx_t_5 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_feature_grad_np, __pyx_t_1, __pyx_v_feature_grad1_np) < 0)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":712
 *         feature_grad_np = np.zeros((Natoms*dim, Nelements))
 *         feature_grad_np[:, :Nelements_2body] = feature_grad1_np
 *         feature_grad_np[:, Nelements_2body:] = feature_grad2_np             # <<<<<<<<<<<<<<
 *         return feature_grad_np
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_Nelements_2body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PySlice_New(__pyx_t_1, Py_None, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 712, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_slice_);
  __Pyx_GIVEREF(__pyx_slice_);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_slice_);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __pyx_t_5 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_feature_grad_np, __pyx_t_1, __pyx_v_feature_grad2_np) < 0)) __PYX_ERR(0, 712, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":713
 *         feature_grad_np[:, :Nelements_2body] = feature_grad1_np
 *         feature_grad_np[:, Nelements_2body:] = feature_grad2_np
 *         return feature_grad_np             # <<<<<<<<<<<<<<
 * 
 *     def get_all_featureGradients(self, atoms_list):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_feature_grad_np);
  __pyx_r = __pyx_v_feature_grad_np;
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":438
 *         return featureMat
 * 
 *     def get_featureGradient(self, atoms):             # <<<<<<<<<<<<<<
 * 
 *         cdef double Rc1 = self.Rc1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.get_featureGradient", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_use_angular);
  __Pyx_XDECREF((PyObject *)__pyx_v_mem);
  __Pyx_XDECREF(__pyx_v_cell);
  __Pyx_XDECREF(__pyx_v_pos_np);
  __Pyx_XDECREF(__pyx_v_cell_displacements_old);
  __Pyx_XDECREF(__pyx_v_bondtypes_2body_old);
  __Pyx_XDECREF(__pyx_v_num_converted_old);
  __Pyx_XDECREF(__pyx_v_feature_grad1_np);
  __Pyx_XDECREF(__pyx_v_bondtypes_3body_old);
  __Pyx_XDECREF(__pyx_v_feature_grad2_np);
  __Pyx_XDECREF(__pyx_v_feature_grad_np);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":715
 *         return feature_grad_np
 * 
 *     def get_all_featureGradients(self, atoms_list):             # <<<<<<<<<<<<<<
 *         feature_grads = np.array([self.get_featureGradient(atoms) for atoms in atoms_list])
 *         feature_grads = np.array(feature_grads)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_9get_all_featureGradients(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_9get_all_featureGradients = {"get_all_featureGradients", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_9get_all_featureGradients, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_9get_all_featureGradients(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_atoms_list = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_all_featureGradients (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_atoms_list,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_atoms_list)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_all_featureGradients", 1, 2, 2, 1); __PYX_ERR(0, 715, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_all_featureGradients") < 0)) __PYX_ERR(0, 715, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_atoms_list = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_all_featureGradients", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 715, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.get_all_featureGradients", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_8get_all_featureGradients(__pyx_self, __pyx_v_self, __pyx_v_atoms_list);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_8get_all_featureGradients(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_atoms_list) {
  PyObject *__pyx_v_feature_grads = NULL;
  PyObject *__pyx_v_atoms = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_all_featureGradients", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":716
 * 
 *     def get_all_featureGradients(self, atoms_list):
 *         feature_grads = np.array([self.get_featureGradient(atoms) for atoms in atoms_list])             # <<<<<<<<<<<<<<
 *         feature_grads = np.array(feature_grads)
 *         return feature_grads
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_array); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (likely(PyList_CheckExact(__pyx_v_atoms_list)) || PyTuple_CheckExact(__pyx_v_atoms_list)) {
    __pyx_t_4 = __pyx_v_atoms_list; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_atoms_list); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 716, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 716, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 716, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 716, __pyx_L1_error)
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 716, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      }
    } else {
      __pyx_t_7 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_7)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 716, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_XDECREF_SET(__pyx_v_atoms, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_get_featureGradient); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_7 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_v_atoms) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_v_atoms);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_feature_grads = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":717
 *     def get_all_featureGradients(self, atoms_list):
 *         feature_grads = np.array([self.get_featureGradient(atoms) for atoms in atoms_list])
 *         feature_grads = np.array(feature_grads)             # <<<<<<<<<<<<<<
 *         return feature_grads
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_array); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_feature_grads) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_feature_grads);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_feature_grads, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":718
 *         feature_grads = np.array([self.get_featureGradient(atoms) for atoms in atoms_list])
 *         feature_grads = np.array(feature_grads)
 *         return feature_grads             # <<<<<<<<<<<<<<
 * 
 *     def __get_neighbour_cells_displacement(self, pbc, cell):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_feature_grads);
  __pyx_r = __pyx_v_feature_grads;
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":715
 *         return feature_grad_np
 * 
 *     def get_all_featureGradients(self, atoms_list):             # <<<<<<<<<<<<<<
 *         feature_grads = np.array([self.get_featureGradient(atoms) for atoms in atoms_list])
 *         feature_grads = np.array(feature_grads)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.get_all_featureGradients", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_feature_grads);
  __Pyx_XDECREF(__pyx_v_atoms);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":720
 *         return feature_grads
 * 
 *     def __get_neighbour_cells_displacement(self, pbc, cell):             # <<<<<<<<<<<<<<
 *         # Calculate neighbour cells
 *         Rc_max = max(self.Rc1+self.sigma1*self.nsigma, self.Rc2)  # relevant cutoff
 */

/* Python wrapper */
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_11__get_neighbour_cells_displacement(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_11__get_neighbour_cells_displacement = {"__get_neighbour_cells_displacement", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_11__get_neighbour_cells_displacement, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_11__get_neighbour_cells_displacement(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_pbc = 0;
  PyObject *__pyx_v_cell = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get_neighbour_cells_displacement (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_pbc,&__pyx_n_s_cell,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pbc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__get_neighbour_cells_displacement", 1, 3, 3, 1); __PYX_ERR(0, 720, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_cell)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__get_neighbour_cells_displacement", 1, 3, 3, 2); __PYX_ERR(0, 720, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__get_neighbour_cells_displacement") < 0)) __PYX_ERR(0, 720, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_self = values[0];
    __pyx_v_pbc = values[1];
    __pyx_v_cell = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__get_neighbour_cells_displacement", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 720, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.__get_neighbour_cells_displacement", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_10__get_neighbour_cells_displacement(__pyx_self, __pyx_v_self, __pyx_v_pbc, __pyx_v_cell);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_10__get_neighbour_cells_displacement(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_pbc, PyObject *__pyx_v_cell) {
  PyObject *__pyx_v_Rc_max = NULL;
  PyObject *__pyx_v_cell_vec_norms = NULL;
  PyObject *__pyx_v_neighbours = NULL;
  long __pyx_v_i;
  PyObject *__pyx_v_ncellmax = NULL;
  PyObject *__pyx_v_neighbourcells_disp = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_y = NULL;
  PyObject *__pyx_v_z = NULL;
  PyObject *__pyx_v_xyz = NULL;
  PyObject *__pyx_v_displacement = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  long __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11 = NULL;
  PyObject *(*__pyx_t_12)(PyObject *);
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get_neighbour_cells_displacement", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":722
 *     def __get_neighbour_cells_displacement(self, pbc, cell):
 *         # Calculate neighbour cells
 *         Rc_max = max(self.Rc1+self.sigma1*self.nsigma, self.Rc2)  # relevant cutoff             # <<<<<<<<<<<<<<
 *         cell_vec_norms = np.linalg.norm(cell, axis=0)
 *         neighbours = []
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Rc2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_Rc1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sigma1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_nsigma); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyNumber_Multiply(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyNumber_Add(__pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_t_4, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 722, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_5 = __pyx_t_1;
  } else {
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = __pyx_t_4;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_5;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_Rc_max = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":723
 *         # Calculate neighbour cells
 *         Rc_max = max(self.Rc1+self.sigma1*self.nsigma, self.Rc2)  # relevant cutoff
 *         cell_vec_norms = np.linalg.norm(cell, axis=0)             # <<<<<<<<<<<<<<
 *         neighbours = []
 *         for i in range(3):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_linalg); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_norm); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_cell);
  __Pyx_GIVEREF(__pyx_v_cell);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_cell);
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_axis, __pyx_int_0) < 0) __PYX_ERR(0, 723, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_cell_vec_norms = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":724
 *         Rc_max = max(self.Rc1+self.sigma1*self.nsigma, self.Rc2)  # relevant cutoff
 *         cell_vec_norms = np.linalg.norm(cell, axis=0)
 *         neighbours = []             # <<<<<<<<<<<<<<
 *         for i in range(3):
 *             if pbc[i]:
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 724, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_neighbours = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":725
 *         cell_vec_norms = np.linalg.norm(cell, axis=0)
 *         neighbours = []
 *         for i in range(3):             # <<<<<<<<<<<<<<
 *             if pbc[i]:
 *                 ncellmax = int(np.ceil(abs(Rc_max/cell_vec_norms[i]))) + 1  # +1 because atoms can be outside unitcell.
 */
  for (__pyx_t_7 = 0; __pyx_t_7 < 3; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":726
 *         neighbours = []
 *         for i in range(3):
 *             if pbc[i]:             # <<<<<<<<<<<<<<
 *                 ncellmax = int(np.ceil(abs(Rc_max/cell_vec_norms[i]))) + 1  # +1 because atoms can be outside unitcell.
 *                 neighbours.append(range(-ncellmax,ncellmax+1))
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_pbc, __pyx_v_i, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 726, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 726, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_6) {

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":727
 *         for i in range(3):
 *             if pbc[i]:
 *                 ncellmax = int(np.ceil(abs(Rc_max/cell_vec_norms[i]))) + 1  # +1 because atoms can be outside unitcell.             # <<<<<<<<<<<<<<
 *                 neighbours.append(range(-ncellmax,ncellmax+1))
 *             else:
 */
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_ceil); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_cell_vec_norms, __pyx_v_i, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_v_Rc_max, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyNumber_Absolute(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_1, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyNumber_Int(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_t_5, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF_SET(__pyx_v_ncellmax, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":728
 *             if pbc[i]:
 *                 ncellmax = int(np.ceil(abs(Rc_max/cell_vec_norms[i]))) + 1  # +1 because atoms can be outside unitcell.
 *                 neighbours.append(range(-ncellmax,ncellmax+1))             # <<<<<<<<<<<<<<
 *             else:
 *                 neighbours.append([0])
 */
      __pyx_t_2 = PyNumber_Negative(__pyx_v_ncellmax); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = __Pyx_PyInt_AddObjC(__pyx_v_ncellmax, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
      __pyx_t_2 = 0;
      __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_neighbours, __pyx_t_5); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 728, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":726
 *         neighbours = []
 *         for i in range(3):
 *             if pbc[i]:             # <<<<<<<<<<<<<<
 *                 ncellmax = int(np.ceil(abs(Rc_max/cell_vec_norms[i]))) + 1  # +1 because atoms can be outside unitcell.
 *                 neighbours.append(range(-ncellmax,ncellmax+1))
 */
      goto __pyx_L5;
    }

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":730
 *                 neighbours.append(range(-ncellmax,ncellmax+1))
 *             else:
 *                 neighbours.append([0])             # <<<<<<<<<<<<<<
 * 
 *         neighbourcells_disp = []
 */
    /*else*/ {
      __pyx_t_5 = PyList_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 730, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_int_0);
      __Pyx_GIVEREF(__pyx_int_0);
      PyList_SET_ITEM(__pyx_t_5, 0, __pyx_int_0);
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_neighbours, __pyx_t_5); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 730, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_L5:;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":732
 *                 neighbours.append([0])
 * 
 *         neighbourcells_disp = []             # <<<<<<<<<<<<<<
 *         for x,y,z in product(*neighbours):
 *             xyz = (x,y,z)
 */
  __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v_neighbourcells_disp = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":733
 * 
 *         neighbourcells_disp = []
 *         for x,y,z in product(*neighbours):             # <<<<<<<<<<<<<<
 *             xyz = (x,y,z)
 *             displacement = np.dot(xyz, cell)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_product); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = PySequence_Tuple(__pyx_v_neighbours); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_4 = __pyx_t_2; __Pyx_INCREF(__pyx_t_4); __pyx_t_9 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_9 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 733, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 733, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 733, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 733, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 733, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 733, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_10(__pyx_t_4);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 733, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 733, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_1 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_1 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 733, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 733, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 733, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_11 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 733, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_12 = Py_TYPE(__pyx_t_11)->tp_iternext;
      index = 0; __pyx_t_5 = __pyx_t_12(__pyx_t_11); if (unlikely(!__pyx_t_5)) goto __pyx_L8_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_1 = __pyx_t_12(__pyx_t_11); if (unlikely(!__pyx_t_1)) goto __pyx_L8_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_1);
      index = 2; __pyx_t_3 = __pyx_t_12(__pyx_t_11); if (unlikely(!__pyx_t_3)) goto __pyx_L8_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_12(__pyx_t_11), 3) < 0) __PYX_ERR(0, 733, __pyx_L1_error)
      __pyx_t_12 = NULL;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L9_unpacking_done;
      __pyx_L8_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_12 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 733, __pyx_L1_error)
      __pyx_L9_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_x, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_y, __pyx_t_1);
    __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_z, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":734
 *         neighbourcells_disp = []
 *         for x,y,z in product(*neighbours):
 *             xyz = (x,y,z)             # <<<<<<<<<<<<<<
 *             displacement = np.dot(xyz, cell)
 *             neighbourcells_disp.append(list(displacement))
 */
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 734, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_x);
    __Pyx_INCREF(__pyx_v_y);
    __Pyx_GIVEREF(__pyx_v_y);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_y);
    __Pyx_INCREF(__pyx_v_z);
    __Pyx_GIVEREF(__pyx_v_z);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_z);
    __Pyx_XDECREF_SET(__pyx_v_xyz, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":735
 *         for x,y,z in product(*neighbours):
 *             xyz = (x,y,z)
 *             displacement = np.dot(xyz, cell)             # <<<<<<<<<<<<<<
 *             neighbourcells_disp.append(list(displacement))
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 735, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dot); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 735, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    __pyx_t_13 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_13 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_xyz, __pyx_v_cell};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 735, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_xyz, __pyx_v_cell};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 735, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 735, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_3) {
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
      }
      __Pyx_INCREF(__pyx_v_xyz);
      __Pyx_GIVEREF(__pyx_v_xyz);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_13, __pyx_v_xyz);
      __Pyx_INCREF(__pyx_v_cell);
      __Pyx_GIVEREF(__pyx_v_cell);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_13, __pyx_v_cell);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 735, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_displacement, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":736
 *             xyz = (x,y,z)
 *             displacement = np.dot(xyz, cell)
 *             neighbourcells_disp.append(list(displacement))             # <<<<<<<<<<<<<<
 * 
 *         return neighbourcells_disp
 */
    __pyx_t_2 = PySequence_List(__pyx_v_displacement); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 736, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_neighbourcells_disp, __pyx_t_2); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 736, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":733
 * 
 *         neighbourcells_disp = []
 *         for x,y,z in product(*neighbours):             # <<<<<<<<<<<<<<
 *             xyz = (x,y,z)
 *             displacement = np.dot(xyz, cell)
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":738
 *             neighbourcells_disp.append(list(displacement))
 * 
 *         return neighbourcells_disp             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_neighbourcells_disp);
  __pyx_r = __pyx_v_neighbourcells_disp;
  goto __pyx_L0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":720
 *         return feature_grads
 * 
 *     def __get_neighbour_cells_displacement(self, pbc, cell):             # <<<<<<<<<<<<<<
 *         # Calculate neighbour cells
 *         Rc_max = max(self.Rc1+self.sigma1*self.nsigma, self.Rc2)  # relevant cutoff
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy.Angular_Fingerprint.__get_neighbour_cells_displacement", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_Rc_max);
  __Pyx_XDECREF(__pyx_v_cell_vec_norms);
  __Pyx_XDECREF(__pyx_v_neighbours);
  __Pyx_XDECREF(__pyx_v_ncellmax);
  __Pyx_XDECREF(__pyx_v_neighbourcells_disp);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_y);
  __Pyx_XDECREF(__pyx_v_z);
  __Pyx_XDECREF(__pyx_v_xyz);
  __Pyx_XDECREF(__pyx_v_displacement);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":691
 * ctypedef long double complex clongdouble_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":692
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":691
 * ctypedef long double complex clongdouble_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":694
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":695
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":694
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":697
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":698
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":697
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":700
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":701
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":700
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":703
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":704
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 704, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":703
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":706
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":707
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":708
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_d), __pyx_n_s_subarray); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(((PyObject*)__pyx_t_3));
    __pyx_r = ((PyObject*)__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":707
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":710
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":706
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("numpy.PyDataType_SHAPE", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":907
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":908
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":909
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_t_1 = PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(1, 909, __pyx_L1_error)

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":907
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("numpy.set_array_base", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":911
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":912
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":913
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":914
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":913
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":915
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":911
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":919
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":920
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":921
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy._core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 921, __pyx_L3_error)

      /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":920
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":922
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy._core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 922, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":923
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy._core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 923, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 923, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":920
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":919
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":925
 *         raise ImportError("numpy._core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":926
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":927
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 927, __pyx_L3_error)

      /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":926
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":928
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy._core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 928, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":929
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 929, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 929, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":926
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":925
 *         raise ImportError("numpy._core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":931
 *         raise ImportError("numpy._core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":932
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":933
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 933, __pyx_L3_error)

      /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":932
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":934
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy._core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 934, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":935
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 935, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 935, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":932
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":931
 *         raise ImportError("numpy._core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":938
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_timedelta64_object", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":950
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":938
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":953
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_datetime64_object", 0);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":965
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":953
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":968
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":975
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return (<PyDatetimeScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyDatetimeScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":968
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":978
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":982
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return (<PyTimedeltaScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyTimedeltaScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":978
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":985
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":989
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return <NPY_DATETIMEUNIT>(<PyDatetimeScalarObject*>obj).obmeta.base             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((NPY_DATETIMEUNIT)((PyDatetimeScalarObject *)__pyx_v_obj)->obmeta.base);
  goto __pyx_L0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":985
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_angular_fingerprintFeature_cy(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_angular_fingerprintFeature_cy},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "angular_fingerprintFeature_cy",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_Angular_Fingerprint, __pyx_k_Angular_Fingerprint, sizeof(__pyx_k_Angular_Fingerprint), 0, 0, 1, 1},
  {&__pyx_n_s_Angular_Fingerprint___get_neighb, __pyx_k_Angular_Fingerprint___get_neighb, sizeof(__pyx_k_Angular_Fingerprint___get_neighb), 0, 0, 1, 1},
  {&__pyx_n_s_Angular_Fingerprint___init, __pyx_k_Angular_Fingerprint___init, sizeof(__pyx_k_Angular_Fingerprint___init), 0, 0, 1, 1},
  {&__pyx_n_s_Angular_Fingerprint__get_neighb, __pyx_k_Angular_Fingerprint__get_neighb, sizeof(__pyx_k_Angular_Fingerprint__get_neighb), 0, 0, 1, 1},
  {&__pyx_n_s_Angular_Fingerprint_get_all_feat, __pyx_k_Angular_Fingerprint_get_all_feat, sizeof(__pyx_k_Angular_Fingerprint_get_all_feat), 0, 0, 1, 1},
  {&__pyx_n_s_Angular_Fingerprint_get_feature, __pyx_k_Angular_Fingerprint_get_feature, sizeof(__pyx_k_Angular_Fingerprint_get_feature), 0, 0, 1, 1},
  {&__pyx_n_s_Angular_Fingerprint_get_featureG, __pyx_k_Angular_Fingerprint_get_featureG, sizeof(__pyx_k_Angular_Fingerprint_get_featureG), 0, 0, 1, 1},
  {&__pyx_n_s_Angular_Fingerprint_get_featureM, __pyx_k_Angular_Fingerprint_get_featureM, sizeof(__pyx_k_Angular_Fingerprint_get_featureM), 0, 0, 1, 1},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_n_s_Natoms, __pyx_k_Natoms, sizeof(__pyx_k_Natoms), 0, 0, 1, 1},
  {&__pyx_n_s_Nbins1, __pyx_k_Nbins1, sizeof(__pyx_k_Nbins1), 0, 0, 1, 1},
  {&__pyx_n_s_Nbins2, __pyx_k_Nbins2, sizeof(__pyx_k_Nbins2), 0, 0, 1, 1},
  {&__pyx_n_s_Nbondtypes_2body, __pyx_k_Nbondtypes_2body, sizeof(__pyx_k_Nbondtypes_2body), 0, 0, 1, 1},
  {&__pyx_n_s_Nbondtypes_3body, __pyx_k_Nbondtypes_3body, sizeof(__pyx_k_Nbondtypes_3body), 0, 0, 1, 1},
  {&__pyx_n_s_Ncells, __pyx_k_Ncells, sizeof(__pyx_k_Ncells), 0, 0, 1, 1},
  {&__pyx_n_s_Nelements, __pyx_k_Nelements, sizeof(__pyx_k_Nelements), 0, 0, 1, 1},
  {&__pyx_n_s_Nelements_2body, __pyx_k_Nelements_2body, sizeof(__pyx_k_Nelements_2body), 0, 0, 1, 1},
  {&__pyx_n_s_Nelements_3body, __pyx_k_Nelements_3body, sizeof(__pyx_k_Nelements_3body), 0, 0, 1, 1},
  {&__pyx_n_s_Ntypes, __pyx_k_Ntypes, sizeof(__pyx_k_Ntypes), 0, 0, 1, 1},
  {&__pyx_n_s_Rc1, __pyx_k_Rc1, sizeof(__pyx_k_Rc1), 0, 0, 1, 1},
  {&__pyx_n_s_Rc2, __pyx_k_Rc2, sizeof(__pyx_k_Rc2), 0, 0, 1, 1},
  {&__pyx_n_s_Rc_max, __pyx_k_Rc_max, sizeof(__pyx_k_Rc_max), 0, 0, 1, 1},
  {&__pyx_n_s_Rij, __pyx_k_Rij, sizeof(__pyx_k_Rij), 0, 0, 1, 1},
  {&__pyx_n_s_RijVec, __pyx_k_RijVec, sizeof(__pyx_k_RijVec), 0, 0, 1, 1},
  {&__pyx_n_s_Rik, __pyx_k_Rik, sizeof(__pyx_k_Rik), 0, 0, 1, 1},
  {&__pyx_n_s_RikVec, __pyx_k_RikVec, sizeof(__pyx_k_RikVec), 0, 0, 1, 1},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_kp_s_agox_models_descriptors_fingerpr, __pyx_k_agox_models_descriptors_fingerpr, sizeof(__pyx_k_agox_models_descriptors_fingerpr), 0, 0, 1, 0},
  {&__pyx_n_s_agox_models_descriptors_fingerpr_2, __pyx_k_agox_models_descriptors_fingerpr_2, sizeof(__pyx_k_agox_models_descriptors_fingerpr_2), 0, 0, 1, 1},
  {&__pyx_n_s_angle, __pyx_k_angle, sizeof(__pyx_k_angle), 0, 0, 1, 1},
  {&__pyx_n_s_angle_grad_i, __pyx_k_angle_grad_i, sizeof(__pyx_k_angle_grad_i), 0, 0, 1, 1},
  {&__pyx_n_s_angle_grad_j, __pyx_k_angle_grad_j, sizeof(__pyx_k_angle_grad_j), 0, 0, 1, 1},
  {&__pyx_n_s_angle_grad_k, __pyx_k_angle_grad_k, sizeof(__pyx_k_angle_grad_k), 0, 0, 1, 1},
  {&__pyx_n_s_arg_low, __pyx_k_arg_low, sizeof(__pyx_k_arg_low), 0, 0, 1, 1},
  {&__pyx_n_s_arg_up, __pyx_k_arg_up, sizeof(__pyx_k_arg_up), 0, 0, 1, 1},
  {&__pyx_n_s_argv, __pyx_k_argv, sizeof(__pyx_k_argv), 0, 0, 1, 1},
  {&__pyx_n_s_array, __pyx_k_array, sizeof(__pyx_k_array), 0, 0, 1, 1},
  {&__pyx_n_s_astype, __pyx_k_astype, sizeof(__pyx_k_astype), 0, 0, 1, 1},
  {&__pyx_n_s_atomic_count, __pyx_k_atomic_count, sizeof(__pyx_k_atomic_count), 0, 0, 1, 1},
  {&__pyx_n_s_atomic_types, __pyx_k_atomic_types, sizeof(__pyx_k_atomic_types), 0, 0, 1, 1},
  {&__pyx_n_s_atoms, __pyx_k_atoms, sizeof(__pyx_k_atoms), 0, 0, 1, 1},
  {&__pyx_n_s_atoms_list, __pyx_k_atoms_list, sizeof(__pyx_k_atoms_list), 0, 0, 1, 1},
  {&__pyx_n_s_axis, __pyx_k_axis, sizeof(__pyx_k_axis), 0, 0, 1, 1},
  {&__pyx_n_s_bin_index, __pyx_k_bin_index, sizeof(__pyx_k_bin_index), 0, 0, 1, 1},
  {&__pyx_n_s_binpos, __pyx_k_binpos, sizeof(__pyx_k_binpos), 0, 0, 1, 1},
  {&__pyx_n_s_binwidth1, __pyx_k_binwidth1, sizeof(__pyx_k_binwidth1), 0, 0, 1, 1},
  {&__pyx_n_s_binwidth2, __pyx_k_binwidth2, sizeof(__pyx_k_binwidth2), 0, 0, 1, 1},
  {&__pyx_n_s_bondtype_index, __pyx_k_bondtype_index, sizeof(__pyx_k_bondtype_index), 0, 0, 1, 1},
  {&__pyx_n_s_bondtypes_2body, __pyx_k_bondtypes_2body, sizeof(__pyx_k_bondtypes_2body), 0, 0, 1, 1},
  {&__pyx_n_s_bondtypes_2body_old, __pyx_k_bondtypes_2body_old, sizeof(__pyx_k_bondtypes_2body_old), 0, 0, 1, 1},
  {&__pyx_n_s_bondtypes_3body, __pyx_k_bondtypes_3body, sizeof(__pyx_k_bondtypes_3body), 0, 0, 1, 1},
  {&__pyx_n_s_bondtypes_3body_keys, __pyx_k_bondtypes_3body_keys, sizeof(__pyx_k_bondtypes_3body_keys), 0, 0, 1, 1},
  {&__pyx_n_s_bondtypes_3body_old, __pyx_k_bondtypes_3body_old, sizeof(__pyx_k_bondtypes_3body_old), 0, 0, 1, 1},
  {&__pyx_n_s_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 0, 1, 1},
  {&__pyx_n_s_cdist, __pyx_k_cdist, sizeof(__pyx_k_cdist), 0, 0, 1, 1},
  {&__pyx_n_s_ceil, __pyx_k_ceil, sizeof(__pyx_k_ceil), 0, 0, 1, 1},
  {&__pyx_n_s_cell, __pyx_k_cell, sizeof(__pyx_k_cell), 0, 0, 1, 1},
  {&__pyx_n_s_cell_displacements, __pyx_k_cell_displacements, sizeof(__pyx_k_cell_displacements), 0, 0, 1, 1},
  {&__pyx_n_s_cell_displacements_old, __pyx_k_cell_displacements_old, sizeof(__pyx_k_cell_displacements_old), 0, 0, 1, 1},
  {&__pyx_n_s_cell_index, __pyx_k_cell_index, sizeof(__pyx_k_cell_index), 0, 0, 1, 1},
  {&__pyx_n_s_cell_index1, __pyx_k_cell_index1, sizeof(__pyx_k_cell_index1), 0, 0, 1, 1},
  {&__pyx_n_s_cell_index2, __pyx_k_cell_index2, sizeof(__pyx_k_cell_index2), 0, 0, 1, 1},
  {&__pyx_n_s_cell_vec_norms, __pyx_k_cell_vec_norms, sizeof(__pyx_k_cell_vec_norms), 0, 0, 1, 1},
  {&__pyx_n_s_center_bin, __pyx_k_center_bin, sizeof(__pyx_k_center_bin), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_kp_s_comparator_for_construction_of, __pyx_k_comparator_for_construction_of, sizeof(__pyx_k_comparator_for_construction_of), 0, 0, 1, 0},
  {&__pyx_n_s_cond_ij, __pyx_k_cond_ij, sizeof(__pyx_k_cond_ij), 0, 0, 1, 1},
  {&__pyx_n_s_cond_ik, __pyx_k_cond_ik, sizeof(__pyx_k_cond_ik), 0, 0, 1, 1},
  {&__pyx_n_s_convert_atom_types, __pyx_k_convert_atom_types, sizeof(__pyx_k_convert_atom_types), 0, 0, 1, 1},
  {&__pyx_n_s_cos_angle, __pyx_k_cos_angle, sizeof(__pyx_k_cos_angle), 0, 0, 1, 1},
  {&__pyx_n_s_count, __pyx_k_count, sizeof(__pyx_k_count), 0, 0, 1, 1},
  {&__pyx_n_s_cwd, __pyx_k_cwd, sizeof(__pyx_k_cwd), 0, 0, 1, 1},
  {&__pyx_n_s_dim, __pyx_k_dim, sizeof(__pyx_k_dim), 0, 0, 1, 1},
  {&__pyx_n_s_displacement, __pyx_k_displacement, sizeof(__pyx_k_displacement), 0, 0, 1, 1},
  {&__pyx_n_s_displacement1, __pyx_k_displacement1, sizeof(__pyx_k_displacement1), 0, 0, 1, 1},
  {&__pyx_n_s_displacement2, __pyx_k_displacement2, sizeof(__pyx_k_displacement2), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_dot, __pyx_k_dot, sizeof(__pyx_k_dot), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_erf, __pyx_k_erf, sizeof(__pyx_k_erf), 0, 0, 1, 1},
  {&__pyx_n_s_erfarg_low, __pyx_k_erfarg_low, sizeof(__pyx_k_erfarg_low), 0, 0, 1, 1},
  {&__pyx_n_s_erfarg_up, __pyx_k_erfarg_up, sizeof(__pyx_k_erfarg_up), 0, 0, 1, 1},
  {&__pyx_n_s_eta, __pyx_k_eta, sizeof(__pyx_k_eta), 0, 0, 1, 1},
  {&__pyx_n_s_fc_grad_ij, __pyx_k_fc_grad_ij, sizeof(__pyx_k_fc_grad_ij), 0, 0, 1, 1},
  {&__pyx_n_s_fc_grad_ik, __pyx_k_fc_grad_ik, sizeof(__pyx_k_fc_grad_ik), 0, 0, 1, 1},
  {&__pyx_n_s_fc_ij, __pyx_k_fc_ij, sizeof(__pyx_k_fc_ij), 0, 0, 1, 1},
  {&__pyx_n_s_fc_ik, __pyx_k_fc_ik, sizeof(__pyx_k_fc_ik), 0, 0, 1, 1},
  {&__pyx_n_s_feature1, __pyx_k_feature1, sizeof(__pyx_k_feature1), 0, 0, 1, 1},
  {&__pyx_n_s_feature1_np, __pyx_k_feature1_np, sizeof(__pyx_k_feature1_np), 0, 0, 1, 1},
  {&__pyx_n_s_feature2, __pyx_k_feature2, sizeof(__pyx_k_feature2), 0, 0, 1, 1},
  {&__pyx_n_s_feature2_np, __pyx_k_feature2_np, sizeof(__pyx_k_feature2_np), 0, 0, 1, 1},
  {&__pyx_n_s_featureMat, __pyx_k_featureMat, sizeof(__pyx_k_featureMat), 0, 0, 1, 1},
  {&__pyx_n_s_feature_grad1, __pyx_k_feature_grad1, sizeof(__pyx_k_feature_grad1), 0, 0, 1, 1},
  {&__pyx_n_s_feature_grad1_np, __pyx_k_feature_grad1_np, sizeof(__pyx_k_feature_grad1_np), 0, 0, 1, 1},
  {&__pyx_n_s_feature_grad2, __pyx_k_feature_grad2, sizeof(__pyx_k_feature_grad2), 0, 0, 1, 1},
  {&__pyx_n_s_feature_grad2_np, __pyx_k_feature_grad2_np, sizeof(__pyx_k_feature_grad2_np), 0, 0, 1, 1},
  {&__pyx_n_s_feature_grad_np, __pyx_k_feature_grad_np, sizeof(__pyx_k_feature_grad_np), 0, 0, 1, 1},
  {&__pyx_n_s_feature_grads, __pyx_k_feature_grads, sizeof(__pyx_k_feature_grads), 0, 0, 1, 1},
  {&__pyx_n_s_feature_np, __pyx_k_feature_np, sizeof(__pyx_k_feature_np), 0, 0, 1, 1},
  {&__pyx_n_s_gamma, __pyx_k_gamma, sizeof(__pyx_k_gamma), 0, 0, 1, 1},
  {&__pyx_n_s_get_all_featureGradients, __pyx_k_get_all_featureGradients, sizeof(__pyx_k_get_all_featureGradients), 0, 0, 1, 1},
  {&__pyx_n_s_get_atomic_numbers, __pyx_k_get_atomic_numbers, sizeof(__pyx_k_get_atomic_numbers), 0, 0, 1, 1},
  {&__pyx_n_s_get_cell, __pyx_k_get_cell, sizeof(__pyx_k_get_cell), 0, 0, 1, 1},
  {&__pyx_n_s_get_feature, __pyx_k_get_feature, sizeof(__pyx_k_get_feature), 0, 0, 1, 1},
  {&__pyx_n_s_get_featureGradient, __pyx_k_get_featureGradient, sizeof(__pyx_k_get_featureGradient), 0, 0, 1, 1},
  {&__pyx_n_s_get_featureMat, __pyx_k_get_featureMat, sizeof(__pyx_k_get_featureMat), 0, 0, 1, 1},
  {&__pyx_n_s_get_neighbour_cells_displaceme, __pyx_k_get_neighbour_cells_displaceme, sizeof(__pyx_k_get_neighbour_cells_displaceme), 0, 0, 1, 1},
  {&__pyx_n_s_get_pbc, __pyx_k_get_pbc, sizeof(__pyx_k_get_pbc), 0, 0, 1, 1},
  {&__pyx_n_s_get_positions, __pyx_k_get_positions, sizeof(__pyx_k_get_positions), 0, 0, 1, 1},
  {&__pyx_n_s_get_volume, __pyx_k_get_volume, sizeof(__pyx_k_get_volume), 0, 0, 1, 1},
  {&__pyx_n_s_getcwd, __pyx_k_getcwd, sizeof(__pyx_k_getcwd), 0, 0, 1, 1},
  {&__pyx_n_s_grad_index, __pyx_k_grad_index, sizeof(__pyx_k_grad_index), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_itertools, __pyx_k_itertools, sizeof(__pyx_k_itertools), 0, 0, 1, 1},
  {&__pyx_n_s_j, __pyx_k_j, sizeof(__pyx_k_j), 0, 0, 1, 1},
  {&__pyx_n_s_k, __pyx_k_k, sizeof(__pyx_k_k), 0, 0, 1, 1},
  {&__pyx_n_s_k1, __pyx_k_k1, sizeof(__pyx_k_k1), 0, 0, 1, 1},
  {&__pyx_n_s_k2, __pyx_k_k2, sizeof(__pyx_k_k2), 0, 0, 1, 1},
  {&__pyx_n_s_k3, __pyx_k_k3, sizeof(__pyx_k_k3), 0, 0, 1, 1},
  {&__pyx_n_s_k_start, __pyx_k_k_start, sizeof(__pyx_k_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_linalg, __pyx_k_linalg, sizeof(__pyx_k_linalg), 0, 0, 1, 1},
  {&__pyx_n_s_m, __pyx_k_m, sizeof(__pyx_k_m), 0, 0, 1, 1},
  {&__pyx_n_s_m1, __pyx_k_m1, sizeof(__pyx_k_m1), 0, 0, 1, 1},
  {&__pyx_n_s_m2, __pyx_k_m2, sizeof(__pyx_k_m2), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_math, __pyx_k_math, sizeof(__pyx_k_math), 0, 0, 1, 1},
  {&__pyx_n_s_maxbin_lim, __pyx_k_maxbin_lim, sizeof(__pyx_k_maxbin_lim), 0, 0, 1, 1},
  {&__pyx_n_s_mem, __pyx_k_mem, sizeof(__pyx_k_mem), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_minbin_lim, __pyx_k_minbin_lim, sizeof(__pyx_k_minbin_lim), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_ncellmax, __pyx_k_ncellmax, sizeof(__pyx_k_ncellmax), 0, 0, 1, 1},
  {&__pyx_n_s_neighbourcells_disp, __pyx_k_neighbourcells_disp, sizeof(__pyx_k_neighbourcells_disp), 0, 0, 1, 1},
  {&__pyx_n_s_neighbours, __pyx_k_neighbours, sizeof(__pyx_k_neighbours), 0, 0, 1, 1},
  {&__pyx_n_s_newbin, __pyx_k_newbin, sizeof(__pyx_k_newbin), 0, 0, 1, 1},
  {&__pyx_n_s_norm, __pyx_k_norm, sizeof(__pyx_k_norm), 0, 0, 1, 1},
  {&__pyx_n_s_normalization, __pyx_k_normalization, sizeof(__pyx_k_normalization), 0, 0, 1, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_nsigma, __pyx_k_nsigma, sizeof(__pyx_k_nsigma), 0, 0, 1, 1},
  {&__pyx_n_s_num, __pyx_k_num, sizeof(__pyx_k_num), 0, 0, 1, 1},
  {&__pyx_n_s_num_converted, __pyx_k_num_converted, sizeof(__pyx_k_num_converted), 0, 0, 1, 1},
  {&__pyx_n_s_num_converted_old, __pyx_k_num_converted_old, sizeof(__pyx_k_num_converted_old), 0, 0, 1, 1},
  {&__pyx_n_s_num_pairs, __pyx_k_num_pairs, sizeof(__pyx_k_num_pairs), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_s_numpy__core_multiarray_failed_to, __pyx_k_numpy__core_multiarray_failed_to, sizeof(__pyx_k_numpy__core_multiarray_failed_to), 0, 0, 1, 0},
  {&__pyx_kp_s_numpy__core_umath_failed_to_impo, __pyx_k_numpy__core_umath_failed_to_impo, sizeof(__pyx_k_numpy__core_umath_failed_to_impo), 0, 0, 1, 0},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_n_s_ones, __pyx_k_ones, sizeof(__pyx_k_ones), 0, 0, 1, 1},
  {&__pyx_n_s_os, __pyx_k_os, sizeof(__pyx_k_os), 0, 0, 1, 1},
  {&__pyx_n_s_pbc, __pyx_k_pbc, sizeof(__pyx_k_pbc), 0, 0, 1, 1},
  {&__pyx_n_s_pi, __pyx_k_pi, sizeof(__pyx_k_pi), 0, 0, 1, 1},
  {&__pyx_n_s_pos, __pyx_k_pos, sizeof(__pyx_k_pos), 0, 0, 1, 1},
  {&__pyx_n_s_pos_i, __pyx_k_pos_i, sizeof(__pyx_k_pos_i), 0, 0, 1, 1},
  {&__pyx_n_s_pos_j, __pyx_k_pos_j, sizeof(__pyx_k_pos_j), 0, 0, 1, 1},
  {&__pyx_n_s_pos_k, __pyx_k_pos_k, sizeof(__pyx_k_pos_k), 0, 0, 1, 1},
  {&__pyx_n_s_pos_np, __pyx_k_pos_np, sizeof(__pyx_k_pos_np), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_product, __pyx_k_product, sizeof(__pyx_k_product), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reshape, __pyx_k_reshape, sizeof(__pyx_k_reshape), 0, 0, 1, 1},
  {&__pyx_n_s_scipy_spatial_distance, __pyx_k_scipy_spatial_distance, sizeof(__pyx_k_scipy_spatial_distance), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_sigma1, __pyx_k_sigma1, sizeof(__pyx_k_sigma1), 0, 0, 1, 1},
  {&__pyx_n_s_sigma2, __pyx_k_sigma2, sizeof(__pyx_k_sigma2), 0, 0, 1, 1},
  {&__pyx_n_s_smearing_norm1, __pyx_k_smearing_norm1, sizeof(__pyx_k_smearing_norm1), 0, 0, 1, 1},
  {&__pyx_n_s_smearing_norm2, __pyx_k_smearing_norm2, sizeof(__pyx_k_smearing_norm2), 0, 0, 1, 1},
  {&__pyx_n_s_sqrt, __pyx_k_sqrt, sizeof(__pyx_k_sqrt), 0, 0, 1, 1},
  {&__pyx_n_s_subarray, __pyx_k_subarray, sizeof(__pyx_k_subarray), 0, 0, 1, 1},
  {&__pyx_n_s_sum, __pyx_k_sum, sizeof(__pyx_k_sum), 0, 0, 1, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_t1, __pyx_k_t1, sizeof(__pyx_k_t1), 0, 0, 1, 1},
  {&__pyx_n_s_t2, __pyx_k_t2, sizeof(__pyx_k_t2), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_n_s_tolist, __pyx_k_tolist, sizeof(__pyx_k_tolist), 0, 0, 1, 1},
  {&__pyx_n_s_tt1, __pyx_k_tt1, sizeof(__pyx_k_tt1), 0, 0, 1, 1},
  {&__pyx_n_s_tt2, __pyx_k_tt2, sizeof(__pyx_k_tt2), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_n_s_type1, __pyx_k_type1, sizeof(__pyx_k_type1), 0, 0, 1, 1},
  {&__pyx_n_s_type2, __pyx_k_type2, sizeof(__pyx_k_type2), 0, 0, 1, 1},
  {&__pyx_n_s_type3, __pyx_k_type3, sizeof(__pyx_k_type3), 0, 0, 1, 1},
  {&__pyx_n_s_type_converter, __pyx_k_type_converter, sizeof(__pyx_k_type_converter), 0, 0, 1, 1},
  {&__pyx_n_s_use_angular, __pyx_k_use_angular, sizeof(__pyx_k_use_angular), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_value1, __pyx_k_value1, sizeof(__pyx_k_value1), 0, 0, 1, 1},
  {&__pyx_n_s_value2, __pyx_k_value2, sizeof(__pyx_k_value2), 0, 0, 1, 1},
  {&__pyx_n_s_volume, __pyx_k_volume, sizeof(__pyx_k_volume), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_xyz, __pyx_k_xyz, sizeof(__pyx_k_xyz), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {&__pyx_n_s_z, __pyx_k_z, sizeof(__pyx_k_z), 0, 0, 1, 1},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_n_s_object); if (!__pyx_builtin_object) __PYX_ERR(0, 94, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 88, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 135, __pyx_L1_error)
  __pyx_builtin_sum = __Pyx_GetBuiltinName(__pyx_n_s_sum); if (!__pyx_builtin_sum) __PYX_ERR(0, 152, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(1, 923, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":711
 * 
 *         feature_grad_np = np.zeros((Natoms*dim, Nelements))
 *         feature_grad_np[:, :Nelements_2body] = feature_grad1_np             # <<<<<<<<<<<<<<
 *         feature_grad_np[:, Nelements_2body:] = feature_grad2_np
 *         return feature_grad_np
 */
  __pyx_slice_ = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice_)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice_);
  __Pyx_GIVEREF(__pyx_slice_);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":923
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy._core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_numpy__core_multiarray_failed_to); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 923, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":929
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_numpy__core_umath_failed_to_impo); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 929, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":82
 *         return 0
 * 
 * def convert_atom_types(num):             # <<<<<<<<<<<<<<
 *     cdef int Natoms = len(num)
 *     cdef list atomic_types = sorted(list(set(num)))
 */
  __pyx_tuple__4 = PyTuple_Pack(7, __pyx_n_s_num, __pyx_n_s_Natoms, __pyx_n_s_atomic_types, __pyx_n_s_Ntypes, __pyx_n_s_num_converted, __pyx_n_s_i, __pyx_n_s_j); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(1, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_agox_models_descriptors_fingerpr, __pyx_n_s_convert_atom_types, 82, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(0, 82, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":94
 *     return num_converted
 * 
 * class Angular_Fingerprint(object):             # <<<<<<<<<<<<<<
 *     """ comparator for construction of angular fingerprints
 *     """
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":97
 *     """ comparator for construction of angular fingerprints
 *     """
 *     def __init__(self, atoms, Rc1=4.0, Rc2=4.0, binwidth1=0.1, Nbins2=30, sigma1=0.2, sigma2=0.10, nsigma=4, eta=1, gamma=3, use_angular=True):             # <<<<<<<<<<<<<<
 *         """ Set a common cut of radius
 *         """
 */
  __pyx_tuple__7 = PyTuple_Pack(29, __pyx_n_s_self, __pyx_n_s_atoms, __pyx_n_s_Rc1, __pyx_n_s_Rc2, __pyx_n_s_binwidth1, __pyx_n_s_Nbins2, __pyx_n_s_sigma1, __pyx_n_s_sigma2, __pyx_n_s_nsigma, __pyx_n_s_eta, __pyx_n_s_gamma, __pyx_n_s_use_angular, __pyx_n_s_type_converter, __pyx_n_s_i, __pyx_n_s_type, __pyx_n_s_count, __pyx_n_s_tt1, __pyx_n_s_tt2, __pyx_n_s_type1, __pyx_n_s_type2, __pyx_n_s_t1, __pyx_n_s_t2, __pyx_n_s_bondtypes_3body_keys, __pyx_n_s_type3, __pyx_n_s_key, __pyx_n_s_k1, __pyx_n_s_k2, __pyx_n_s_k3, __pyx_n_s_type); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);
  __pyx_codeobj__8 = (PyObject*)__Pyx_PyCode_New(12, 0, 29, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__7, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_agox_models_descriptors_fingerpr, __pyx_n_s_init, 97, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__8)) __PYX_ERR(0, 97, __pyx_L1_error)
  __pyx_tuple__9 = PyTuple_Pack(10, ((PyObject*)__pyx_float_4_0), ((PyObject*)__pyx_float_4_0), ((PyObject*)__pyx_float_0_1), ((PyObject *)__pyx_int_30), ((PyObject*)__pyx_float_0_2), ((PyObject*)__pyx_float_0_10), ((PyObject *)__pyx_int_4), ((PyObject *)__pyx_int_1), ((PyObject *)__pyx_int_3), ((PyObject *)Py_True)); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":197
 *         self.neighbourcells_disp = self.__get_neighbour_cells_displacement(self.pbc, self.cell)
 * 
 *     def get_feature(self, atoms):             # <<<<<<<<<<<<<<
 *         """
 *         """
 */
  __pyx_tuple__10 = PyTuple_Pack(82, __pyx_n_s_self, __pyx_n_s_atoms, __pyx_n_s_Rc1, __pyx_n_s_Rc2, __pyx_n_s_binwidth1, __pyx_n_s_binwidth2, __pyx_n_s_Nbins1, __pyx_n_s_Nbins2, __pyx_n_s_sigma1, __pyx_n_s_sigma2, __pyx_n_s_nsigma, __pyx_n_s_eta, __pyx_n_s_gamma, __pyx_n_s_use_angular, __pyx_n_s_volume, __pyx_n_s_dim, __pyx_n_s_m1, __pyx_n_s_m2, __pyx_n_s_smearing_norm1, __pyx_n_s_smearing_norm2, __pyx_n_s_Nelements_2body, __pyx_n_s_Nelements_3body, __pyx_n_s_Nelements, __pyx_n_s_mem, __pyx_n_s_cell, __pyx_n_s_Natoms, __pyx_n_s_pos_np, __pyx_n_s_pos, __pyx_n_s_m, __pyx_n_s_Ncells, __pyx_n_s_cell_displacements_old, __pyx_n_s_cell_displacements, __pyx_n_s_Ntypes, __pyx_n_s_Nbondtypes_2body, __pyx_n_s_bondtypes_2body_old, __pyx_n_s_bondtypes_2body, __pyx_n_s_num_converted_old, __pyx_n_s_num_converted, __pyx_n_s_feature1, __pyx_n_s_num_pairs, __pyx_n_s_center_bin, __pyx_n_s_minbin_lim, __pyx_n_s_maxbin_lim, __pyx_n_s_newbin, __pyx_n_s_bondtype_index, __pyx_n_s_type1, __pyx_n_s_type2, __pyx_n_s_Rij, __pyx_n_s_normalization, __pyx_n_s_binpos, __pyx_n_s_c, __pyx_n_s_erfarg_low, __pyx_n_s_erfarg_up, __pyx_n_s_value, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_n, __pyx_n_s_cell_index, __pyx_n_s_displacement, __pyx_n_s_feature1_np, __pyx_n_s_Nbondtypes_3body, __pyx_n_s_bondtypes_3body_old, __pyx_n_s_bondtypes_3body, __pyx_n_s_feature2, __pyx_n_s_RijVec, __pyx_n_s_RikVec, __pyx_n_s_angle, __pyx_n_s_k, __pyx_n_s_cond_ij, __pyx_n_s_cond_ik, __pyx_n_s_type3, __pyx_n_s_pos_i, __pyx_n_s_cell_index1, __pyx_n_s_displacement1, __pyx_n_s_pos_j, __pyx_n_s_cell_index2, __pyx_n_s_displacement2, __pyx_n_s_k_start, __pyx_n_s_pos_k, __pyx_n_s_Rik, __pyx_n_s_feature2_np, __pyx_n_s_feature_np); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);
  __pyx_codeobj__11 = (PyObject*)__Pyx_PyCode_New(2, 0, 82, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_agox_models_descriptors_fingerpr, __pyx_n_s_get_feature, 197, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__11)) __PYX_ERR(0, 197, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":433
 *         return feature_np
 * 
 *     def get_featureMat(self, atoms_list):             # <<<<<<<<<<<<<<
 *         featureMat = np.array([self.get_feature(atoms) for atoms in atoms_list])
 *         featureMat = np.array(featureMat)
 */
  __pyx_tuple__12 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_atoms_list, __pyx_n_s_featureMat, __pyx_n_s_atoms); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);
  __pyx_codeobj__13 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__12, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_agox_models_descriptors_fingerpr, __pyx_n_s_get_featureMat, 433, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__13)) __PYX_ERR(0, 433, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":438
 *         return featureMat
 * 
 *     def get_featureGradient(self, atoms):             # <<<<<<<<<<<<<<
 * 
 *         cdef double Rc1 = self.Rc1
 */
  __pyx_tuple__14 = PyTuple_Pack(95, __pyx_n_s_self, __pyx_n_s_atoms, __pyx_n_s_Rc1, __pyx_n_s_Rc2, __pyx_n_s_binwidth1, __pyx_n_s_binwidth2, __pyx_n_s_Nbins1, __pyx_n_s_Nbins2, __pyx_n_s_sigma1, __pyx_n_s_sigma2, __pyx_n_s_nsigma, __pyx_n_s_eta, __pyx_n_s_gamma, __pyx_n_s_use_angular, __pyx_n_s_volume, __pyx_n_s_dim, __pyx_n_s_m1, __pyx_n_s_m2, __pyx_n_s_smearing_norm1, __pyx_n_s_smearing_norm2, __pyx_n_s_Nelements_2body, __pyx_n_s_Nelements_3body, __pyx_n_s_Nelements, __pyx_n_s_mem, __pyx_n_s_cell, __pyx_n_s_Natoms, __pyx_n_s_pos_np, __pyx_n_s_pos, __pyx_n_s_m, __pyx_n_s_Ncells, __pyx_n_s_cell_displacements_old, __pyx_n_s_cell_displacements, __pyx_n_s_Ntypes, __pyx_n_s_Nbondtypes_2body, __pyx_n_s_bondtypes_2body_old, __pyx_n_s_bondtypes_2body, __pyx_n_s_num_converted_old, __pyx_n_s_num_converted, __pyx_n_s_feature_grad1, __pyx_n_s_RijVec, __pyx_n_s_num_pairs, __pyx_n_s_center_bin, __pyx_n_s_minbin_lim, __pyx_n_s_maxbin_lim, __pyx_n_s_newbin, __pyx_n_s_bondtype_index, __pyx_n_s_type1, __pyx_n_s_type2, __pyx_n_s_Rij, __pyx_n_s_normalization, __pyx_n_s_binpos, __pyx_n_s_c, __pyx_n_s_arg_low, __pyx_n_s_arg_up, __pyx_n_s_value1, __pyx_n_s_value2, __pyx_n_s_value, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_n, __pyx_n_s_pos_i, __pyx_n_s_cell_index, __pyx_n_s_displacement, __pyx_n_s_pos_j, __pyx_n_s_grad_index, __pyx_n_s_feature_grad1_np, __pyx_n_s_Nbondtypes_3body, __pyx_n_s_bondtypes_3body_old, __pyx_n_s_bondtypes_3body, __pyx_n_s_feature_grad2, __pyx_n_s_RikVec, __pyx_n_s_angle_grad_i, __pyx_n_s_angle_grad_j, __pyx_n_s_angle_grad_k, __pyx_n_s_angle, __pyx_n_s_cos_angle, __pyx_n_s_a, __pyx_n_s_k, __pyx_n_s_cond_ij, __pyx_n_s_cond_ik, __pyx_n_s_bin_index, __pyx_n_s_type3, __pyx_n_s_cell_index1, __pyx_n_s_displacement1, __pyx_n_s_cell_index2, __pyx_n_s_displacement2, __pyx_n_s_k_start, __pyx_n_s_pos_k, __pyx_n_s_Rik, __pyx_n_s_fc_ij, __pyx_n_s_fc_ik, __pyx_n_s_fc_grad_ij, __pyx_n_s_fc_grad_ik, __pyx_n_s_feature_grad2_np, __pyx_n_s_feature_grad_np); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);
  __pyx_codeobj__15 = (PyObject*)__Pyx_PyCode_New(2, 0, 95, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__14, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_agox_models_descriptors_fingerpr, __pyx_n_s_get_featureGradient, 438, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__15)) __PYX_ERR(0, 438, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":715
 *         return feature_grad_np
 * 
 *     def get_all_featureGradients(self, atoms_list):             # <<<<<<<<<<<<<<
 *         feature_grads = np.array([self.get_featureGradient(atoms) for atoms in atoms_list])
 *         feature_grads = np.array(feature_grads)
 */
  __pyx_tuple__16 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_atoms_list, __pyx_n_s_feature_grads, __pyx_n_s_atoms); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 715, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);
  __pyx_codeobj__17 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__16, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_agox_models_descriptors_fingerpr, __pyx_n_s_get_all_featureGradients, 715, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__17)) __PYX_ERR(0, 715, __pyx_L1_error)

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":720
 *         return feature_grads
 * 
 *     def __get_neighbour_cells_displacement(self, pbc, cell):             # <<<<<<<<<<<<<<
 *         # Calculate neighbour cells
 *         Rc_max = max(self.Rc1+self.sigma1*self.nsigma, self.Rc2)  # relevant cutoff
 */
  __pyx_tuple__18 = PyTuple_Pack(14, __pyx_n_s_self, __pyx_n_s_pbc, __pyx_n_s_cell, __pyx_n_s_Rc_max, __pyx_n_s_cell_vec_norms, __pyx_n_s_neighbours, __pyx_n_s_i, __pyx_n_s_ncellmax, __pyx_n_s_neighbourcells_disp, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_z, __pyx_n_s_xyz, __pyx_n_s_displacement); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 720, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);
  __pyx_codeobj__19 = (PyObject*)__Pyx_PyCode_New(3, 0, 14, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__18, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_agox_models_descriptors_fingerpr, __pyx_n_s_get_neighbour_cells_displaceme, 720, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__19)) __PYX_ERR(0, 720, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_1 = PyFloat_FromDouble(0.1); if (unlikely(!__pyx_float_0_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_2 = PyFloat_FromDouble(0.2); if (unlikely(!__pyx_float_0_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_0 = PyFloat_FromDouble(4.0); if (unlikely(!__pyx_float_4_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_10 = PyFloat_FromDouble(0.10); if (unlikely(!__pyx_float_0_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_30 = PyInt_FromLong(30); if (unlikely(!__pyx_int_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType_0_29_36(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyArray_Descr),__Pyx_ImportType_CheckSize_Ignore_0_29_36); if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(1, 237, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyArrayIterObject),__Pyx_ImportType_CheckSize_Ignore_0_29_36); if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(1, 256, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyArrayMultiIterObject),__Pyx_ImportType_CheckSize_Ignore_0_29_36); if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(1, 260, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyArrayObject),__Pyx_ImportType_CheckSize_Ignore_0_29_36); if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(1, 272, __pyx_L1_error)
  __pyx_ptype_5numpy_generic = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "generic", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyObject),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5numpy_generic) __PYX_ERR(1, 739, __pyx_L1_error)
  __pyx_ptype_5numpy_number = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "number", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyObject),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5numpy_number) __PYX_ERR(1, 741, __pyx_L1_error)
  __pyx_ptype_5numpy_integer = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "integer", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyObject),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5numpy_integer) __PYX_ERR(1, 743, __pyx_L1_error)
  __pyx_ptype_5numpy_signedinteger = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "signedinteger", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyObject),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5numpy_signedinteger) __PYX_ERR(1, 745, __pyx_L1_error)
  __pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "unsignedinteger", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyObject),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(1, 747, __pyx_L1_error)
  __pyx_ptype_5numpy_inexact = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "inexact", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyObject),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5numpy_inexact) __PYX_ERR(1, 749, __pyx_L1_error)
  __pyx_ptype_5numpy_floating = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "floating", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyObject),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5numpy_floating) __PYX_ERR(1, 751, __pyx_L1_error)
  __pyx_ptype_5numpy_complexfloating = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "complexfloating", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyObject),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5numpy_complexfloating) __PYX_ERR(1, 753, __pyx_L1_error)
  __pyx_ptype_5numpy_flexible = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "flexible", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyObject),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5numpy_flexible) __PYX_ERR(1, 755, __pyx_L1_error)
  __pyx_ptype_5numpy_character = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "character", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyObject),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5numpy_character) __PYX_ERR(1, 757, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType_0_29_36(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(PyUFuncObject),__Pyx_ImportType_CheckSize_Ignore_0_29_36); if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(1, 821, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("cymem.cymem"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5cymem_5cymem_PyMalloc = __Pyx_ImportType_0_29_36(__pyx_t_1, "cymem.cymem", "PyMalloc", sizeof(struct __pyx_obj_5cymem_5cymem_PyMalloc), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(struct __pyx_obj_5cymem_5cymem_PyMalloc),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5cymem_5cymem_PyMalloc) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_vtabptr_5cymem_5cymem_PyMalloc = (struct __pyx_vtabstruct_5cymem_5cymem_PyMalloc*)__Pyx_GetVtable(__pyx_ptype_5cymem_5cymem_PyMalloc->tp_dict); if (unlikely(!__pyx_vtabptr_5cymem_5cymem_PyMalloc)) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_ptype_5cymem_5cymem_PyFree = __Pyx_ImportType_0_29_36(__pyx_t_1, "cymem.cymem", "PyFree", sizeof(struct __pyx_obj_5cymem_5cymem_PyFree), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(struct __pyx_obj_5cymem_5cymem_PyFree),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5cymem_5cymem_PyFree) __PYX_ERR(3, 10, __pyx_L1_error)
  __pyx_vtabptr_5cymem_5cymem_PyFree = (struct __pyx_vtabstruct_5cymem_5cymem_PyFree*)__Pyx_GetVtable(__pyx_ptype_5cymem_5cymem_PyFree->tp_dict); if (unlikely(!__pyx_vtabptr_5cymem_5cymem_PyFree)) __PYX_ERR(3, 10, __pyx_L1_error)
  __pyx_ptype_5cymem_5cymem_Pool = __Pyx_ImportType_0_29_36(__pyx_t_1, "cymem.cymem", "Pool", sizeof(struct __pyx_obj_5cymem_5cymem_Pool), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(struct __pyx_obj_5cymem_5cymem_Pool),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5cymem_5cymem_Pool) __PYX_ERR(3, 16, __pyx_L1_error)
  __pyx_vtabptr_5cymem_5cymem_Pool = (struct __pyx_vtabstruct_5cymem_5cymem_Pool*)__Pyx_GetVtable(__pyx_ptype_5cymem_5cymem_Pool->tp_dict); if (unlikely(!__pyx_vtabptr_5cymem_5cymem_Pool)) __PYX_ERR(3, 16, __pyx_L1_error)
  __pyx_ptype_5cymem_5cymem_Address = __Pyx_ImportType_0_29_36(__pyx_t_1, "cymem.cymem", "Address", sizeof(struct __pyx_obj_5cymem_5cymem_Address), __PYX_GET_STRUCT_ALIGNMENT_0_29_36(struct __pyx_obj_5cymem_5cymem_Address),__Pyx_ImportType_CheckSize_Warn_0_29_36); if (!__pyx_ptype_5cymem_5cymem_Address) __PYX_ERR(3, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initangular_fingerprintFeature_cy(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initangular_fingerprintFeature_cy(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_angular_fingerprintFeature_cy(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_angular_fingerprintFeature_cy(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_angular_fingerprintFeature_cy(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'angular_fingerprintFeature_cy' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_angular_fingerprintFeature_cy(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("angular_fingerprintFeature_cy", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_agox__models__descriptors__fingerprint_cython__angular_fingerprintFeature_cy) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy")) {
      if (unlikely(PyDict_SetItemString(modules, "agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  (void)__Pyx_modinit_type_init_code();
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":1
 * import os             # <<<<<<<<<<<<<<
 * import sys
 * from math import erf
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_os, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_os, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":2
 * import os
 * import sys             # <<<<<<<<<<<<<<
 * from math import erf
 * from itertools import product
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_sys, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sys, __pyx_t_1) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":3
 * import os
 * import sys
 * from math import erf             # <<<<<<<<<<<<<<
 * from itertools import product
 * from scipy.spatial.distance import cdist
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_erf);
  __Pyx_GIVEREF(__pyx_n_s_erf);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_erf);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_math, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_erf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_erf, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":4
 * import sys
 * from math import erf
 * from itertools import product             # <<<<<<<<<<<<<<
 * from scipy.spatial.distance import cdist
 * 
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_product);
  __Pyx_GIVEREF(__pyx_n_s_product);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_product);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_itertools, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_product); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_product, __pyx_t_2) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":5
 * from math import erf
 * from itertools import product
 * from scipy.spatial.distance import cdist             # <<<<<<<<<<<<<<
 * 
 * import time
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_cdist);
  __Pyx_GIVEREF(__pyx_n_s_cdist);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_cdist);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_scipy_spatial_distance, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_cdist); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cdist, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":7
 * from scipy.spatial.distance import cdist
 * 
 * import time             # <<<<<<<<<<<<<<
 * 
 * import numpy as np
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_time, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_time, __pyx_t_2) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":9
 * import time
 * 
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * 
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_2) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":17
 * cimport cython
 * 
 * try:             # <<<<<<<<<<<<<<
 *     cwd = sys.argv[1]
 * except:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":18
 * 
 * try:
 *     cwd = sys.argv[1]             # <<<<<<<<<<<<<<
 * except:
 *     cwd = os.getcwd()
 */
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_sys); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_argv); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (PyDict_SetItem(__pyx_d, __pyx_n_s_cwd, __pyx_t_2) < 0) __PYX_ERR(0, 18, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":17
 * cimport cython
 * 
 * try:             # <<<<<<<<<<<<<<
 *     cwd = sys.argv[1]
 * except:
 */
    }
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L7_try_end;
    __pyx_L2_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":19
 * try:
 *     cwd = sys.argv[1]
 * except:             # <<<<<<<<<<<<<<
 *     cwd = os.getcwd()
 * 
 */
    /*except:*/ {
      __Pyx_AddTraceback("agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_1, &__pyx_t_6) < 0) __PYX_ERR(0, 19, __pyx_L4_except_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_6);

      /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":20
 *     cwd = sys.argv[1]
 * except:
 *     cwd = os.getcwd()             # <<<<<<<<<<<<<<
 * 
 * # Custom functions
 */
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_os); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 20, __pyx_L4_except_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_getcwd); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 20, __pyx_L4_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 20, __pyx_L4_except_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (PyDict_SetItem(__pyx_d, __pyx_n_s_cwd, __pyx_t_7) < 0) __PYX_ERR(0, 20, __pyx_L4_except_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L3_exception_handled;
    }
    __pyx_L4_except_error:;

    /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":17
 * cimport cython
 * 
 * try:             # <<<<<<<<<<<<<<
 *     cwd = sys.argv[1]
 * except:
 */
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L3_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    __pyx_L7_try_end:;
  }

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":82
 *         return 0
 * 
 * def convert_atom_types(num):             # <<<<<<<<<<<<<<
 *     cdef int Natoms = len(num)
 *     cdef list atomic_types = sorted(list(set(num)))
 */
  __pyx_t_6 = PyCFunction_NewEx(&__pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_1convert_atom_types, NULL, __pyx_n_s_agox_models_descriptors_fingerpr_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_convert_atom_types, __pyx_t_6) < 0) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":94
 *     return num_converted
 * 
 * class Angular_Fingerprint(object):             # <<<<<<<<<<<<<<
 *     """ comparator for construction of angular fingerprints
 *     """
 */
  __pyx_t_6 = __Pyx_CalculateMetaclass(NULL, __pyx_tuple__6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = __Pyx_Py3MetaclassPrepare(__pyx_t_6, __pyx_tuple__6, __pyx_n_s_Angular_Fingerprint, __pyx_n_s_Angular_Fingerprint, (PyObject *) NULL, __pyx_n_s_agox_models_descriptors_fingerpr_2, __pyx_kp_s_comparator_for_construction_of); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":97
 *     """ comparator for construction of angular fingerprints
 *     """
 *     def __init__(self, atoms, Rc1=4.0, Rc2=4.0, binwidth1=0.1, Nbins2=30, sigma1=0.2, sigma2=0.10, nsigma=4, eta=1, gamma=3, use_angular=True):             # <<<<<<<<<<<<<<
 *         """ Set a common cut of radius
 *         """
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_1__init__, 0, __pyx_n_s_Angular_Fingerprint___init, NULL, __pyx_n_s_agox_models_descriptors_fingerpr_2, __pyx_d, ((PyObject *)__pyx_codeobj__8)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_tuple__9);
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_init, __pyx_t_2) < 0) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":197
 *         self.neighbourcells_disp = self.__get_neighbour_cells_displacement(self.pbc, self.cell)
 * 
 *     def get_feature(self, atoms):             # <<<<<<<<<<<<<<
 *         """
 *         """
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_3get_feature, 0, __pyx_n_s_Angular_Fingerprint_get_feature, NULL, __pyx_n_s_agox_models_descriptors_fingerpr_2, __pyx_d, ((PyObject *)__pyx_codeobj__11)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_get_feature, __pyx_t_2) < 0) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":433
 *         return feature_np
 * 
 *     def get_featureMat(self, atoms_list):             # <<<<<<<<<<<<<<
 *         featureMat = np.array([self.get_feature(atoms) for atoms in atoms_list])
 *         featureMat = np.array(featureMat)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_5get_featureMat, 0, __pyx_n_s_Angular_Fingerprint_get_featureM, NULL, __pyx_n_s_agox_models_descriptors_fingerpr_2, __pyx_d, ((PyObject *)__pyx_codeobj__13)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_get_featureMat, __pyx_t_2) < 0) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":438
 *         return featureMat
 * 
 *     def get_featureGradient(self, atoms):             # <<<<<<<<<<<<<<
 * 
 *         cdef double Rc1 = self.Rc1
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_7get_featureGradient, 0, __pyx_n_s_Angular_Fingerprint_get_featureG, NULL, __pyx_n_s_agox_models_descriptors_fingerpr_2, __pyx_d, ((PyObject *)__pyx_codeobj__15)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_get_featureGradient, __pyx_t_2) < 0) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":715
 *         return feature_grad_np
 * 
 *     def get_all_featureGradients(self, atoms_list):             # <<<<<<<<<<<<<<
 *         feature_grads = np.array([self.get_featureGradient(atoms) for atoms in atoms_list])
 *         feature_grads = np.array(feature_grads)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_9get_all_featureGradients, 0, __pyx_n_s_Angular_Fingerprint_get_all_feat, NULL, __pyx_n_s_agox_models_descriptors_fingerpr_2, __pyx_d, ((PyObject *)__pyx_codeobj__17)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 715, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_get_all_featureGradients, __pyx_t_2) < 0) __PYX_ERR(0, 715, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":720
 *         return feature_grads
 * 
 *     def __get_neighbour_cells_displacement(self, pbc, cell):             # <<<<<<<<<<<<<<
 *         # Calculate neighbour cells
 *         Rc_max = max(self.Rc1+self.sigma1*self.nsigma, self.Rc2)  # relevant cutoff
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4agox_6models_11descriptors_18fingerprint_cython_29angular_fingerprintFeature_cy_19Angular_Fingerprint_11__get_neighbour_cells_displacement, 0, __pyx_n_s_Angular_Fingerprint___get_neighb, NULL, __pyx_n_s_agox_models_descriptors_fingerpr_2, __pyx_d, ((PyObject *)__pyx_codeobj__19)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 720, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_Angular_Fingerprint__get_neighb, __pyx_t_2) < 0) __PYX_ERR(0, 720, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":94
 *     return num_converted
 * 
 * class Angular_Fingerprint(object):             # <<<<<<<<<<<<<<
 *     """ comparator for construction of angular fingerprints
 *     """
 */
  __pyx_t_2 = __Pyx_Py3ClassCreate(__pyx_t_6, __pyx_n_s_Angular_Fingerprint, __pyx_tuple__6, __pyx_t_1, NULL, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Angular_Fingerprint, __pyx_t_2) < 0) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "agox/models/descriptors/fingerprint_cython/angular_fingerprintFeature_cy.pyx":1
 * import os             # <<<<<<<<<<<<<<
 * import sys
 * from math import erf
 */
  __pyx_t_6 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_6) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "../../../../.cache/uv/builds-v0/.tmppjLvcS/lib/python3.11/site-packages/numpy/__init__.pxd":985
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init agox.models.descriptors.fingerprint_cython.angular_fingerprintFeature_cy");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
    else state = (PyGILState_STATE)0;
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#if defined(__Pyx_CyFunction_USED) && defined(NDEBUG)
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* SliceObject */
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(PyObject* obj, PyObject* value,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_ass_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_ass_slice(obj, cstart, cstop, value);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_ass_subscript))
#endif
    {
        int result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_ass_subscript(obj, py_slice, value);
#else
        result = value ? PyObject_SetItem(obj, py_slice, value) : PyObject_DelItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object does not support slice %.10s",
        Py_TYPE(obj)->tp_name, value ? "assignment" : "deletion");
bad:
    return -1;
}

/* py_abs */
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *n) {
    if (likely(Py_SIZE(n) == -1)) {
        return PyLong_FromLong(((PyLongObject*)n)->ob_digit[0]);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    {
        PyObject *copy = _PyLong_Copy((PyLongObject*)n);
        if (likely(copy)) {
            __Pyx_SET_SIZE(copy, -Py_SIZE(copy));
        }
        return copy;
    }
#else
    return PyNumber_Negative(n);
#endif
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_0_29_36
#define __PYX_HAVE_RT_ImportType_0_29_36
static PyTypeObject *__Pyx_ImportType_0_29_36(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_0_29_36 check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_0_29_36 && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_0_29_36 && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CalculateMetaclass */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
#include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
#if PY_MAJOR_VERSION > 2
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
#else
            PyErr_SetString(PyExc_TypeError,
                            "unbound method needs an argument");
#endif
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double x, long double y) {
      return ::std::complex< long double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double x, long double y) {
      return x + y*(__pyx_t_long_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double x, long double y) {
      __pyx_t_long_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_sum_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        __pyx_t_long_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_diff_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        __pyx_t_long_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_prod_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        __pyx_t_long_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_quot_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_long_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsl(b.real) >= fabsl(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_long_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                long double r = b.imag / b.real;
                long double s = (long double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_long_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            long double r = b.real / b.imag;
            long double s = (long double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_long_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_quot_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_long_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            long double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_long_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_neg_long__double(__pyx_t_long_double_complex a) {
        __pyx_t_long_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_long__double(__pyx_t_long_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_conj_long__double(__pyx_t_long_double_complex a) {
        __pyx_t_long_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE long double __Pyx_c_abs_long__double(__pyx_t_long_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtl(z.real*z.real + z.imag*z.imag);
          #else
            return hypotl(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_pow_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
            __pyx_t_long_double_complex z;
            long double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    long double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_long__double(a, a);
                    case 3:
                        z = __Pyx_c_prod_long__double(a, a);
                        return __Pyx_c_prod_long__double(z, a);
                    case 4:
                        z = __Pyx_c_prod_long__double(a, a);
                        return __Pyx_c_prod_long__double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = powl(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2l(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_long__double(a);
                theta = atan2l(a.imag, a.real);
            }
            lnr = logl(r);
            z_r = expl(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosl(z_theta);
            z.imag = z_r * sinl(z_theta);
            return z;
        }
    #endif
#endif

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[5];
    int same=1, i, found_dot;
    const char* rt_from_call = Py_GetVersion();
    PyOS_snprintf(ctversion, 5, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    found_dot = 0;
    for (i = 0; i < 4; i++) {
        if (!ctversion[i]) {
            same = (rt_from_call[i] < '0' || rt_from_call[i] > '9');
            break;
        }
        if (rt_from_call[i] != ctversion[i]) {
            same = 0;
            break;
        }
    }
    if (!same) {
        char rtversion[5] = {'\0'};
        char message[200];
        for (i=0; i<4; ++i) {
            if (rt_from_call[i] == '.') {
                if (found_dot) break;
                found_dot = 1;
            } else if (rt_from_call[i] < '0' || rt_from_call[i] > '9') {
                break;
            }
            rtversion[i] = rt_from_call[i];
        }
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
